@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -10,6 +10,7 @@ import re
 
 from xpra.client.client_widget_base import ClientWidgetBase
 from xpra.os_util import bytestostr, OSX, WIN32, is_Wayland
+from xpra.common import GRAVITY_STR
 from xpra.util import typedict, envbool, envint, WORKSPACE_UNSET, WORKSPACE_NAMES
 from xpra.log import Logger
 
@@ -27,7 +28,6 @@ alphalog = Logger("alpha")
 
 SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
-AWT_DIALOG_WORKAROUND = envbool("XPRA_AWT_DIALOG_WORKAROUND", WIN32)
 SET_SIZE_CONSTRAINTS = envbool("XPRA_SET_SIZE_CONSTRAINTS", True)
 DEFAULT_GRAVITY = envint("XPRA_DEFAULT_GRAVITY", 0)
 OVERRIDE_GRAVITY = envint("XPRA_OVERRIDE_GRAVITY", 0)
@@ -35,18 +35,17 @@ OVERRIDE_GRAVITY = envint("XPRA_OVERRIDE_GRAVITY", 0)
 
 class ClientWindowBase(ClientWidgetBase):
 
-    #(overriden in subclasses)
-    NAME_TO_HINT = {}
-
     def __init__(self, client, group_leader, watcher_pid, wid,
                  wx, wy, ww, wh, bw, bh,
                  metadata, override_redirect, client_properties,
-                 border, max_window_size, default_cursor_data, pixel_depth):
+                 border, max_window_size, default_cursor_data, pixel_depth,
+                 headerbar="no"):
         log("%s%s", type(self),
             (client, group_leader, watcher_pid, wid,
              wx, wy, ww, wh, bw, bh,
              metadata, override_redirect, client_properties,
-             border, max_window_size, default_cursor_data, pixel_depth))
+             border, max_window_size, default_cursor_data, pixel_depth,
+             headerbar))
         super().__init__(client, watcher_pid, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
@@ -65,7 +64,6 @@ class ClientWindowBase(ClientWidgetBase):
         self._sticky = False
         self._skip_pager = False
         self._skip_taskbar = False
-        self._sticky = False
         self._iconified = False
         self._focused = False
         self.window_gravity = OVERRIDE_GRAVITY or DEFAULT_GRAVITY
@@ -75,8 +73,12 @@ class ClientWindowBase(ClientWidgetBase):
         self.max_window_size = max_window_size
         self.button_state = {}
         self.pixel_depth = pixel_depth      #0 for default
-        self.window_offset = None           #actual vs reported coordinates
+        #window_offset is the delta between the location of the window requested by the server,
+        #and where we actually end up mapping it on the client
+        #(ie: when we reposition an OR window to ensure it is visible on screen)
+        self.window_offset = None
         self.pending_refresh = []
+        self.headerbar = headerbar
 
         self.init_window(metadata)
         self.setup_window(bw, bh)
@@ -111,6 +113,49 @@ class ClientWindowBase(ClientWidgetBase):
         self.set_decorated(metadata.boolget("decorations", True))
 
 
+    def get_info(self):
+        attributes = []
+        if self._fullscreen:
+            attributes.append("fullscreen")
+        if self._maximized:
+            attributes.append("maximized")
+        if self._above:
+            attributes.append("above")
+        if self._below:
+            attributes.append("below")
+        if self._shaded:
+            attributes.append("shaded")
+        if self._sticky:
+            attributes.append("sticky")
+        if self._skip_pager:
+            attributes.append("skip-pager")
+        if self._skip_taskbar:
+            attributes.append("skip-taskbar")
+        if self._iconified:
+            attributes.append("iconified")
+        if self._focused:
+            attributes.append("focused")
+        info = super().get_info()
+        info.update({
+            "override-redirect"     : self._override_redirect,
+            #"group-leader"          : self.group_leader,
+            "position"              : self._pos,
+            "size"                  : self._size,
+            "client-properties"     : self._client_properties,
+            "set-initial-position"  : self._set_initial_position,
+            "size-constraints"      : dict(self.size_constraints),
+            "geometry-hints"        : dict(self.geometry_hints),
+            "content-type"          : self.content_type,
+            "attributes"            : attributes,
+            "gravity"               : GRAVITY_STR.get(self.window_gravity),
+            #"border"                : self.border or "",
+            #cursor_data
+            "max-size"              : self.max_window_size,
+            "button-state"          : self.button_state,
+            "offset"                : self.window_offset,
+            })
+        return info
+
     def get_desktop_workspace(self):
         return None
 
@@ -210,6 +255,70 @@ class ClientWindowBase(ClientWidgetBase):
                 }
             }
 
+    def _get_window_title(self, metadata):
+        try:
+            title = bytestostr(self._client.title).replace("\0", "")
+            if title.find("@")<0:
+                return title
+            #perform metadata variable substitutions:
+            #full of py3k unicode headaches that don't need to be
+            default_values = {
+                "title"           : "<untitled window>",
+                "client-machine"  : "<unknown machine>",
+                "windowid"        : str(self._id),
+                "server-machine"  : getattr(self._client, "_remote_hostname", None) or "<unknown machine>",
+                "server-display"  : getattr(self._client, "_remote_display", None) or "<unknown display>",
+                }
+            def getvar(var):
+                #"hostname" is magic:
+                #we try harder to find a useful value to show:
+                if var in ("hostname", "hostinfo"):
+                    if var=="hostinfo" and getattr(self._client, "mmap_enabled", False):
+                        #this is a local connection for sure
+                        server_display = getattr(self._client, "server_display", None)
+                        if server_display:
+                            return server_display
+                    #try to find the hostname:
+                    proto = getattr(self._client, "_protocol", None)
+                    if proto:
+                        conn = getattr(proto, "_conn", None)
+                        if conn:
+                            hostname = conn.info.get("host") or bytestostr(conn.target)
+                            if hostname:
+                                return hostname
+                    for m in ("client-machine", "server-machine"):
+                        value = getvar(m)
+                        if value not in (
+                            "localhost",
+                            "localhost.localdomain",
+                            "<unknown machine>",
+                            "",
+                            None):
+                            return value
+                    return "<unknown machine>"
+                value = metadata.bytesget(var) or self._metadata.bytesget(var)
+                if value is None:
+                    return default_values.get(var, "<unknown %s>" % var)
+                try:
+                    return value.decode("utf-8")
+                except UnicodeDecodeError:
+                    return str(value)
+            def metadata_replace(match):
+                atvar = match.group(0)          #ie: '@title@'
+                var = atvar[1:len(atvar)-1]     #ie: 'title'
+                if not var:
+                    #atvar = "@@"
+                    return "@"
+                return getvar(var)
+            sub = r"@[\w\-]*@"
+            replaced = re.sub(sub, metadata_replace, title)
+            metalog("re.sub%s=%s", (sub, metadata_replace, title), replaced)
+            return replaced
+        except Exception as e:
+            log.error("Error parsing window title:")
+            log.error(" %s", e)
+            return ""
+
     def set_metadata(self, metadata):
         metalog("set_metadata(%s)", metadata)
         debug_props = [x for x in PROPERTIES_DEBUG if x in metadata.keys()]
@@ -222,59 +331,7 @@ class ClientWindowBase(ClientWidgetBase):
             self.reset_icon()
 
         if b"title" in metadata:
-            try:
-                title = bytestostr(self._client.title).replace("\0", "")
-                if title.find("@")>=0:
-                    #perform metadata variable substitutions:
-                    #full of py3k unicode headaches that don't need to be
-                    default_values = {
-                        "title"           : "<untitled window>",
-                        "client-machine"  : "<unknown machine>",
-                        "windowid"        : str(self._id),
-                        "server-machine"  : getattr(self._client, "_remote_hostname", None) or "<unknown machine>",
-                        "server-display"  : getattr(self._client, "_remote_display", None) or "<unknown display>",
-                        }
-                    def getvar(var):
-                        #"hostname" is magic:
-                        #we try harder to find a useful value to show:
-                        if var=="hostname":
-                            #try to find the hostname:
-                            proto = getattr(self._client, "_protocol", None)
-                            if proto:
-                                conn = getattr(proto, "_conn", None)
-                                if conn:
-                                    hostname = conn.info.get("host") or bytestostr(conn.target)
-                                    if hostname:
-                                        return hostname
-                            for m in ("client-machine", "server-machine"):
-                                value = getvar(m)
-                                if value not in (
-                                    "localhost",
-                                    "localhost.localdomain",
-                                    "<unknown machine>",
-                                    "",
-                                    None):
-                                    return value
-                            return "<unknown machine>"
-                        value = self._metadata.bytesget(var)
-                        if value is None:
-                            return default_values.get(var, "<unknown %s>" % var)
-                        try:
-                            return value.decode("utf-8")
-                        except UnicodeDecodeError:
-                            return str(value)
-                    def metadata_replace(match):
-                        atvar = match.group(0)          #ie: '@title@'
-                        var = atvar[1:len(atvar)-1]     #ie: 'title'
-                        if not var:
-                            #atvar = "@@"
-                            return "@"
-                        return getvar(var)
-                    title = re.sub(r"@[\w\-]*@", metadata_replace, title)
-            except Exception as e:
-                log.error("Error parsing window title:")
-                log.error(" %s", e)
-                title = ""
+            title = self._get_window_title(metadata)
             self.set_title(title)
 
         if b"icon-title" in metadata:
@@ -303,7 +360,7 @@ class ClientWindowBase(ClientWidgetBase):
             self.set_modal(modal)
 
         #apply window-type hint if window has not been mapped yet:
-        if b"window-type" in metadata and not self.is_mapped():
+        if b"window-type" in metadata and not self.get_mapped():
             window_types = metadata.strtupleget("window-type")
             self.set_window_type(window_types)
 
@@ -333,7 +390,7 @@ class ClientWindowBase(ClientWidgetBase):
                     #win32 without opengl can't do transparency,
                     #so it triggers too many warnings
                     l = log.warn
-                l("Warning: window %s changed its transparency attribute", self._id)
+                l("Warning: window %#x changed its transparency attribute", self._id)
                 l(" from %s to %s, behaviour is undefined", self._has_alpha, new_alpha)
                 self._has_alpha = new_alpha
 
@@ -539,10 +596,7 @@ class ClientWindowBase(ClientWidgetBase):
             #and that keys are always "bytes":
             #(in practice this code should never fire, just here as a reminder)
             for x in ("max_width", "max_height"):
-                try:
-                    del hints[x]
-                except KeyError:
-                    pass
+                hints.pop(x, None)
             #bug 2214: GTK3 on win32 gets confused if we specify a large max-size
             # and it will mess up maximizing the window
             if not WIN32 or (maxw<32000 or maxh<32000):
@@ -568,24 +622,7 @@ class ClientWindowBase(ClientWidgetBase):
 
 
     def set_window_type(self, window_types):
-        hints = 0
-        for window_type in window_types:
-            #win32 workaround:
-            if AWT_DIALOG_WORKAROUND and window_type=="DIALOG" and self._metadata.boolget("skip-taskbar"):
-                wm_class = self._metadata.strtupleget("class-instance", (None, None), 2, 2)
-                if wm_class and len(wm_class)==2 and wm_class[0] and wm_class[0].startswith("sun-awt-X11"):
-                    #replace "DIALOG" with "NORMAL":
-                    if "NORMAL" in window_types:
-                        continue
-                    window_type = "NORMAL"
-            hint = self.NAME_TO_HINT.get(window_type, None)
-            if hint is not None:
-                hints |= hint
-            else:
-                log("ignoring unknown window type hint: %s", window_type)
-        log("set_window_type(%s) hints=%s", window_types, hints)
-        if hints:
-            self.set_type_hint(hints)
+        pass
 
     def set_workspace(self, workspace):
         pass
@@ -668,7 +705,11 @@ class ClientWindowBase(ClientWidgetBase):
         if b:
             b.toggle()
             log("magic_key%s border=%s", args, b)
-            self.queue_draw_area(0, 0, *self._size)
+            self.repaint(0, 0, *self._size)
+
+    def repaint(self, x, y, w, h):
+        #self.queue_draw_area(0, 0, *self._size)
+        raise NotImplementedError("no repaint on %s", type(self))
 
     def refresh_window(self, *args):
         log("refresh_window(%s) wid=%s", args, self._id)
@@ -694,6 +735,9 @@ class ClientWindowBase(ClientWidgetBase):
                 self.pending_refresh.append((x, y, width, height))
             if options.intget("flush", 0)==0:
                 callbacks.append(self.after_draw_refresh)
+        if coding=="void":
+            fire_paint_callbacks(callbacks)
+            return
         backing.draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
 
     def after_draw_refresh(self, success, message=""):
@@ -705,16 +749,16 @@ class ClientWindowBase(ClientWidgetBase):
         if backing.repaint_all or self._client.xscale!=1 or self._client.yscale!=1 or is_Wayland():
             #easy: just repaint the whole window:
             rw, rh = self.get_size()
-            self.idle_add(self.queue_draw_area, 0, 0, rw, rh)
+            self.idle_add(self.repaint, 0, 0, rw, rh)
             return
         pr = self.pending_refresh
         self.pending_refresh = []
         for x, y, w, h in pr:
             rx, ry, rw, rh = self._client.srect(x, y, w, h)
-            #if self.window_offset:
-            #    rx -= self.window_offset[0]
-            #    ry -= self.window_offset[1]
-            self.idle_add(self.queue_draw_area, rx, ry, rw, rh)
+            if self.window_offset:
+                rx += self.window_offset[0]
+                ry += self.window_offset[1]
+            self.idle_add(self.repaint, rx, ry, rw, rh)
 
     def eos(self):
         """ Note: this runs from the draw thread (not UI thread) """
@@ -729,7 +773,7 @@ class ClientWindowBase(ClientWidgetBase):
         #with normal windows, we just queue a draw request
         #and let the expose event paint the spinner
         w, h = self.get_size()
-        self.queue_draw_area(0, 0, w, h)
+        self.repaint(0, 0, w, h)
 
     def can_have_spinner(self):
         if self._backing is None:
@@ -753,6 +797,11 @@ class ClientWindowBase(ClientWidgetBase):
     def void(self):
         pass
 
+    def show_window_info(self, *args):
+        from xpra.client.gtk_base.window_info import WindowInfo
+        wi = WindowInfo(self._client, self)
+        wi.show()
+
     def show_session_info(self, *args):
         self._client.show_session_info(*args)
 
