@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2013 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,16 +14,16 @@ from OpenGL.GL import (
     GL_PROJECTION, GL_MODELVIEW,
     GL_UNPACK_ROW_LENGTH, GL_UNPACK_ALIGNMENT,
     GL_TEXTURE_MAG_FILTER, GL_TEXTURE_MIN_FILTER, GL_NEAREST,
-    GL_UNSIGNED_BYTE, GL_LUMINANCE, GL_LINEAR,
+    GL_UNSIGNED_BYTE, GL_UNSIGNED_SHORT,
+    GL_LUMINANCE, GL_LINEAR,
     GL_TEXTURE0, GL_TEXTURE1, GL_TEXTURE2, GL_QUADS, GL_LINE_LOOP, GL_LINES, GL_COLOR_BUFFER_BIT,
     GL_TEXTURE_WRAP_S, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER,
     GL_DONT_CARE, GL_TRUE, GL_DEPTH_TEST, GL_SCISSOR_TEST, GL_LIGHTING, GL_DITHER,
-    GL_RGB, GL_RGBA, GL_BGR, GL_BGRA, GL_RGBA8, GL_RGB8, GL_RGB10_A2, GL_RGB565, GL_RGB5_A1, GL_RGBA4,
+    GL_RGB, GL_RGBA, GL_BGR, GL_BGRA, GL_RGBA8, GL_RGB8, GL_RGB10_A2, GL_RGB565, GL_RGB5_A1, GL_RGBA4, GL_RGBA16,
     GL_UNSIGNED_INT_2_10_10_10_REV, GL_UNSIGNED_INT_10_10_10_2, GL_UNSIGNED_SHORT_5_6_5,
     GL_BLEND, GL_ONE, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA,
     GL_TEXTURE_MAX_LEVEL, GL_TEXTURE_BASE_LEVEL,
     GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST,
-    glLineStipple, GL_LINE_STIPPLE,
     glTexEnvi, GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE,
     glHint,
     glBlendFunc,
@@ -55,7 +55,7 @@ from xpra.os_util import (
     POSIX,
     DummyContextManager,
     )
-from xpra.util import envint, envbool, repr_ellipsized
+from xpra.util import envint, envbool, repr_ellipsized, first_time
 from xpra.client.paint_colors import get_paint_box_color
 from xpra.codecs.codec_constants import get_subsampling_divs
 from xpra.client.window_backing_base import (
@@ -88,12 +88,7 @@ if SAVE_BUFFERS not in ("png", "jpeg", None):
     SAVE_BUFFERS = None
 if SAVE_BUFFERS:
     from OpenGL.GL import glGetTexImage     #pylint: disable=ungrouped-imports
-    try:
-        import numpy
-        from PIL import Image, ImageOps
-    except ImportError:
-        log.warn("Warning: cannot save buffers, numpy is not installed")
-        SAVE_BUFFERS = False
+    from PIL import Image, ImageOps
 
 
 PIXEL_FORMAT_TO_CONSTANT = {
@@ -119,6 +114,13 @@ PIXEL_FORMAT_TO_DATATYPE = {
     "BGRX"  : GL_UNSIGNED_BYTE,
     "RGBA"  : GL_UNSIGNED_BYTE,
     "RGBX"  : GL_UNSIGNED_BYTE,
+    "YUV420P" : GL_UNSIGNED_BYTE,
+    "YUV422P" : GL_UNSIGNED_BYTE,
+    "YUV444P" : GL_UNSIGNED_BYTE,
+    "GBRP"  : GL_UNSIGNED_BYTE,
+    "GBRP16" : GL_UNSIGNED_SHORT,
+    "YUV444P10" : GL_UNSIGNED_SHORT,
+    "YUV444P16" : GL_UNSIGNED_SHORT,
     }
 CONSTANT_TO_PIXEL_FORMAT = {
     GL_BGR   : "BGR",
@@ -133,11 +135,13 @@ INTERNAL_FORMAT_TO_STR = {
     GL_RGB565       : "RGB565",
     GL_RGB5_A1      : "RGB5_A1",
     GL_RGBA4        : "RGBA4",
+    GL_RGBA16       : "GL_RGBA16",
     }
 DATATYPE_TO_STR = {
     GL_UNSIGNED_INT_2_10_10_10_REV  : "UNSIGNED_INT_2_10_10_10_REV",
     GL_UNSIGNED_INT_10_10_10_2      : "UNSIGNED_INT_10_10_10_2",
     GL_UNSIGNED_BYTE                : "UNSIGNED_BYTE",
+    GL_UNSIGNED_SHORT               : "UNSIGNED_SHORT",
     GL_UNSIGNED_SHORT_5_6_5         : "UNSIGNED_SHORT_5_6_5",
     }
 
@@ -227,7 +231,7 @@ class GLWindowBackingBase(WindowBackingBase):
     RGB_MODES = ["YUV420P", "YUV422P", "YUV444P", "GBRP", "BGRA", "BGRX", "RGBA", "RGBX", "RGB", "BGR"]
     HAS_ALPHA = GL_ALPHA_SUPPORTED
 
-    def __init__(self, wid : int, window_alpha, pixel_depth=0):
+    def __init__(self, wid : int, window_alpha : bool, pixel_depth : int=0):
         self.wid = wid
         self.texture_pixel_format = None
         #this is the pixel format we are currently updating the fbo with
@@ -259,6 +263,18 @@ class GLWindowBackingBase(WindowBackingBase):
         self.repaint_all = True
         self._backing.show()
 
+    def get_info(self):
+        info = super().get_info()
+        info.update({
+            "type"                  : "OpenGL",
+            "bit-depth"             : self.bit_depth,
+            "pixel-format"          : self.pixel_format,
+            "texture-pixel-format"  : CONSTANT_TO_PIXEL_FORMAT.get(self.texture_pixel_format, str(self.texture_pixel_format)),
+            "internal-format"       : INTERNAL_FORMAT_TO_STR.get(self.internal_format, str(self.internal_format)),
+            })
+        return info
+
+
     def init_gl_config(self):
         raise NotImplementedError()
 
@@ -280,10 +296,15 @@ class GLWindowBackingBase(WindowBackingBase):
 
     def init_formats(self):
         self.RGB_MODES = list(GLWindowBackingBase.RGB_MODES)
-        if self.bit_depth==30:
+        if self.bit_depth>32:
+            self.internal_format = GL_RGBA16
+            self.RGB_MODES.append("r210")
+            #self.RGB_MODES.append("GBRP16")
+        elif self.bit_depth==30:
             self.internal_format = GL_RGB10_A2
             self.RGB_MODES.append("r210")
-        elif self.bit_depth==16:
+            #self.RGB_MODES.append("GBRP16")
+        elif 0<self.bit_depth<=16:
             if self._alpha_enabled:
                 if envbool("XPRA_GL_RGBA4", True):
                     self.internal_format = GL_RGBA4
@@ -296,6 +317,8 @@ class GLWindowBackingBase(WindowBackingBase):
                 self.RGB_MODES.append("BGR565")
                 self.RGB_MODES.append("RGB565")
         else:
+            if self.bit_depth not in (0, 24, 32) and first_time("bit-depth-%i" % self.bit_depth):
+                log.warn("Warning: invalid bit depth %i, using 24", self.bit_depth)
             #assume 24:
             if self._alpha_enabled:
                 self.internal_format = GL_RGBA8
@@ -621,7 +644,7 @@ class GLWindowBackingBase(WindowBackingBase):
                 glBlitFramebuffer(x, bh-y, x+w, bh-(y+h),
                                   x+xdelta, bh-(y+ydelta), x+w+xdelta, bh-(y+h+ydelta),
                                   GL_COLOR_BUFFER_BIT, GL_NEAREST)
-                self.paint_box("scroll", True, x+xdelta, y+ydelta, x+w+xdelta, y+h+ydelta)
+                self.paint_box("scroll", x+xdelta, y+ydelta, x+w+xdelta, y+h+ydelta)
                 glFlush()
 
             self.swap_fbos()
@@ -784,11 +807,15 @@ class GLWindowBackingBase(WindowBackingBase):
         glReadBuffer(GL_COLOR_ATTACHMENT0)
         glViewport(0, 0, bw, bh)
         size = bw*bh*4
-        data = numpy.empty(size)
-        img_data = glGetTexImage(target, 0, GL_BGRA, GL_UNSIGNED_BYTE, data)
-        img = Image.frombuffer("RGBA", (bw, bh), img_data, "raw", "BGRA", bw*4)
+        from xpra.buffers.membuf import get_membuf  #@UnresolvedImport
+        membuf = get_membuf(size)
+        glGetTexImage(target, 0, GL_BGRA, GL_UNSIGNED_BYTE, membuf.get_mem_ptr())
+        pixels = memoryview(membuf).tobytes()
+        img = Image.frombuffer("RGBA", (bw, bh), pixels, "raw", "BGRA", bw*4)
         img = ImageOps.flip(img)
         kwargs = {}
+        if not self._alpha_enabled or SAVE_BUFFERS=="jpeg":
+            img = img.convert("RGB")
         if SAVE_BUFFERS=="jpeg":
             kwargs = {
                       "quality"     : 0,
@@ -925,23 +952,18 @@ class GLWindowBackingBase(WindowBackingBase):
         glBindTexture(target, 0)
         glDisable(target)
 
-    def paint_box(self, encoding : str, is_delta : bool, x : int, y : int, w : int, h : int):
+    def paint_box(self, encoding : str, x : int, y : int, w : int, h : int):
         #show region being painted if debug paint box is enabled only:
         if self.paint_box_line_width<=0:
             return
         glLineWidth(self.paint_box_line_width+0.5+int(encoding=="scroll")*2)
-        if is_delta:
-            glLineStipple(1, 0xaaaa)
-            glEnable(GL_LINE_STIPPLE)
         glBegin(GL_LINE_LOOP)
         color = get_paint_box_color(encoding)
-        log("Painting colored box around %s screen update using: %s (delta=%s)", encoding, color, is_delta)
+        log("Painting colored box around %s screen update using: %s", encoding, color)
         glColor4f(*color)
         for px,py in ((x, y), (x+w, y), (x+w, y+h), (x, y+h)):
             glVertex2i(px, py)
         glEnd()
-        if is_delta:
-            glDisable(GL_LINE_STIPPLE)
 
 
     def pixels_for_upload(self, img_data):
@@ -962,7 +984,7 @@ class GLWindowBackingBase(WindowBackingBase):
         return "copy:bytes(%s)" % type(img_data), strtobytes(img_data)
 
     def set_alignment(self, width : int, rowstride : int, pixel_format):
-        bytes_per_pixel = len(pixel_format)       #ie: BGRX -> 4
+        bytes_per_pixel = len(pixel_format)       #ie: BGRX -> 4, Y -> 1, YY -> 2
         # Compute alignment and row length
         row_length = 0
         alignment = 1
@@ -983,11 +1005,14 @@ class GLWindowBackingBase(WindowBackingBase):
 
     def paint_jpeg(self, img_data, x : int, y : int, width : int, height : int, options, callbacks):
         if JPEG_YUV and width>=2 and height>=2:
-            img = self.jpeg_decoder.decompress_to_yuv(img_data, width, height, options)
+            img = self.jpeg_decoder.decompress_to_yuv(img_data)
             flush = options.intget("flush", 0)
-            self.idle_add(self.gl_paint_planar, YUV2RGB_FULL_SHADER, flush, "jpeg", img, x, y, width, height, width, height, callbacks)
+            w = img.get_width()
+            h = img.get_height()
+            self.idle_add(self.gl_paint_planar, YUV2RGB_FULL_SHADER, flush, "jpeg", img,
+                          x, y, w, h, width, height, options, callbacks)
         else:
-            img = self.jpeg_decoder.decompress_to_rgb("BGRX", img_data, width, height, options)
+            img = self.jpeg_decoder.decompress_to_rgb("BGRX", img_data)
             self.idle_add(self.do_paint_rgb, "BGRX", img.get_pixels(), x, y, width, height, img.get_rowstride(), options, callbacks)
 
     def paint_webp(self, img_data, x : int, y : int, width : int, height : int, options, callbacks):
@@ -996,13 +1021,19 @@ class GLWindowBackingBase(WindowBackingBase):
         if subsampling=="YUV420P" and WEBP_YUV and self.webp_decoder and not WEBP_PILLOW and not has_alpha and width>=2 and height>=2:
             img = self.webp_decoder.decompress_yuv(img_data)
             flush = options.intget("flush", 0)
-            self.idle_add(self.gl_paint_planar, YUV2RGB_SHADER, flush, "webp", img, x, y, width, height, width, height, callbacks)
+            w = img.get_width()
+            h = img.get_height()
+            self.idle_add(self.gl_paint_planar, YUV2RGB_SHADER, flush, "webp", img,
+                          x, y, w, h, width, height, options, callbacks)
             return
         super().paint_webp(img_data, x, y, width, height, options, callbacks)
 
-    def do_paint_rgb(self, rgb_format, img_data, x : int, y : int, width : int, height : int, rowstride, options, callbacks):
+    def do_paint_rgb(self, rgb_format, img_data,
+                     x : int, y : int, width : int, height : int, render_width : int, render_height : int,
+                     rowstride, options, callbacks):
         log("%s.do_paint_rgb(%s, %s bytes, x=%d, y=%d, width=%d, height=%d, rowstride=%d, options=%s)",
             self, rgb_format, len(img_data), x, y, width, height, rowstride, options)
+        x, y = self.gravity_adjust(x, y, options)
         context = self.gl_context()
         if not context:
             log("%s._do_paint_rgb(..) no context!", self)
@@ -1020,6 +1051,7 @@ class GLWindowBackingBase(WindowBackingBase):
 
             with context:
                 self.gl_init()
+                scaling = width!=render_width or height!=render_height
 
                 #convert it to a GL constant:
                 pformat = PIXEL_FORMAT_TO_CONSTANT.get(rgb_format)
@@ -1027,15 +1059,19 @@ class GLWindowBackingBase(WindowBackingBase):
                 ptype = PIXEL_FORMAT_TO_DATATYPE.get(rgb_format)
                 assert pformat is not None, "could not find pixel type for %s" % rgb_format
 
-                self.gl_marker("%s update at (%d,%d) size %dx%d (%s bytes), using GL %s format=%s / %s to internal format=%s",
-                               rgb_format, x, y, width, height, len(img_data), upload, CONSTANT_TO_PIXEL_FORMAT.get(pformat), DATATYPE_TO_STR.get(ptype), INTERNAL_FORMAT_TO_STR.get(self.internal_format))
+                self.gl_marker("%s update at (%d,%d) size %dx%d (%s bytes) to %dx%d, using GL %s format=%s / %s to internal format=%s",
+                               rgb_format, x, y, width, height, len(img_data), render_width, render_height,
+                               upload, CONSTANT_TO_PIXEL_FORMAT.get(pformat), DATATYPE_TO_STR.get(ptype), INTERNAL_FORMAT_TO_STR.get(self.internal_format))
 
                 # Upload data as temporary RGB texture
                 target = GL_TEXTURE_RECTANGLE_ARB
                 glEnable(target)
                 glBindTexture(target, self.textures[TEX_RGB])
                 self.set_alignment(width, rowstride, rgb_format)
-                glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
+                mag_filter = GL_NEAREST
+                if scaling:
+                    mag_filter = GL_LINEAR
+                glTexParameteri(target, GL_TEXTURE_MAG_FILTER, mag_filter)
                 glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
                 glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER)
                 glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER)
@@ -1046,19 +1082,19 @@ class GLWindowBackingBase(WindowBackingBase):
                 glTexCoord2i(0, 0)
                 glVertex2i(x, y)
                 glTexCoord2i(0, height)
-                glVertex2i(x, y+height)
+                glVertex2i(x, y+render_height)
                 glTexCoord2i(width, height)
-                glVertex2i(x+width, y+height)
+                glVertex2i(x+render_width, y+render_height)
                 glTexCoord2i(width, 0)
-                glVertex2i(x+width, y)
+                glVertex2i(x+render_width, y)
                 glEnd()
 
                 glBindTexture(target, 0)
                 glDisable(target)
-                self.paint_box(options.strget("encoding"), options.intget("delta", -1)>=0, x, y, width, height)
+                self.paint_box(options.strget("encoding"), x, y, render_width, render_height)
                 # Present update to screen
                 if not self.draw_needs_refresh:
-                    self.present_fbo(x, y, width, height, options.intget("flush", 0))
+                    self.present_fbo(x, y, render_width, render_height, options.intget("flush", 0))
                 # present_fbo has reset state already
             fire_paint_callbacks(callbacks)
             return
@@ -1076,20 +1112,26 @@ class GLWindowBackingBase(WindowBackingBase):
             #copy so the data will be usable (usually a str)
             img.clone_pixel_data()
         pixel_format = img.get_pixel_format()
-        if pixel_format=="GBRP":
+        if pixel_format in ("GBRP10", "YUV444P10"):
+            #call superclass to handle csc
+            #which will end up calling paint rgb with r210 data
+            return super().do_video_paint(img, x, y, enc_width, enc_height, width, height, options, callbacks)
+        if pixel_format.startswith("GBRP"):
             shader = RGBP2RGB_SHADER
         else:
             shader = YUV2RGB_SHADER
         self.idle_add(self.gl_paint_planar, shader, options.intget("flush", 0), options.strget("encoding"), img,
-                      x, y, enc_width, enc_height, width, height, callbacks)
+                      x, y, enc_width, enc_height, width, height, options, callbacks)
 
     def gl_paint_planar(self, shader, flush, encoding, img,
-                        x : int, y : int, enc_width : int, enc_height : int, width : int, height : int, callbacks):
+                        x : int, y : int, enc_width : int, enc_height : int, width : int, height : int,
+                        options, callbacks):
         #this function runs in the UI thread, no video_decoder lock held
-        log("gl_paint_planar%s", (flush, encoding, img, x, y, enc_width, enc_height, width, height, callbacks))
+        log("gl_paint_planar%s", (flush, encoding, img, x, y, enc_width, enc_height, width, height, options, callbacks))
+        x, y = self.gravity_adjust(x, y, options)
         try:
             pixel_format = img.get_pixel_format()
-            assert pixel_format in ("YUV420P", "YUV422P", "YUV444P", "GBRP"), \
+            assert pixel_format in ("YUV420P", "YUV422P", "YUV444P", "GBRP", "GBRP16", "YUV444P16"), \
                 "sorry the GL backing does not handle pixel format '%s' yet!" % (pixel_format)
 
             context = self.gl_context()
@@ -1104,12 +1146,12 @@ class GLWindowBackingBase(WindowBackingBase):
 
                 # Update FBO texture
                 x_scale, y_scale = 1, 1
-                if width!=enc_width or height!=enc_height:
+                if scaling:
                     x_scale = width/enc_width
                     y_scale = height/enc_height
 
                 self.render_planar_update(x, y, enc_width, enc_height, x_scale, y_scale, shader)
-                self.paint_box(encoding, False, x, y, width, height)
+                self.paint_box(encoding, x, y, width, height)
                 fire_paint_callbacks(callbacks, True)
                 # Present it on screen
                 if not self.draw_needs_refresh:
@@ -1130,6 +1172,7 @@ class GLWindowBackingBase(WindowBackingBase):
         assert self.textures is not None, "no OpenGL textures!"
         log("%s.update_planar_textures%s", self, (width, height, img, pixel_format))
 
+        upload_format = PIXEL_FORMAT_TO_DATATYPE[pixel_format]
         divs = get_subsampling_divs(pixel_format)
         if self.pixel_format is None or self.pixel_format!=pixel_format or self.texture_size!=(width, height):
             self.gl_marker("Creating new planar textures, pixel format %s (was %s), texture size %s (was %s)",
@@ -1147,17 +1190,18 @@ class GLWindowBackingBase(WindowBackingBase):
                     mag_filter = GL_LINEAR
                 glTexParameteri(target, GL_TEXTURE_MAG_FILTER, mag_filter)
                 glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
-                glTexImage2D(target, 0, GL_LUMINANCE, width//div_w, height//div_h, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, None)
+                glTexImage2D(target, 0, GL_LUMINANCE, width//div_w, height//div_h, 0, GL_LUMINANCE, upload_format, None)
                 #glBindTexture(target, 0)        #redundant: we rebind below:
 
         self.gl_marker("updating planar textures: %sx%s %s", width, height, pixel_format)
         rowstrides = img.get_rowstride()
         img_data = img.get_pixels()
+        BPP = 2 if pixel_format.endswith("P16") else 1
         assert len(rowstrides)==3 and len(img_data)==3
         for texture, index, tex_name in (
-            (GL_TEXTURE0, TEX_Y, "Y"),
-            (GL_TEXTURE1, TEX_U, "U"),
-            (GL_TEXTURE2, TEX_V, "V"),
+            (GL_TEXTURE0, TEX_Y, pixel_format[0:1]*BPP),
+            (GL_TEXTURE1, TEX_U, pixel_format[1:2]*BPP),
+            (GL_TEXTURE2, TEX_V, pixel_format[2:3]*BPP),
             ):
             div_w, div_h = divs[index]
             w = width//div_w
@@ -1179,14 +1223,14 @@ class GLWindowBackingBase(WindowBackingBase):
                 glTexParameteri(target, GL_TEXTURE_MAX_LEVEL, 0)
             except Exception:
                 pass
-            glTexSubImage2D(target, 0, 0, 0, w, h, GL_LUMINANCE, GL_UNSIGNED_BYTE, pixel_data)
+            glTexSubImage2D(target, 0, 0, 0, w, h, GL_LUMINANCE, upload_format, pixel_data)
             glBindTexture(target, 0)
         #glActiveTexture(GL_TEXTURE0)    #redundant, we always call render_planar_update afterwards
 
     def render_planar_update(self, rx : int, ry : int, rw : int, rh : int, x_scale=1, y_scale=1, shader=YUV2RGB_SHADER):
         log("%s.render_planar_update%s pixel_format=%s",
             self, (rx, ry, rw, rh, x_scale, y_scale, shader), self.pixel_format)
-        if self.pixel_format not in ("YUV420P", "YUV422P", "YUV444P", "GBRP"):
+        if self.pixel_format not in ("YUV420P", "YUV422P", "YUV444P", "GBRP", "GBRP16", "YUV444P16"):
             #not ready to render yet
             return
         self.gl_marker("painting planar update, format %s", self.pixel_format)
