@@ -4,7 +4,9 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os.path
 import sys
+
 from gi.repository import GLib
 from gi.repository import GObject
 
@@ -12,7 +14,8 @@ from xpra.util import (
     nonl, sorted_nicely, print_nested_dict, envint, flatten_dict, typedict,
     disconnect_is_an_error, ellipsizer, DONE, first_time,
     )
-from xpra.os_util import bytestostr, get_hex_uuid, POSIX, OSX, hexstr
+from xpra.os_util import bytestostr, strtobytes, get_hex_uuid, POSIX, OSX, hexstr
+from xpra.simple_stats import std_unit
 from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT
 from xpra.exit_codes import (
     EXIT_OK, EXIT_CONNECTION_LOST, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR,
@@ -55,15 +58,8 @@ class GObjectXpraClient(GObject.GObject, XpraClientBase):
 
 
     def install_signal_handlers(self):
-        #only register the glib signal handler
-        #once the main loop is running,
-        #before that we just trigger a KeyboardInterrupt
-        def do_install_signal_handlers():
-            sstr = "%s Client" % self.client_type()
-            from xpra.gtk_common.gobject_compat import register_os_signals, register_SIGUSR_signals
-            register_os_signals(self.handle_app_signal, sstr)
-            register_SIGUSR_signals(sstr)
-        GLib.idle_add(do_install_signal_handlers)
+        from xpra.gtk_common.gobject_compat import install_signal_handlers
+        install_signal_handlers("%s Client" % self.client_type(), self.handle_app_signal)
 
 
     def setup_connection(self, conn):
@@ -110,8 +106,10 @@ class GObjectXpraClient(GObject.GObject, XpraClientBase):
         if self.exit_code is None:
             self.exit_code = exit_code
         self.cleanup()
-        GLib.timeout_add(50, self.glib_mainloop.quit)
+        GLib.timeout_add(50, self.exit_loop)
 
+    def exit_loop(self):
+        self.glib_mainloop.quit()
 
 
 class CommandConnectClient(GObjectXpraClient):
@@ -128,6 +126,7 @@ class CommandConnectClient(GObjectXpraClient):
         #so don't try probing for printers, etc
         self.file_transfer = False
         self.printing = False
+        self.command_timeout = None
         #don't bother with many of these things for one-off commands:
         for x in ("ui_client", "wants_aliases", "wants_encodings",
                   "wants_versions", "wants_features", "wants_sound", "windows",
@@ -138,12 +137,17 @@ class CommandConnectClient(GObjectXpraClient):
     def setup_connection(self, conn):
         protocol = super().setup_connection(conn)
         if conn.timeout>0:
-            GLib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
+            self.command_timeout = GLib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
         return protocol
 
     def timeout(self, *_args):
         log.warn("timeout!")    # pragma: no cover
 
+    def cancel_command_timeout(self):
+        ct = self.command_timeout
+        if ct:
+            self.command_timeout = None
+            GLib.source_remove(ct)
 
     def _process_connection_lost(self, _packet):
         #override so we don't log a warning
@@ -160,6 +164,7 @@ class CommandConnectClient(GObjectXpraClient):
         #* we don't care about sending anything back after hello
         log("server_capabilities: %s", ellipsizer(caps))
         log("protocol state: %s", self._protocol.save_state())
+        self.cancel_command_timeout()
         self.do_command(caps)
         return True
 
@@ -236,8 +241,13 @@ class ScreenshotXpraClient(CommandConnectClient):
             self.warn_and_quit(EXIT_OK,
                                "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
             return
-        with open(self.screenshot_filename, 'wb') as f:
-            f.write(img_data)
+        if self.screenshot_filename=="-":
+            output = os.fdopen(sys.stdout.fileno(), "wb", closefd=False)
+        else:
+            output = open(self.screenshot_filename, "wb")
+        with output:
+            output.write(img_data)
+            output.flush()
         self.warn_and_quit(EXIT_OK, "screenshot %sx%s saved to: %s" % (w, h, self.screenshot_filename))
 
     def init_packet_handlers(self):
@@ -370,6 +380,96 @@ class MonitorXpraClient(SendCommandConnectClient):
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
 
+class ShellXpraClient(SendCommandConnectClient):
+    """
+        Provides an interactive shell with the socket it connects to
+    """
+
+    def __init__(self, opts):
+        super().__init__(opts)
+        self.stdin_io_watch = None
+        self.stdin_buffer = ""
+        self.hello_extra["shell"] = "True"
+
+    def timeout(self, *args):
+        pass
+
+    def cleanup(self):
+        siw = self.stdin_io_watch
+        if siw:
+            self.stdin_io_watch = None
+            self.source_remove(siw)
+        super().cleanup()
+
+    def do_command(self, caps : typedict):
+        if not caps.boolget("shell"):
+            msg = "this server does not support the 'shell' subcommand"
+            log.error(msg)
+            self.disconnect_and_quit(EXIT_UNSUPPORTED, msg)
+            return
+        #start reading from stdin:
+        self.install_signal_handlers()
+        stdin = sys.stdin
+        fileno = stdin.fileno()
+        import fcntl
+        fl = fcntl.fcntl(fileno, fcntl.F_GETFL)
+        fcntl.fcntl(fileno, fcntl.F_SETFL, fl | os.O_NONBLOCK)
+        self.stdin_io_watch = GLib.io_add_watch(sys.stdin,
+                                                GLib.PRIORITY_DEFAULT, GLib.IO_IN,
+                                                self.stdin_ready)
+        self.print_prompt()
+
+    def stdin_ready(self, *_args):
+        data = sys.stdin.read()
+        #log.warn("stdin=%r", data)
+        self.stdin_buffer += data
+        sent = 0
+        if self.stdin_buffer.endswith("\n"):
+            for line in self.stdin_buffer.splitlines():
+                if line:
+                    if line.rstrip("\n\r") in ("quit", "exit"):
+                        self.disconnect_and_quit(EXIT_OK, "user requested %s" % line)
+                        self.stdin_io_watch = None
+                        return False
+                    self.send("shell-exec", line.encode())
+                    sent += 1
+        self.stdin_buffer = ""
+        if not sent:
+            self.print_prompt()
+        return True
+
+    def init_packet_handlers(self):
+        super().init_packet_handlers()
+        self._packet_handlers["shell-reply"] = self._process_shell_reply
+        self._packet_handlers["ping"] = self._process_ping
+
+    def _process_ping(self, packet):
+        echotime = packet[1]
+        self.send("ping_echo", echotime, 0, 0, 0, -1)
+
+    def _process_shell_reply(self, packet):
+        fd = packet[1]
+        message = packet[2]
+        if fd==1:
+            stream = sys.stdout
+        elif fd==2:
+            stream = sys.stderr
+        else:
+            raise Exception("invalid file descriptor %i" % fd)
+        s = message.decode("utf8")
+        if s.endswith("\n"):
+            s = s[:-1]
+        stream.write("%s" % s)
+        stream.flush()
+        if fd==2:
+            stream.write("\n")
+            self.print_prompt()
+
+    def print_prompt(self):
+        sys.stdout.write("> ")
+        sys.stdout.flush()
+
+
 class VersionXpraClient(HelloRequestClient):
     """ This client does one thing only:
         it queries the server for version information and prints it out
@@ -392,6 +492,7 @@ class VersionXpraClient(HelloRequestClient):
             self.warn_and_quit(EXIT_FAILURE, "server did not provide the version information")
         else:
             sys.stdout.write("%s\n" % (v,))
+            sys.stdout.flush()
             self.quit(EXIT_OK)
 
 
@@ -420,7 +521,12 @@ class ControlXpraClient(CommandConnectClient):
     def make_hello(self):
         capabilities = super().make_hello()
         log("make_hello() adding command request '%s' to %s", self.command, capabilities)
-        capabilities["command_request"] = self.command
+        def b(s):
+            try:
+                return s.encode("utf8")
+            except:
+                return strtobytes(s)
+        capabilities["command_request"] = tuple(b(x) for x in self.command)
         capabilities["request"] = "command"
         return capabilities
 
@@ -437,25 +543,26 @@ class PrintClient(SendCommandConnectClient):
         #TODO: load as needed...
         def sizeerr(size):
             self.warn_and_quit(EXIT_FILE_TOO_BIG,
-                               "the file is too large: %iMB (the file size limit is %iMB)" % (size//1024//1024, self.file_size_limit))
+                               "the file is too large: %sB (the file size limit is %sB)" % (
+                                   std_unit(size), std_unit(self.file_size_limit)))
             return
         if self.filename=="-":
             #replace with filename proposed
             self.filename = command[2]
             #read file from stdin
-            self.file_data = sys.stdin.read()
+            with open(sys.stdin.fileno(), mode='rb', closefd=False) as stdin_binary:
+                self.file_data = stdin_binary.read()
             log("read %i bytes from stdin", len(self.file_data))
         else:
-            import os.path
             size = os.path.getsize(self.filename)
-            if size>self.file_size_limit*1024*1024:
+            if size>self.file_size_limit:
                 sizeerr(size)
                 return
             from xpra.os_util import load_binary_file
             self.file_data = load_binary_file(self.filename)
             log("read %i bytes from %s", len(self.file_data), self.filename)
         size = len(self.file_data)
-        if size>self.file_size_limit*1024*1024:
+        if size>self.file_size_limit:
             sizeerr(size)
             return
         assert self.file_data, "no data found for '%s'" % self.filename
