@@ -7,25 +7,30 @@
 import os.path
 import sys
 import time
-from gi.repository import Gtk, Gdk, GdkPixbuf
+from gi.repository import Gtk, Gdk
 
 from xpra.gtk_common.gtk_util import (
-    add_close_accel, scaled_image,
+    add_close_accel, scaled_image, get_icon_pixbuf,
     get_display_info, get_default_root_window,
     choose_file, get_gtk_version_info,
     )
 from xpra.os_util import hexstr
 from xpra.platform.gui import force_focus
-from xpra.util import nonl, envint, repr_ellipsized
+from xpra.util import nonl, envint, envbool, repr_ellipsized
 from xpra.log import Logger
 
 log = Logger("util")
 
 STEP_DELAY = envint("XPRA_BUG_REPORT_STEP_DELAY", 0)
+OBFUSCATE = envbool("XPRA_OBFUSCATE_BUG_REPORT", True)
 
 
 class BugReport:
 
+    def __init__(self):
+        self.checkboxes = {}
+        self.server_log = None
+
     def init(self, show_about=True, get_server_info=None, opengl_info=None, includes=None):
         self.show_about = show_about
         self.get_server_info = get_server_info
@@ -40,7 +45,7 @@ class BugReport:
         self.window.set_default_size(400, 300)
         self.window.set_title("Xpra Bug Report")
 
-        icon_pixbuf = self.get_icon("bugs.png")
+        icon_pixbuf = get_icon_pixbuf("bugs.png")
         if icon_pixbuf:
             self.window.set_icon(icon_pixbuf)
         self.window.set_position(Gtk.WindowPosition.CENTER)
@@ -50,7 +55,7 @@ class BugReport:
 
         # Title
         hbox = Gtk.HBox(False, 0)
-        icon_pixbuf = self.get_icon("xpra.png")
+        icon_pixbuf = get_icon_pixbuf("xpra.png")
         if icon_pixbuf and self.show_about:
             from xpra.gtk_common.about import about
             logo_button = Gtk.Button("")
@@ -113,7 +118,7 @@ class BugReport:
                     "executable"    : sys.executable,
                     "version"       : get_version_info(),
                     "platform"      : get_platform_info(),
-                    "host"          : get_host_info(),
+                    "host"          : get_host_info(OBFUSCATE),
                     "paths"         : get_path_info(),
                     "gtk"           : get_gtk_version_info(),
                     "gui"           : get_gui_info(),
@@ -158,31 +163,36 @@ class BugReport:
                 #take_screenshot() returns: w, h, "png", rowstride, data
                 return take_screenshot_fn()[4]
             get_screenshot = _get_screenshot
+        def get_server_log():
+            return self.server_log
         self.toggles = (
-                   ("system",       "txt",  "System",           get_sys_info,
-                    "Xpra version, platform and host information"),
-                   ("network",      "txt",  "Network",          get_net_info,
-                    "Compression, packet encoding and encryption"),
-                   ("encoding",     "txt",  "Encodings",        codec_versions,
-                    "Picture encodings supported"),
-                   ("opengl",       "txt",  "OpenGL",           get_gl_info,
-                    "OpenGL driver and features"),
-                   ("sound",        "txt",  "Sound",            get_sound_info,
-                    "Sound codecs and GStreamer version information"),
-                   ("keyboard",     "txt",  "Keyboard Mapping", get_gtk_keymap,
-                    "Keyboard layout and key mapping"),
-                   ("xpra-info",    "txt",  "Server Info",      self.get_server_info,
-                    "Full server information from 'xpra info'"),
-                   ("screenshot",   "png",  "Screenshot",       get_screenshot,
-                    ""),
-                   )
-        for name, _, title, value_cb, tooltip in self.toggles:
+            ("system",       "txt",  "System",           get_sys_info,   True,
+             "Xpra version, platform and host information - including hostname and account information"),
+            ("server-log",   "txt",  "Server Log",       get_server_log, bool(self.server_log),
+             "Xpra version, platform and host information - including hostname and account information"),
+            ("network",      "txt",  "Network",          get_net_info,   True,
+             "Compression, packet encoding and encryption"),
+            ("encoding",     "txt",  "Encodings",        codec_versions, bool(codec_versions),
+             "Picture encodings supported"),
+            ("opengl",       "txt",  "OpenGL",           get_gl_info,    bool(self.opengl_info),
+             "OpenGL driver and features"),
+            ("sound",        "txt",  "Sound",            get_sound_info, bool(get_sound_info),
+             "Sound codecs and GStreamer version information"),
+            ("keyboard",     "txt",  "Keyboard Mapping", get_gtk_keymap, True,
+             "Keyboard layout and key mapping"),
+            ("xpra-info",    "txt",  "Server Info",      self.get_server_info,   bool(self.get_server_info),
+             "Full server information from 'xpra info'"),
+            ("screenshot",   "png",  "Screenshot",       get_screenshot, bool(get_screenshot),
+             ""),
+            )
+        self.checkboxes = {}
+        for name, _, title, value_cb, sensitive, tooltip in self.toggles:
             cb = Gtk.CheckButton(title+[" (not available)", ""][bool(value_cb)])
             cb.set_active(self.includes.get(name, True))
-            cb.set_sensitive(bool(value_cb))
+            cb.set_sensitive(sensitive)
             cb.set_tooltip_text(tooltip)
             ibox.pack_start(cb)
-            setattr(self, name, cb)
+            self.checkboxes[name] = cb
 
         # Buttons:
         hbox = Gtk.HBox(False, 20)
@@ -192,7 +202,7 @@ class BugReport:
             btn.set_tooltip_text(tooltip)
             btn.connect("clicked", callback)
             if icon_name:
-                icon = self.get_icon(icon_name)
+                icon = get_icon_pixbuf(icon_name)
                 if icon:
                     btn.set_image(scaled_image(icon, 24))
             hbox.pack_start(btn)
@@ -210,6 +220,14 @@ class BugReport:
         self.window.add(vbox)
 
 
+    def set_server_log_data(self, filedata):
+        self.server_log = filedata
+        cb = self.checkboxes.get("server-log")
+        log("set_server_log_data(%i bytes) cb=%s", len(filedata or b""), cb)
+        if cb:
+            cb.set_sensitive(bool(filedata))
+
+
     def show(self):
         log("show()")
         if not self.window:
@@ -248,14 +266,6 @@ class BugReport:
         Gtk.main_quit()
 
 
-    def get_icon(self, icon_name):
-        from xpra.platform.paths import get_icon_dir
-        icon_filename = os.path.join(get_icon_dir(), icon_name)
-        if os.path.exists(icon_filename):
-            return GdkPixbuf.Pixbuf.new_from_file(icon_filename)
-        return None
-
-
     def get_data(self):
         log("get_data() collecting bug report data")
         data = []
@@ -263,10 +273,10 @@ class BugReport:
         buf = tb.get_text(*tb.get_bounds(), include_hidden_chars=False)
         if buf:
             data.append(("Description", "", "txt", buf))
-        for name, dtype, title, value_cb, tooltip in self.toggles:
+        for name, dtype, title, value_cb, _, tooltip in self.toggles:
             if not bool(value_cb):
                 continue
-            cb = getattr(self, name)
+            cb = self.checkboxes.get(name)
             assert cb is not None
             if not cb.get_active():
                 continue
@@ -285,7 +295,7 @@ class BugReport:
                     value = e
                     dtype = "txt"
             if value is None:
-                value = "not available"
+                s = "not available"
             elif isinstance(value, dict):
                 s = os.linesep.join("%s : %s" % (k.ljust(32), nonl(str(v))) for k,v in sorted(value.items()))
             elif isinstance(value, (list, tuple)):
@@ -336,7 +346,7 @@ class BugReport:
                     try:
                         try:
                             import tempfile
-                            temp = tempfile.NamedTemporaryFile(prefix="xpra.", suffix=".screenshot", delete=False)
+                            temp = tempfile.NamedTemporaryFile(prefix="xpra.", suffix=".%s" % dtype, delete=False)
                             with temp:
                                 temp.write(s)
                                 temp.flush()
@@ -344,7 +354,7 @@ class BugReport:
                             log.error("Error: cannot create mmap file:")
                             log.error(" %s", e)
                         else:
-                            zf.write(temp.name, cfile, zipfile.ZIP_STORED)
+                            zf.write(temp.name, cfile, zipfile.ZIP_STORED if dtype=="png" else zipfile.ZIP_DEFLATED)
                     finally:
                         if temp:
                             os.unlink(temp.name)
@@ -355,7 +365,7 @@ class BugReport:
             dialog = Gtk.MessageDialog(self.window, 0, Gtk.MessageType.WARNING,
                                        Gtk.ButtonsType.CLOSE, "Failed to save ZIP file")
             dialog.format_secondary_text("%s" % e)
-            def close(*args):
+            def close(*_args):
                 dialog.destroy()
             dialog.connect("response", close)
             dialog.show_all()
