@@ -6,7 +6,7 @@
 
 from xpra.platform import program_context
 from xpra.platform.gui import force_focus
-from xpra.gtk_common.gtk_util import add_close_accel
+from xpra.gtk_common.gtk_util import add_close_accel, get_icon_pixbuf
 
 import sys
 import gi
@@ -16,9 +16,14 @@ from gi.repository import Gtk, GLib   #pylint: disable=wrong-import-position
 
 def make_window():
     window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
+    window.set_title("Text Entry")
     window.connect("destroy", Gtk.main_quit)
     window.set_default_size(320, 200)
     window.set_border_width(20)
+    window.set_position(Gtk.WindowPosition.CENTER)
+    icon = get_icon_pixbuf("font.png")
+    if icon:
+        window.set_icon(icon)
 
     entry = Gtk.Entry()
     entry.set_text("hello")
@@ -35,6 +40,10 @@ def main():
             w.show_all()
             w.present()
         GLib.idle_add(show_with_focus)
+        def signal_handler(*_args):
+            Gtk.main_quit()
+        from xpra.gtk_common.gobject_compat import register_os_signals
+        register_os_signals(signal_handler)
         Gtk.main()
 
 
