@@ -5,7 +5,7 @@
 
 from xpra.platform import program_context
 from xpra.platform.gui import force_focus
-from xpra.gtk_common.gtk_util import add_close_accel
+from xpra.gtk_common.gtk_util import add_close_accel, get_icon_pixbuf
 
 import gi
 gi.require_version("Gtk", "3.0")
@@ -15,9 +15,12 @@ opacity = 50
 
 def make_window():
     win = Gtk.Window()
-
-    win.set_title('Alpha Demo')
+    win.set_position(Gtk.WindowPosition.CENTER)
+    win.set_title('Opacity Test')
     win.connect('delete-event', Gtk.main_quit)
+    icon = get_icon_pixbuf("windows.png")
+    if icon:
+        win.set_icon(icon)
 
     btn = Gtk.Button(label="Change Opacity")
     def change_opacity(*_args):
@@ -38,6 +41,10 @@ def main():
             w.show_all()
             w.present()
         add_close_accel(w, Gtk.main_quit)
+        from xpra.gtk_common.gobject_compat import register_os_signals
+        def signal_handler(*_args):
+            Gtk.main_quit()
+        register_os_signals(signal_handler)
         GLib.idle_add(show_with_focus)
         Gtk.main()
         return 0
