@@ -10,14 +10,13 @@ import weakref
 from gi.repository import Gtk, Gdk, GdkPixbuf
 
 from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS, XSHAPE
-from xpra.gtk_common.quit import gtk_main_quit_really, gtk_main_quit_on_fatal_exceptions_enable
 from xpra.util import (
     updict, pver, iround, flatten_dict,
     envbool, envint, repr_ellipsized, csv, first_time, typedict,
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID,
     )
 from xpra.os_util import (
-    bytestostr, strtobytes, hexstr, monotonic_time,
+    bytestostr, strtobytes, hexstr, monotonic_time, load_binary_file,
     WIN32, OSX, POSIX, is_Wayland,
     )
 from xpra.simple_stats import std_unit
@@ -35,8 +34,10 @@ from xpra.gtk_common.gobject_util import no_arg_signal
 from xpra.client.ui_client_base import UIXpraClient
 from xpra.client.gobject_client_base import GObjectXpraClient
 from xpra.client.gtk_base.gtk_keyboard_helper import GTKKeyboardHelper
+from xpra.client.gtk_base.css_overrides import inject_css_overrides
 from xpra.client.mixins.window_manager import WindowClient
 from xpra.platform.paths import get_icon_filename
+from xpra.platform.gui import force_focus
 from xpra.platform.gui import (
     get_window_frame_sizes, get_window_frame_size,
     system_bell, get_wm_name, get_fixed_cursor_size,
@@ -52,6 +53,7 @@ filelog = Logger("gtk", "client", "file")
 clipboardlog = Logger("gtk", "client", "clipboard")
 notifylog = Logger("gtk", "notify")
 grablog = Logger("client", "grab")
+focuslog = Logger("client", "focus")
 
 missing_cursor_names = set()
 
@@ -66,6 +68,8 @@ OPENGL_MIN_SIZE = envint("XPRA_OPENGL_MIN_SIZE", 32)
 NO_OPENGL_WINDOW_TYPES = os.environ.get("XPRA_NO_OPENGL_WINDOW_TYPES",
                                         "DOCK,TOOLBAR,MENU,UTILITY,SPLASH,DROPDOWN_MENU,POPUP_MENU,TOOLTIP,NOTIFICATION,COMBO,DND").split(",")
 
+inject_css_overrides()
+
 
 class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     __gsignals__ = {}
@@ -79,6 +83,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def __init__(self):
         GObjectXpraClient.__init__(self)
         UIXpraClient.__init__(self)
+        self.shortcuts_info = None
         self.session_info = None
         self.bug_report = None
         self.file_size_dialog = None
@@ -137,10 +142,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         if HAS_X11_BINDINGS:
             self.setup_frame_request_windows()
         UIXpraClient.run(self)
-        gtk_main_quit_on_fatal_exceptions_enable()
         self.gtk_main()
         log("GTKXpraClient.run_main_loop() main loop ended, returning exit_code=%s", self.exit_code)
-        return  self.exit_code
+        return self.exit_code
 
     def gtk_main(self):
         log("GTKXpraClient.gtk_main() calling %s", Gtk.main)
@@ -171,11 +175,15 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         force_quit()
 
     def exit(self):
-        log("GTKXpraClient.exit() calling %s", gtk_main_quit_really)
-        gtk_main_quit_really()
+        self.show_progress(100, "terminating")
+        log("GTKXpraClient.exit() calling %s", Gtk.main_quit)
+        Gtk.main_quit()
 
     def cleanup(self):
         log("GTKXpraClient.cleanup()")
+        if self.shortcuts_info:
+            self.shortcuts_info.destroy()
+            self.shortcuts_info = None
         if self.session_info:
             self.session_info.destroy()
             self.session_info = None
@@ -205,12 +213,33 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #if server supports it, enable UI thread monitoring workaround when needed:
         def UI_resumed():
             self.send("resume", True, tuple(self._id_to_window.keys()))
+            #maybe the system was suspended?
+            #so we may want to call WindowClient.resume()
+            resume = getattr(self, "resume", None)
+            if resume:
+                resume()
         def UI_failed():
             self.send("suspend", True, tuple(self._id_to_window.keys()))
         self.UI_watcher.add_resume_callback(UI_resumed)
         self.UI_watcher.add_fail_callback(UI_failed)
 
 
+    def get_vrefresh(self):
+        #try via GTK:
+        rates = {}
+        display = Gdk.Display.get_default()
+        for m in range(display.get_n_monitors()):
+            monitor = display.get_monitor(m)
+            log("monitor %i (%s) refresh-rate=%i", m, monitor.get_model(), monitor.get_refresh_rate())
+            rates[m] = monitor.get_refresh_rate()
+        rate = -1
+        if rates:
+            rate = iround(min(rates.values())/1000)
+        if rate<30 or rate>250:
+            rate = super().get_vrefresh()
+        return rate
+
+
     def get_notifier_classes(self):
         #subclasses may add their toolkit specific variants
         #by overriding this method
@@ -236,6 +265,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def do_process_challenge_prompt(self, packet, prompt="password"):
+        self.show_progress(100, "authentication")
         dialog = Gtk.Dialog("Server Authentication",
                None,
                Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT)
@@ -390,11 +420,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         from xpra.client.gtk_base.open_requests import getOpenRequestsWindow
         timeout = self.remote_file_ask_timeout
         def rec_answer(accept, newopenit=openit):
-            if int(accept)==1:
-                #record our response, so we will accept the file
+            from xpra.net.file_transfer import ACCEPT
+            if int(accept)==ACCEPT:
+                #record our response, so we will actually accept the file when the packets arrive:
                 self.data_send_requests[send_id] = (dtype, url, printit, newopenit)
             cb_answer(accept)
-        self.file_ask_dialog = getOpenRequestsWindow()
+        self.file_ask_dialog = getOpenRequestsWindow(self.show_file_upload, self.cancel_download)
         self.file_ask_dialog.add_request(rec_answer, send_id, dtype, url, filesize, printit, openit, timeout)
         self.file_ask_dialog.show()
 
@@ -406,29 +437,36 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def show_ask_data_dialog(self, *_args):
         from xpra.client.gtk_base.open_requests import getOpenRequestsWindow
-        self.file_ask_dialog = getOpenRequestsWindow(self.show_file_upload)
+        self.file_ask_dialog = getOpenRequestsWindow(self.show_file_upload, self.cancel_download)
         self.file_ask_dialog.show()
 
+    def transfer_progress_update(self, send=True, transfer_id=0, elapsed=0, position=0, total=0, error=None):
+        fad = self.file_ask_dialog
+        if fad:
+            self.idle_add(fad.transfer_progress_update, send, transfer_id, elapsed, position, total, error)
+
 
     def accept_data(self, send_id, dtype, url, printit, openit):
         #check if we have accepted this file via the GUI:
         r = self.data_send_requests.pop(send_id, None)
-        if r:
-            edtype = r[0]
-            eurl = r[1]
-            if edtype!=dtype or eurl!=url:
-                filelog.warn("Warning: the file attributes are different")
-                filelog.warn(" from the ones that were used to accept the transfer")
-                s = bytestostr
-                if edtype!=dtype:
-                    filelog.warn(" expected data type '%s' but got '%s'", s(edtype), s(dtype))
-                if eurl!=url:
-                    filelog.warn(" expected data type '%s' but got '%s'", s(eurl), s(url))
-                return None
-            #return the printit and openit flag we got from the UI:
-            return (r[2], r[3])
-        from xpra.net.file_transfer import FileTransferHandler
-        return FileTransferHandler.accept_data(self, send_id, dtype, url, printit, openit)
+        if not r:
+            filelog("accept_data: data send request %s not found", send_id)
+            from xpra.net.file_transfer import FileTransferHandler
+            return FileTransferHandler.accept_data(self, send_id, dtype, url, printit, openit)
+        edtype = r[0]
+        eurl = r[1]
+        if edtype!=dtype or eurl!=url:
+            filelog.warn("Warning: the file attributes are different")
+            filelog.warn(" from the ones that were used to accept the transfer")
+            s = bytestostr
+            if edtype!=dtype:
+                filelog.warn(" expected data type '%s' but got '%s'", s(edtype), s(dtype))
+            if eurl!=url:
+                filelog.warn(" expected url '%s',", s(eurl))
+                filelog.warn("  but got url '%s'", s(url))
+            return None
+        #return the printit and openit flag we got from the UI:
+        return (r[2], r[3])
 
     def file_size_warning(self, action, location, basefilename, filesize, limit):
         if self.file_size_dialog:
@@ -438,8 +476,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         parent = None
         msgs = (
                 "Warning: cannot %s the file '%s'" % (action, basefilename),
-                "this file is too large: %sB" % std_unit(filesize, unit=1024),
-                "the %s file size limit is %iMB" % (location, limit),
+                "this file is too large: %sB" % std_unit(filesize),
+                "the %s file size limit is %iB" % (location, std_unit(limit)),
                 )
         self.file_size_dialog = Gtk.MessageDialog(parent, Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.INFO,
                                                   Gtk.ButtonsType.CLOSE, "\n".join(msgs))
@@ -457,8 +495,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.file_size_dialog = None
             fsd.destroy()
 
-    def download_server_log(self, *_args):
-        self.send_request_file("${XPRA_SERVER_LOG}", self.open_files)
+    def download_server_log(self, callback=None):
+        filename = "${XPRA_SERVER_LOG}"
+        if callback:
+            self.file_request_callback[filename] = callback
+        self.send_request_file(filename, self.open_files)
 
     def send_download_request(self, *_args):
         command = ["xpra", "send-file"]
@@ -526,12 +567,25 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def show_about(self, *_args):
         from xpra.gtk_common.about import about
+        force_focus()
         about()
 
+    def show_shortcuts(self, *_args):
+        if self.shortcuts_info and not self.shortcuts_info.is_closed:
+            force_focus()
+            self.shortcuts_info.present()
+            return
+        from xpra.client.gtk3.show_shortcuts import ShortcutInfo
+        kh = self.keyboard_helper
+        assert kh, "no keyboard helper"
+        self.shortcuts_info = ShortcutInfo(kh.shortcut_modifiers, kh.key_shortcuts)
+        self.shortcuts_info.show_all()
+
     def show_session_info(self, *args):
         if self.session_info and not self.session_info.is_closed:
             #exists already: just raise its window:
             self.session_info.set_args(*args)
+            force_focus()
             self.session_info.present()
             return
         pixbuf = self.get_pixbuf("statistics.png")
@@ -542,11 +596,13 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         from xpra.client.gtk_base.session_info import SessionInfo
         self.session_info = SessionInfo(self, self.session_name, pixbuf, conn, self.get_pixbuf)
         self.session_info.set_args(*args)
+        force_focus()
         self.session_info.show_all()
 
     def show_bug_report(self, *_args):
         self.send_info_request()
         if self.bug_report:
+            force_focus()
             self.bug_report.show()
             return
         from xpra.client.gtk_base.bug_report import BugReport
@@ -566,6 +622,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.bug_report.show()
         #gives the server time to send an info response..
         #(by the time the user clicks on copy, it should have arrived, we hope!)
+        def got_server_log(filename, filesize):
+            log("got_server_log(%s, %s)", filename, filesize)
+            filedata = load_binary_file(filename)
+            self.bug_report.set_server_log_data(filedata)
+        self.download_server_log(got_server_log)
         self.timeout_add(200, init_bug_report)
 
 
@@ -758,14 +819,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     "icons.max_size"    : (128, 128),   #limit
                     })
         capabilities["opengl"] = self.opengl_props
-        from xpra.client import window_backing_base
-        if self._protocol._conn.socktype=="udp":
-            #lossy protocol means we can't use delta regions:
-            log("no delta buckets with udp, since we can drop paint packets")
-            window_backing_base.DELTA_BUCKETS = 0
-        updict(capabilities, "encoding", {
-                    "delta_buckets"     : window_backing_base.DELTA_BUCKETS,
-                    })
         return capabilities
 
 
@@ -920,7 +973,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         except RuntimeError as e:
             log.error("Error: failed to create cursor:")
             log.error(" %s", e)
-            log.error(" using %s of size %ix%i with hotspot at %ix%i", cursor_pixbuf, w, h, x, y)
+            log.error(" Gdk.Cursor.new_from_pixbuf%s", (display, cursor_pixbuf, x, y))
+            log.error(" using size %ix%i with hotspot at %ix%i", w, h, x, y)
             c = None
         return c
 
@@ -972,13 +1026,22 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def _process_raise_window(self, packet):
         wid = packet[1]
         window = self._id_to_window.get(wid)
-        log("going to raise window %s - %s", wid, window)
+        focuslog("going to raise window %s - %s", wid, window)
         if window:
             if window.has_toplevel_focus():
                 log("window already has top level focus")
                 return
             window.present()
 
+    def _process_restack_window(self, packet):
+        wid, detail, other_wid = packet[1:4]
+        above = bool(detail==0)
+        window = self._id_to_window.get(wid)
+        other_window = self._id_to_window.get(other_wid)
+        focuslog("restack window %s - %s %s %s",
+            wid, window, ["below", "above"][above], other_window)
+        if window:
+            window.get_window().restack(other_window, above)  #Above=0
 
     def opengl_setup_failure(self, summary = "Xpra OpenGL Acceleration Failure", body=""):
         def delayed_notify():
@@ -994,13 +1057,13 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #ie: "on:native,gtk", "auto", "no"
         #ie: "probe-failed:SIGSEGV"
         #ie: "probe-success"
-        enable_opengl = (enable_opengl or "").lower()
+        enable_opengl = (enable_opengl or "")
         parts = enable_opengl.split(":", 1)
-        enable_option = parts[0]            #ie: "on"
+        enable_option = parts[0].lower()        #ie: "on"
         opengllog("init_opengl: enable_option=%s", enable_option)
         if enable_option in ("probe-failed", "probe-error", "probe-crash"):
             msg = enable_option.replace("-", " ")
-            if len(parts)>1:
+            if len(parts)>1 and any(len(x) for x in parts[1:]):
                 msg += ": %s" % csv(parts[1:])
             self.opengl_props["info"] = "disabled, %s" % msg
             self.opengl_setup_failure(body=msg)
@@ -1099,7 +1162,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 l("Warning: OpenGL windows will be clamped to the maximum texture size %ix%i",
                   self.gl_texture_size_limit, self.gl_texture_size_limit)
                 l(" for OpenGL %s renderer '%s'", pver(self.opengl_props.get("opengl", "")), self.opengl_props.get("renderer", "unknown"))
-            if self.opengl_enabled and enable_opengl!="probe-success":
+            if self.opengl_enabled and enable_opengl!="probe-success" and not self.opengl_force:
                 draw_result = test_gl_client_window(self.GLClientWindowClass, max_window_size=self.max_window_size, pixel_depth=self.pixel_depth)
                 if not draw_result.get("success", False):
                     err("OpenGL test rendering failed:", draw_result.get("message", "unknown error"))
@@ -1161,6 +1224,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 return False
             if not metadata.boolget("decorations", True):
                 return False
+            hbl = (self.headerbar or "").lower().strip()
+            if hbl not in FALSE_OPTIONS:
+                #any risk that we may end up using headerbar,
+                #means we can't enable opengl
+                return False
         return True
 
     def toggle_opengl(self, *_args):
