@@ -1439,7 +1439,11 @@ class GTKTrayMenuBase(MenuHelper):
         pixbuf = None
         if app_name and not icondata:
             #try to load from our icons:
-            icon_filename = os.path.join(get_icon_dir(), "%s.png" % app_name.decode("utf-8").lower())
+            try:
+                nstr = app_name.decode("utf-8").lower()
+            except UnicodeDecodeError:
+                nstr = bytestostr(app_name).lower()
+            icon_filename = os.path.join(get_icon_dir(), "%s.png" % nstr)
             if os.path.exists(icon_filename):
                 pixbuf = GdkPixbuf.Pixbuf.new_from_file(icon_filename)
         if not pixbuf and icondata:
