@@ -275,22 +275,23 @@ class WindowClient(StubClientMixin):
 
 
     def get_info(self):
-        return {
-            "windows" : {
-                "count"         : len(self._window_to_id),
-                "min-size"      : self.min_window_size,
-                "max-size"      : self.max_window_size,
-                "draw-counter"  : self._draw_counter,
-                "read-only"     : self.readonly,
-                "wheel" : {
-                    "delta-x"   : self.wheel_deltax,
-                    "delta-y"   : self.wheel_deltay,
-                },
-                "focused"       : self._focused or 0,
-                "grabbed"       : self._window_with_grab or 0,
-                "buttons"       : self._button_state,
-            }
+        info = {
+            "count"         : len(self._window_to_id),
+            "min-size"      : self.min_window_size,
+            "max-size"      : self.max_window_size,
+            "draw-counter"  : self._draw_counter,
+            "read-only"     : self.readonly,
+            "wheel" : {
+                "delta-x"   : self.wheel_deltax,
+                "delta-y"   : self.wheel_deltay,
+            },
+            "focused"       : self._focused or 0,
+            "grabbed"       : self._window_with_grab or 0,
+            "buttons"       : self._button_state,
         }
+        for wid, window in tuple(self._id_to_window.items()):
+            info[wid] = window.get_info()
+        return {"windows" : info}
 
 
     ######################################################################
@@ -308,7 +309,7 @@ class WindowClient(StubClientMixin):
             "double_click.distance"     : get_double_click_distance(),
             #features:
             "bell"                      : self.client_supports_bell,
-            "vrefresh"                  : get_vrefresh(),
+            "vrefresh"                  : self.get_vrefresh(),
             "windows"                   : self.windows_enabled,
             "auto_refresh_delay"        : int(self.auto_refresh_delay*1000),
             #system tray forwarding:
@@ -321,11 +322,15 @@ class WindowClient(StubClientMixin):
             })
         return caps
 
+    def get_vrefresh(self):
+        return get_vrefresh()
+
     def get_window_caps(self) -> dict:
         return {
             #implemented in the gtk client:
             "min-size"                  : self.min_window_size,
             "max-size"                  : self.max_window_size,
+            "restack"                   : True,
             }
 
 
@@ -773,7 +778,8 @@ class WindowClient(StubClientMixin):
                 window = cwc(self, group_leader_window, watcher_pid, wid,
                              wx, wy, ww, wh, bw, bh,
                              metadata, override_redirect, client_properties,
-                             border, self.max_window_size, self.default_cursor_data, self.pixel_depth)
+                             border, self.max_window_size, self.default_cursor_data, self.pixel_depth,
+                             self.headerbar)
                 break
             except Exception:
                 log.warn("failed to instantiate %s", cwc, exc_info=True)
@@ -783,7 +789,7 @@ class WindowClient(StubClientMixin):
         log("make_new_window(..) window(%i)=%s", wid, window)
         self._id_to_window[wid] = window
         self._window_to_id[window] = wid
-        window.show()
+        window.show_all()
         if override_redirect:
             if self.should_force_grab(metadata):
                 grablog.warn("forcing grab for OR window %i, matches %s", wid, OR_FORCE_GRAB)
@@ -955,10 +961,9 @@ class WindowClient(StubClientMixin):
         override_redirect = window._override_redirect
         backing = window._backing
         current_icon = window._current_icon
-        delta_pixel_data, video_decoder, csc_decoder, decoder_lock = None, None, None, None
+        video_decoder, csc_decoder, decoder_lock = None, None, None
         try:
             if backing:
-                delta_pixel_data = backing._delta_pixel_data
                 video_decoder = backing._video_decoder
                 csc_decoder = backing._csc_decoder
                 decoder_lock = backing._decoder_lock
@@ -976,14 +981,8 @@ class WindowClient(StubClientMixin):
             #(so it will reset the video encoders, etc)
             if not window.is_OR():
                 self.send("unmap-window", wid)
-            try:
-                del self._id_to_window[wid]
-            except KeyError:
-                pass
-            try:
-                del self._window_to_id[window]
-            except KeyError:
-                pass
+            self._id_to_window.pop(wid, None)
+            self._window_to_id.pop(window, None)
             #create the new window,
             #which should honour the new state of the opengl_enabled flag if that's what we changed,
             #or the new dimensions, etc
@@ -993,7 +992,6 @@ class WindowClient(StubClientMixin):
             #restore the attributes we had saved from it
             if backing:
                 backing = window._backing
-                backing._delta_pixel_data = delta_pixel_data
                 backing._video_decoder = video_decoder
                 backing._csc_decoder = csc_decoder
                 backing._decoder_lock = decoder_lock
@@ -1083,7 +1081,11 @@ class WindowClient(StubClientMixin):
             window.resize(aw, ah, resize_counter)
 
     def _process_raise_window(self, packet):
-        #only implemented in gtk2 for now
+        #implemented in gtk subclass
+        pass
+
+    def _process_restack_window(self, packet):
+        #implemented in gtk subclass
         pass
 
 
@@ -1154,8 +1156,6 @@ class WindowClient(StubClientMixin):
             del self._id_to_window[wid]
             del self._window_to_id[window]
             self.destroy_window(wid, window)
-        if not self._id_to_window:
-            log("last window gone, clearing key repeat")
         self.set_tray_icon()
 
     def may_reenable_modal_windows(self, window):
@@ -1187,13 +1187,8 @@ class WindowClient(StubClientMixin):
                 wids.remove(wid)
                 if not wids:
                     log("last window, removing watcher %s", signalwatcher)
-                    try:
-                        del self._signalwatcher_to_wids[signalwatcher]
-                    except KeyError:
-                        log("destroy_window(%i, %s) error killing signal watcher %s",
-                            wid, window, signalwatcher, exc_info=True)
-                    else:
-                        self.kill_signalwatcher(signalwatcher)
+                    self._signalwatcher_to_wids.pop(signalwatcher, None)
+                    self.kill_signalwatcher(signalwatcher)
                     #now remove any pids that use this watcher:
                     for pid, w in tuple(self._pid_to_signalwatcher.items()):
                         if w==signalwatcher:
@@ -1346,8 +1341,10 @@ class WindowClient(StubClientMixin):
             self.reinit_windows()
         self.reinit_window_icons()
 
-    def control_refresh(self, wid, suspend_resume, refresh, quality=100, options={}, client_properties={}):
+    def control_refresh(self, wid, suspend_resume, refresh, quality=100, options=None, client_properties=None):
         packet = ["buffer-refresh", wid, 0, quality]
+        options = options or {}
+        client_properties = client_properties or {}
         options["refresh-now"] = bool(refresh)
         if suspend_resume is True:
             options["batch"] = {
@@ -1419,10 +1416,11 @@ class WindowClient(StubClientMixin):
                 window.eos()
             return
         x, y, width, height, coding, data, packet_sequence, rowstride = packet[2:10]
+        coding = bytestostr(coding)
         if not window:
             #window is gone
             def draw_cleanup():
-                if bytestostr(coding)==b"mmap":
+                if coding=="mmap":
                     assert self.mmap_enabled
                     from xpra.net.mmap_pipe import int_from_buffer
                     #we need to ack the data to free the space!
@@ -1441,7 +1439,7 @@ class WindowClient(StubClientMixin):
         options = typedict(options)
         dtype = DRAW_TYPES.get(type(data), type(data))
         drawlog("process_draw: %7i %8s for window %3i, sequence %8i, %4ix%-4i at %4i,%-4i using %6s encoding with options=%s",
-                len(data), dtype, wid, packet_sequence, width, height, x, y, bytestostr(coding), options)
+                len(data), dtype, wid, packet_sequence, width, height, x, y, coding, options)
         start = monotonic_time()
         def record_decode_time(success, message=""):
             if success>0:
@@ -1460,7 +1458,7 @@ class WindowClient(StubClientMixin):
                 decode_time = 0
                 paintlog("record_decode_time(%s, %s) decoding or painting skipped on wid=%s, %s: %sx%s",
                          success, message, wid, coding, width, height)
-            self.send_damage_sequence(wid, packet_sequence, width, height, decode_time, str(message))
+            self.send_damage_sequence(wid, packet_sequence, width, height, decode_time, repr_ellipsized(message, 512))
         self._draw_counter += 1
         if PAINT_FAULT_RATE>0 and (self._draw_counter % PAINT_FAULT_RATE)==0:
             drawlog.warn("injecting paint fault for %s draw packet %i, sequence number=%i",
@@ -1534,6 +1532,7 @@ class WindowClient(StubClientMixin):
             "new-override-redirect":self._process_new_override_redirect,
             "new-tray":             self._process_new_tray,
             "raise-window":         self._process_raise_window,
+            "restack-window":       self._process_restack_window,
             "initiate-moveresize":  self._process_initiate_moveresize,
             "window-move-resize":   self._process_window_move_resize,
             "window-resized":       self._process_window_resized,
