@@ -4,15 +4,21 @@
 # later version. See the file COPYING for details.
 
 import os
-import sys
 import curses
 import signal
+import traceback
 from subprocess import Popen, PIPE, DEVNULL
 from datetime import datetime, timedelta
 
-from xpra import __version__
-from xpra.util import typedict, std, envint, csv, engs
-from xpra.os_util import platform_name, bytestostr, monotonic_time, POSIX
+from xpra.version_util import caps_to_version
+from xpra.util import typedict, std, envint, csv, engs, repr_ellipsized
+from xpra.os_util import (
+    platform_name, get_machine_id,
+    bytestostr, monotonic_time,
+    POSIX, SIGNAMES,
+    )
+from xpra.exit_codes import EXIT_STR
+from xpra.make_thread import start_thread
 from xpra.client.gobject_client_base import MonitorXpraClient
 from xpra.gtk_common.gobject_compat import register_os_signals
 from xpra.platform.dotxpra import DotXpra
@@ -24,25 +30,24 @@ from xpra.log import Logger
 log = Logger("gobject", "client")
 
 REFRESH_RATE = envint("XPRA_REFRESH_RATE", 1)
+CURSES_LOG = os.environ.get("XPRA_CURSES_LOG")
 
 WHITE = 0
 GREEN = 1
 YELLOW = 2
 RED = 3
 
-Escape = 27
-EXIT_KEYS = (Escape, ord("q"), ord("Q"))
+EXIT_KEYS = (ord("q"), ord("Q"))
+PAUSE_KEYS = (ord("p"), ord("P"))
+SIGNAL_KEYS = {
+    3 : signal.SIGINT,
+    26 : signal.SIGSTOP,
+    }
+
 
 def get_title():
-    title = "Xpra top %s" % __version__
-    try:
-        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS
-        title += "-r%s" % REVISION
-        if LOCAL_MODIFICATIONS:
-            title += "M"
-    except ImportError:
-        pass
-    return title
+    from xpra.version_util import full_version_str
+    return "Xpra top %s" % full_version_str()
 
 def curses_init():
     stdscr = curses.initscr()
@@ -50,7 +55,7 @@ def curses_init():
     stdscr.nodelay(True)
     stdscr.clear()
     curses.noecho()
-    curses.cbreak()
+    curses.raw()
     curses.start_color()
     curses.use_default_colors()
     #for i in range(0, curses.COLORS):
@@ -62,13 +67,19 @@ def curses_init():
     return stdscr
 
 def curses_clean(stdscr):
+    if not stdscr:
+        return
     stdscr.keypad(False)
     curses.nocbreak()
     curses.echo()
     curses.endwin()
 
 def curses_err(stdscr, e):
-    import traceback
+    if CURSES_LOG:
+        with open(CURSES_LOG, "ab") as f:
+            f.write(b"%s\n" % e)
+            f.write(traceback.format_exc().encode())
+        return
     stdscr.addstr(0, 0, str(e))
     for i, l in enumerate(traceback.format_exc().split("\n")):
         try:
@@ -90,50 +101,61 @@ def box(stdscr, x, y, w, h, ul, ur, ll, lr):
 class TopClient:
 
     def __init__(self, opts):
-        self.stdscr = curses_init()
+        self.stdscr = None
         self.socket_dirs = opts.socket_dirs
         self.socket_dir = opts.socket_dir
         self.position = 0
         self.selected_session = None
+        self.message = None
         self.exit_code = None
-        self.subprocess_exit_code = None
         self.dotxpra = DotXpra(self.socket_dir, self.socket_dirs)
+        self.last_getch = 0
+        self.psprocess = {}
 
     def run(self):
+        self.setup()
         for signum in (signal.SIGINT, signal.SIGTERM):
             signal.signal(signum, self.signal_handler)
         self.update_loop()
         self.cleanup()
         return self.exit_code
 
-    def signal_handler(self, *_args):
-        self.exit_code = 128+signal.SIGINT
+    def signal_handler(self, signum, *_args):
+        self.exit_code = 128+signum
+
+    def setup(self):
+        self.stdscr = curses_init()
+        curses.cbreak()
 
     def cleanup(self):
-        curses_clean(self.stdscr)
+        scr = self.stdscr
+        if scr:
+            curses.nocbreak()
+            scr.erase()
+            curses_clean(scr)
+            self.stdscr = None
 
     def update_loop(self):
         while self.exit_code is None:
             self.update_screen()
-            curses.halfdelay(50)
-            v = self.stdscr.getch()
-            #print("v=%s" % (v,))
+            elapsed = int(1000*monotonic_time()-self.last_getch)
+            delay = max(100, min(1000, 1000-elapsed))//100
+            curses.halfdelay(delay)
+            try:
+                v = self.stdscr.getch()
+            except Exception:
+                v = -1
+            self.last_getch = int(1000*monotonic_time())
             if v in EXIT_KEYS:
                 self.exit_code = 0
-            elif v==258:    #down arrow
+            if v in SIGNAL_KEYS:
+                self.exit_code = 128+SIGNAL_KEYS[v]
+            if v==258:    #down arrow
                 self.position += 1
             elif v==259:    #up arrow
                 self.position = max(self.position-1, 0)
             elif v==10 and self.selected_session:
-                #show this session:
-                cmd = get_nodock_command()+["top", self.selected_session]
-                try:
-                    self.cleanup()
-                    proc = Popen(cmd)
-                    exit_code = proc.wait()
-                    #TODO: show exit code, especially if non-zero
-                finally:
-                    self.stdscr = curses_init()
+                self.show_selected_session()
             elif v in (ord("s"), ord("S")):
                 self.run_subcommand("stop")
             elif v in (ord("a"), ord("A")):
@@ -141,12 +163,40 @@ class TopClient:
             elif v in (ord("d"), ord("D")):
                 self.run_subcommand("detach")
 
+    def show_selected_session(self):
+        #show this session:
+        try:
+            self.cleanup()
+            env = os.environ.copy()
+            #we only deal with local sessions, should be fast:
+            env["XPRA_CONNECT_TIMEOUT"] = "3"
+            proc = self.do_run_subcommand("top", env=env)
+            if not proc:
+                self.message = monotonic_time(), "failed to execute subprocess", curses.color_pair(RED)
+                return
+            exit_code = proc.wait()
+            txt = "top subprocess terminated"
+            attr = 0
+            if exit_code!=0:
+                attr = curses.color_pair(RED)
+                txt += " with error code %i" % exit_code
+                if exit_code in EXIT_STR:
+                    txt += " (%s)" % EXIT_STR.get(exit_code, "").replace("_", " ")
+                elif (exit_code-128) in SIGNAMES:   #pylint: disable=superfluous-parens
+                    txt += " (%s)" % SIGNAMES[exit_code-128]
+            self.message = monotonic_time(), txt, attr
+        finally:
+            self.setup()
+
     def run_subcommand(self, subcommand):
+        return self.do_run_subcommand(subcommand, stdout=DEVNULL, stderr=DEVNULL)
+
+    def do_run_subcommand(self, subcommand, **kwargs):
         cmd = get_nodock_command()+[subcommand, self.selected_session]
         try:
-            Popen(cmd, stdout=DEVNULL, stderr=DEVNULL)
-        except:
-            pass
+            return Popen(cmd, **kwargs)
+        except Exception:
+            return None
 
 
     def update_screen(self):
@@ -182,6 +232,15 @@ class TopClient:
             hpos += 1
             if height<=hpos:
                 return
+            if self.message:
+                ts, txt, attr = self.message
+                if monotonic_time()-ts<10:
+                    self.stdscr.addstr(hpos, 0, txt, attr)
+                    hpos += 1
+                    if height<=hpos:
+                        return
+                else:
+                    self.message = None
             n = len(displays)
             for i, (display, state_paths) in enumerate(displays.items()):
                 if height<=hpos:
@@ -198,6 +257,8 @@ class TopClient:
                 else:
                     attr = 0
                 for s in info:
+                    if len(s)>=width-4:
+                        s = s[:width-6]+".."
                     s = s.ljust(width-4)
                     self.stdscr.addstr(hpos, 2, s, attr)
                     hpos += 1
@@ -234,6 +295,24 @@ class TopClient:
             else:
                 info[0] = "%s  %s" % (display, name)
                 info.insert(1, "uuid=%s, type=%s" % (uuid, stype))
+            machine_id = d.get("machine-id")
+            if machine_id is None or machine_id==get_machine_id():
+                try:
+                    pid = int(d.get("pid"))
+                except (ValueError, TypeError):
+                    pass
+                else:
+                    try:
+                        process = self.psprocess.get(pid)
+                        if not process:
+                            import psutil
+                            process = psutil.Process(pid)
+                            self.psprocess[pid] = process
+                        else:
+                            cpu = process.cpu_percent()
+                            info[0] += ", %i%% CPU" % (cpu)
+                    except Exception:
+                        pass
         return info
 
     def get_display_id_info(self, path):
@@ -274,58 +353,140 @@ class TopSessionClient(MonitorXpraClient):
 
     def __init__(self, *args):
         super().__init__(*args)
+        self.log_file = None
+        if CURSES_LOG:
+            self.log_file = open(CURSES_LOG, "ab")
         self.info_request_pending = False
         self.server_last_info = typedict()
         self.server_last_info_time = 0
         self.info_timer = 0
+        self.paused = False
+        self.stdscr = None
+        self.psprocess = {}
+        self.setup()
+        start_thread(self.input_thread, "input-thread", daemon=True)
+
+    def client_type(self):
+        #overriden in subclasses!
+        return "top"
+
+    def server_connection_established(self, caps):
+        self.log("server_connection_established(%s)" % repr_ellipsized(caps))
+        self.log("traceback: %s" % (traceback.extract_stack(),))
+        r = super().server_connection_established(caps)
         self.stdscr = curses_init()
         self.update_screen()
+        return r
+
+    def setup(self):
+        self.stdscr = curses_init()
+        try:
+            curses.cbreak()
+            curses.halfdelay(10)
+        except Exception as e:
+            self.log("failed to configure curses: %s" % e)
 
     def run(self):
-        register_os_signals(self.signal_handler, "Top Client")
-        return super().run()
-
-    def signal_handler(self, *_args):
+        register_os_signals(self.signal_handler, None)
+        v = super().run()
+        self.log("run()=%s" % v)
         self.cleanup()
-        sys.exit(128+signal.SIGINT)
+        self.close_log()
+        return v
+
+    def signal_handler(self, signum, *args):
+        self.log("exit_code=%s" % self.exit_code)
+        self.log("signal_handler(%s, %s)" % (signum, args,))
+        self.quit(128+signum)
+        self.log("exit_code=%s" % self.exit_code)
 
     def cleanup(self):
         self.cancel_info_timer()
         MonitorXpraClient.cleanup(self)
         curses_clean(self.stdscr)
 
+    def close_log(self):
+        log_file = self.log_file
+        if log_file:
+            self.log("closing log")
+            self.log_file = None
+            log_file.close()
+
+    def log(self, message):
+        if self.log_file:
+            now = datetime.now()
+            self.log_file.write(("%s %s\n" % (now.strftime("%Y/%m/%d %H:%M:%S.%f"), message)).encode())
+            self.log_file.flush()
+
+    def err(self, e):
+        if self.log_file:
+            self.log_file.write(b"%s\n" % e)
+            self.log_file.write(traceback.format_exc().encode())
+        else:
+            curses_err(self.stdscr, e)
+
     def update_screen(self):
-        self.stdscr.erase()
-        try:
-            self.do_update_screen()
-        finally:
-            self.stdscr.refresh()
-        v = self.stdscr.getch()
-        if v in EXIT_KEYS:
-            self.quit(0)
+        self.log("signal handlers=%s" % signal.getsignal(signal.SIGINT))
+        self.log("update_screen()")
+        if not self.paused:
+            self.stdscr.erase()
+            try:
+                self.do_update_screen()
+            except Exception as e:
+                self.err(e)
+            finally:
+                self.stdscr.refresh()
+
+    def input_thread(self):
+        while self.exit_code is None:
+            try:
+                curses.halfdelay(10)
+                v = self.stdscr.getch()
+            except Exception as e:
+                self.log("getch() %s" % e)
+                v = -1
+            self.log("getch()=%s" % v)
+            if v==-1:
+                continue
+            if v in EXIT_KEYS:
+                self.log("exit on key '%s'" % v)
+                self.quit(0)
+                break
+            if v in SIGNAL_KEYS:
+                self.log("exit on signal key '%s'" % v)
+                self.quit(128+SIGNAL_KEYS[v])
+                break
+            if v in PAUSE_KEYS:
+                self.paused = not self.paused
 
     def do_update_screen(self):
+        self.log("do_update_screen()")
         #c = self.stdscr.getch()
         #if c==curses.KEY_RESIZE:
         height, width = self.stdscr.getmaxyx()
         #log.info("update_screen() %ix%i", height, width)
         title = get_title()
-        x = max(0, width//2-len(title)//2)
         sli = self.server_last_info
+        def _addstr(pad, y, x, s, *args):
+            if len(s)+x>=width-pad:
+                s = s[:max(0, width-x-2-pad)]+".."
+            self.stdscr.addstr(y, x, s, *args)
+        def addstr_main(y, x, s, *args):
+            _addstr(0, y, x, s, *args)
+        def addstr_box(y, x, s, *args):
+            _addstr(2, y, x, s, *args)
         try:
-            self.stdscr.addstr(0, x, title, curses.A_BOLD)
+            x = max(0, width//2-len(title)//2)
+            addstr_main(0, x, title, curses.A_BOLD)
             if height<=1:
                 return
             server_info = self.slidictget("server")
-            build = self.slidictget("build")
-            v = build.strget("version")
-            revision = build.strget("revision")
-            if v and revision:
-                v = " version %s-r%s" % (v, revision)
+            build = self.slidictget("server", "build")
+            vstr = caps_to_version(build)
             mode = server_info.strget("mode", "server")
             python_info = typedict(server_info.dictget("python", {}))
             bits = python_info.intget("bits", 32)
-            server_str = "Xpra %s server%s %i-bit" % (mode, std(v), bits)
+            server_str = "Xpra %s server version %s %i-bit" % (mode, vstr, bits)
             proxy_info = self.slidictget("proxy")
             if proxy_info:
                 proxy_platform_info = typedict(proxy_info.dictget("platform", {}))
@@ -338,7 +499,7 @@ class TopSessionClient(MonitorXpraClient):
                     platform_name(proxy_platform, proxy_distro or proxy_release),
                     std(proxy_version or "unknown")
                     )
-            self.stdscr.addstr(1, 0, server_str)
+            addstr_main(1, 0, server_str)
             if height<=2:
                 return
             #load and uptime:
@@ -355,12 +516,28 @@ class TopSessionClient(MonitorXpraClient):
             if load and len(load)==3:
                 float_load = tuple(v/1000.0 for v in load)
                 load_average = ", load average: %1.2f, %1.2f, %1.2f" % float_load
-            self.stdscr.addstr(2, 0, "xpra top - %s%s, %2i users%s" % (
+            addstr_main(2, 0, "xpra top - %s%s, %2i users%s" % (
                                now.strftime("%H:%M:%S"), uptime, nclients, load_average))
             if height<=3:
                 return
             thread_info = self.slidictget("threads")
             rinfo = "%i threads" % thread_info.intget("count")
+            server_pid = server_info.intget("pid", 0)
+            if server_pid:
+                rinfo += ", pid %i" % server_pid
+                machine_id = server_info.get("machine-id")
+                if machine_id is None or machine_id==get_machine_id():
+                    try:
+                        process = self.psprocess.get(server_pid)
+                        if not process:
+                            import psutil
+                            process = psutil.Process(server_pid)
+                            self.psprocess[server_pid] = process
+                        else:
+                            cpu = process.cpu_percent()
+                            rinfo += ", %i%% CPU" % (cpu)
+                    except Exception:
+                        pass
             cpuinfo = self.slidictget("cpuinfo")
             if cpuinfo:
                 rinfo += ", %s" % cpuinfo.strget("hz_actual")
@@ -371,7 +548,7 @@ class TopSessionClient(MonitorXpraClient):
             elif elapsed>2:
                 rinfo += " - last updated %i seconds ago" % elapsed
                 color = RED
-            self.stdscr.addstr(3, 0, rinfo, curses.color_pair(color))
+            addstr_main(3, 0, rinfo, curses.color_pair(color))
             if height<=4:
                 return
             #display:
@@ -388,21 +565,25 @@ class TopSessionClient(MonitorXpraClient):
             if cursor_info:
                 cx, cy = cursor_info.inttupleget("position", (0, 0))
                 dinfo.append("cursor at %ix%i" % (cx, cy))
-            self.stdscr.addstr(4, 0, csv(dinfo))
+            display_info = self.slidictget("display")
+            pid = display_info.intget("pid")
+            if pid:
+                dinfo.append("pid %i" % pid)
+            addstr_main(4, 0, csv(dinfo))
             if height<=5:
                 return
             hpos = 5
-            gl_info = self.get_gl_info(self.slidictget("opengl"))
+            gl_info = self.get_gl_info(display_info.dictget("opengl"))
             if gl_info:
-                self.stdscr.addstr(5, 0, gl_info)
+                addstr_main(5, 0, gl_info)
                 hpos += 1
 
             if hpos<height-3:
                 hpos += 1
                 if nclients==0:
-                    self.stdscr.addstr(hpos, 0, "no clients connected")
+                    addstr_main(hpos, 0, "no clients connected")
                 else:
-                    self.stdscr.addstr(hpos, 0, "%i client%s connected:" % (nclients, engs(nclients)))
+                    addstr_main(hpos, 0, "%i client%s connected:" % (nclients, engs(nclients)))
                 hpos += 1
             client_info = self.slidictget("client")
             client_no = 0
@@ -425,19 +606,19 @@ class TopSessionClient(MonitorXpraClient):
                 if hpos+2+l>height:
                     if hpos<height:
                         more = nclients-client_no
-                        self.stdscr.addstr(hpos, 0, "%i client%s not shown" % (more, engs(more)), curses.A_BOLD)
+                        addstr_box(hpos, 0, "%i client%s not shown" % (more, engs(more)), curses.A_BOLD)
                     break
                 self.box(1, hpos, width-2, 2+l)
                 for i, info in enumerate(ci):
                     info_text, color = info
                     cpair = curses.color_pair(color)
-                    self.stdscr.addstr(hpos+i+1, 2, info_text, cpair)
+                    addstr_box(hpos+i+1, 2, info_text, cpair)
                 hpos += 2+l
 
             windows = self.slidictget("windows")
             if hpos<height-3:
                 hpos += 1
-                self.stdscr.addstr(hpos, 0, "%i window%s:" % (len(windows), engs(windows)))
+                addstr_main(hpos, 0, "%i window%s:" % (len(windows), engs(windows)))
                 hpos += 1
             wins = tuple(windows.values())
             nwindows = len(wins)
@@ -447,17 +628,17 @@ class TopSessionClient(MonitorXpraClient):
                 if hpos+2+l>height:
                     if hpos<height:
                         more = nwindows-win_no
-                        self.stdscr.addstr(hpos, 0, "terminal window is too small: %i window%s not shown" % \
+                        addstr_main(hpos, 0, "terminal window is too small: %i window%s not shown" % \
                                            (more, engs(more)), curses.A_BOLD)
                     break
                 self.box(1, hpos, width-2, 2+l)
                 for i, info in enumerate(wi):
                     info_text, color = info
                     cpair = curses.color_pair(color)
-                    self.stdscr.addstr(hpos+i+1, 2, info_text, cpair)
+                    addstr_box(hpos+i+1, 2, info_text, cpair)
                 hpos += 2+l
         except Exception as e:
-            curses_err(self.stdscr, e)
+            self.err(e)
 
     def slidictget(self, *parts):
         return self.dictget(self.server_last_info, *parts)
@@ -470,7 +651,8 @@ class TopSessionClient(MonitorXpraClient):
     def get_client_info(self, ci):
         #version info:
         ctype = ci.strget("type", "unknown")
-        title = "%s client version %s-r%s" % (ctype, ci.strget("version"), ci.strget("revision"))
+        title = "%s client version" % ctype
+        title += caps_to_version(ci)
         chost = ci.strget("hostname")
         conn_info = ""
         if chost:
@@ -481,7 +663,12 @@ class TopSessionClient(MonitorXpraClient):
             conn_info += "using %s %s" % (cinfo.strget("type"), cinfo.strget("protocol-type"))
             conn_info += ", with %s and %s" % (cinfo.strget("encoder"), cinfo.strget("compressor"))
         gl_info = self.get_gl_info(ci.dictget("opengl"))
-        #batch delay:
+        #audio
+        audio_info = []
+        for mode in ("speaker", "microphone"):
+            audio_info.append(self._audio_info(ci, mode))
+        audio_info.append(self._avsync_info(ci))
+        #batch delay / latency:
         b_info = typedict(ci.dictget("batch", {}))
         bi_info = typedict(b_info.dictget("delay", {}))
         bcur = bi_info.intget("cur")
@@ -490,34 +677,44 @@ class TopSessionClient(MonitorXpraClient):
             bcur,
             bavg,
             )
-        bcolor = GREEN
-        if bcur>50:
-            bcolor = YELLOW
-        elif bcur>100:
-            bcolor = RED
         #client latency:
         pl = self.slidictget("connection", "client", "ping_latency")
         lcur = pl.intget("cur")
         lavg = pl.intget("avg")
         lmin = pl.intget("min")
-        latency_info = "latency: %i (%i)" % (lcur, lavg)
-        lcolor = GREEN
-        if lcur>20:
-            if lcur>2*lmin:
-                lcolor = YELLOW
-            elif lcur>3*lmin:
-                lcolor = RED
-        audio_info = []
-        for mode in ("speaker", "microphone"):
-            audio_info.append(self._audio_info(ci, mode))
-        audio_info.append(self._avsync_info(ci))
+        bl_color = GREEN
+        if bcur>50 or (lcur>20 and lcur>2*lmin):
+            bl_color = YELLOW
+        elif bcur>100 or (lcur>20 and lcur>3*lmin):
+            bl_color = RED
+        batch_latency = batch_info.ljust(24)+"latency: %i (%i)" % (lcur, lavg)
+        #speed / quality:
+        edict = typedict(ci.dictget("encoding") or {})
+        qs_info = ""
+        qs_color = GREEN
+        if edict:
+            sinfo = typedict(edict.dictget("speed") or {})
+            if sinfo:
+                cur = sinfo.intget("cur")
+                avg = sinfo.intget("avg")
+                qs_info = "speed: %s%% (avg: %s%%)" % (cur, avg)
+            qinfo = typedict(edict.dictget("quality") or {})
+            if qinfo:
+                qs_info = qs_info.ljust(24)
+                cur = qinfo.intget("cur")
+                avg = qinfo.intget("avg")
+                qs_info += "quality: %s%% (avg: %s%%)" % (cur, avg)
+                if avg<70:
+                    qs_color = YELLOW
+                if avg<50:
+                    qs_color = RED
         return tuple((s, c) for s,c in (
             (title, WHITE),
             (conn_info, WHITE),
             (gl_info, WHITE),
             (csv(audio_info), WHITE),
-            (batch_info, bcolor),
-            (latency_info, lcolor),
+            (batch_latency, bl_color),
+            (qs_info, qs_color),
             ) if s)
 
     def _audio_info(self, ci, mode="speaker"):
@@ -557,7 +754,13 @@ class TopSessionClient(MonitorXpraClient):
                     v = GRAVITY_STR.get(v, v)
                 return "%s=%s" % (k, str(v))
             g_str += " - %s" % csv(sc_str(k, v) for k,v in sc.items())
+        line1 = ""
+        pid = wi.intget("pid", 0)
+        if pid:
+            line1 = "pid %i: " % pid
         title = wi.strget("title", "")
+        if title:
+            line1 += ' "%s"' % title
         attrs = [
             x for x in (
                 "above", "below", "bypass-compositor",
@@ -571,7 +774,10 @@ class TopSessionClient(MonitorXpraClient):
             attrs.insert(0, "hidden")
         wtype = wi.strtupleget("window-type", ("NORMAL",))
         tinfo = " - ".join(csv(x) for x in (wtype, attrs) if x)
-        info = (title, g_str, tinfo)
+        info = []
+        if line1:
+            info.append(line1)
+        info += [g_str, tinfo]
         return tuple((x, WHITE) for x in info if x)
 
     def get_gl_info(self, gli):
@@ -587,7 +793,7 @@ class TopSessionClient(MonitorXpraClient):
             if isinstance(v, (tuple, list)):
                 return sep.join(bytestostr(x) for x in v)
             return bytestostr(v)
-        if not gli.boolget("enabled", False):
+        if not gli.boolget("enabled", True):
             return "OpenGL disabled %s" % gli.strget("message", "")
         gl_info = "OpenGL %s enabled: %s" % (strget("opengl", "."), gli.strget("renderer") or gli.strget("vendor"))
         depth = gli.intget("depth")
@@ -609,6 +815,7 @@ class TopSessionClient(MonitorXpraClient):
         self.timeout_add(REFRESH_RATE*1000, self.send_info_request)
 
     def send_info_request(self):
+        self.log("send_info_request()")
         categories = ()
         if not self.info_request_pending:
             self.info_request_pending = True
@@ -623,10 +830,12 @@ class TopSessionClient(MonitorXpraClient):
         self.add_packet_handler("info-response", self._process_info_response, False)
 
     def _process_server_event(self, packet):
+        self.log("server event: %s" % (packet,))
         self.last_server_event = packet[1:]
         self.update_screen()
 
     def _process_info_response(self, packet):
+        self.log("info response: %s" % repr_ellipsized(packet))
         self.cancel_info_timer()
         self.info_request_pending = False
         self.server_last_info = typedict(packet[1])
@@ -641,5 +850,6 @@ class TopSessionClient(MonitorXpraClient):
             self.source_remove(it)
 
     def info_timeout(self):
+        self.log("info timeout")
         self.update_screen()
         return True
