@@ -7,6 +7,7 @@
 import os
 import struct
 import re
+from io import BytesIO
 from gi.repository import GLib
 
 from xpra.net.compression import Compressible
@@ -83,8 +84,9 @@ def must_discard_extra(target):
 
 
 def _filter_targets(targets):
-    f = tuple(target for target in (bytestostr(x) for x in targets) if not must_discard(target))
-    log("_filter_targets(%s)=%s", targets, f)
+    targets_strs = tuple(bytestostr(x) for x in targets)
+    f = tuple(target for target in targets_strs if not must_discard(target))
+    log("_filter_targets(%s)=%s", csv(targets_strs), f)
     return f
 
 #CARD32 can actually be 64-bits...
@@ -620,24 +622,16 @@ class ClipboardProxyCore:
             self._have_token, self._greedy_client, self._block_owner_change)
         if not self._enabled or self._block_owner_change:
             return
-        if self._have_token or (self._greedy_client and self._can_send):
-            if self._have_token or DELAY_SEND_TOKEN<0:
-                #token ownership will change or told not to wait
-                GLib.idle_add(self.emit_token)
-            elif not self._emit_token_timer:
-                #we had it already, this can wait:
-                #TODO: don't throttle clients without "want-targets" attribute
-                # (sending the token is only expensive for those)
-                self.schedule_emit_token()
+        if self._have_token or ((self._greedy_client or self._want_targets) and self._can_send):
+            self.schedule_emit_token()
 
     def schedule_emit_token(self):
-        if self._have_token or DELAY_SEND_TOKEN<0:
+        if self._have_token or (not self._want_targets and not self._greedy_client) or DELAY_SEND_TOKEN<0:
             #token ownership will change or told not to wait
             GLib.idle_add(self.emit_token)
         elif not self._emit_token_timer:
-            #we had it already, this can wait:
-            #TODO: don't throttle clients without "want-targets" attribute
-            # (sending the token is only expensive for those)
+            #we already had sent the token,
+            #or sending it is expensive, so wait a bit:
             self.do_schedule_emit_token()
 
     def do_schedule_emit_token(self):
@@ -692,3 +686,55 @@ class ClipboardProxyCore:
     # contents of this clipboard:
     def get_contents(self, target, cb):
         pass
+
+
+    def filter_data(self, dtype=None, dformat=None, data=None, trusted=False, output_dtype=None):
+        log("filter_data(%s, %s, %i %s, %s, %s)",
+            dtype, dformat, len(data), type(data), trusted, output_dtype)
+        if not data:
+            return data
+        IMAGE_OVERLAY = os.environ.get("XPRA_CLIPBOARD_IMAGE_OVERLAY", None)
+        if IMAGE_OVERLAY and not os.path.exists(IMAGE_OVERLAY):
+            IMAGE_OVERLAY = None
+        IMAGE_STAMP = envbool("XPRA_CLIPBOARD_IMAGE_STAMP", False)
+        SANITIZE_IMAGES = envbool("XPRA_SANITIZE_IMAGES", True)
+        if dtype in ("image/png", "image/jpeg", "image/tiff") and (
+            (output_dtype is not None and dtype!=output_dtype) or
+            IMAGE_STAMP or
+            IMAGE_OVERLAY or
+            (SANITIZE_IMAGES and not trusted)
+            ):
+            from xpra.codecs.pillow.decoder import open_only
+            img_type = dtype.split("/")[-1]
+            img = open_only(data, (img_type, ))
+            has_alpha = img.mode=="RGBA"
+            if not has_alpha and IMAGE_OVERLAY:
+                img = img.convert("RGBA")
+            w, h = img.size
+            if IMAGE_OVERLAY:
+                from PIL import Image   #@UnresolvedImport
+                overlay = Image.open(IMAGE_OVERLAY)
+                if overlay.mode!="RGBA":
+                    log.warn("Warning: cannot use overlay image '%s'", IMAGE_OVERLAY)
+                    log.warn(" invalid mode '%s'", overlay.mode)
+                else:
+                    log("adding clipboard image overlay to %s", dtype)
+                    overlay_resized = overlay.resize((w, h), Image.ANTIALIAS)
+                    composite = Image.alpha_composite(img, overlay_resized)
+                    if not has_alpha and img.mode=="RGBA":
+                        composite = composite.convert("RGB")
+                    img = composite
+            if IMAGE_STAMP:
+                log("adding clipboard image stamp to %s", dtype)
+                from datetime import datetime
+                from PIL import ImageDraw
+                img_draw = ImageDraw.Draw(img)
+                w, h = img.size
+                img_draw.text((10, max(0, h//2-16)), 'via Xpra, %s' % datetime.now().isoformat(), fill='black')
+            #now save it:
+            img_type = (output_dtype or dtype).split("/")[-1]
+            buf = BytesIO()
+            img.save(buf, img_type.upper()) #ie: "PNG"
+            data = buf.getvalue()
+            buf.close()
+        return data
