@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2018-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,15 +8,15 @@ import sys
 import os.path
 import subprocess
 
-from gi.repository import GLib, Pango, Gtk, Gdk, Gio, GdkPixbuf
+from gi.repository import GLib, Pango, Gtk, Gdk, Gio
 
 from xpra.gtk_common.gobject_compat import register_os_signals
 from xpra.gtk_common.gtk_util import (
     add_close_accel,
     add_window_accel, imagebutton,
-    scaled_image,
+    scaled_image, get_icon_pixbuf,
     )
-from xpra.platform.paths import get_icon_dir, get_xpra_command
+from xpra.platform.paths import get_xpra_command
 from xpra.os_util import OSX, WIN32, platform_name
 from xpra.log import Logger
 from xpra.gtk_common.about import about
@@ -51,12 +51,6 @@ def exec_command(cmd):
     log("exec_command(%s)=%s", cmd, proc)
     return proc
 
-def get_pixbuf(icon_name):
-    icon_filename = os.path.join(get_icon_dir(), icon_name)
-    if os.path.exists(icon_filename):
-        return GdkPixbuf.Pixbuf.new_from_file(icon_filename)
-    return None
-
 
 class GUI(Gtk.Window):
 
@@ -90,7 +84,7 @@ class GUI(Gtk.Window):
         self.set_resizable(True)
         self.set_decorated(True)
         self.set_position(Gtk.WindowPosition.CENTER)
-        icon = get_pixbuf("xpra")
+        icon = get_icon_pixbuf("xpra.png")
         if icon:
             self.set_icon(icon)
         add_close_accel(self, self.quit)
@@ -108,18 +102,18 @@ class GUI(Gtk.Window):
         self.widgets = []
         label_font = Pango.FontDescription("sans 16")
         if has_client:
-            icon = get_pixbuf("browse.png")
+            icon = get_icon_pixbuf("browse.png")
             self.browse_button = imagebutton("Browse", icon,
                                              "Browse and connect to local sessions", clicked_callback=self.browse,
                                              icon_size=48, label_font=label_font)
             self.widgets.append(self.browse_button)
-            icon = get_pixbuf("connect.png")
+            icon = get_icon_pixbuf("connect.png")
             self.connect_button = imagebutton("Connect", icon,
                                               "Connect to a session", clicked_callback=self.show_launcher,
                                               icon_size=48, label_font=label_font)
             self.widgets.append(self.connect_button)
         if has_server:
-            icon = get_pixbuf("server-connected.png")
+            icon = get_icon_pixbuf("server-connected.png")
             self.shadow_button = imagebutton("Shadow", icon,
                                              "Start a shadow server", clicked_callback=self.start_shadow,
                                              icon_size=48, label_font=label_font)
@@ -127,7 +121,7 @@ class GUI(Gtk.Window):
                 self.shadow_button.set_tooltip_text("This build of Xpra does not support starting sessions")
                 self.shadow_button.set_sensitive(False)
             self.widgets.append(self.shadow_button)
-            icon = get_pixbuf("windows.png")
+            icon = get_icon_pixbuf("windows.png")
             self.start_button = imagebutton("Start", icon,
                                             "Start a session", clicked_callback=self.start,
                                             icon_size=48, label_font=label_font)
@@ -225,6 +219,7 @@ class GUI(Gtk.Window):
     def start(self, *_args):
         if not self.start_session:
             self.start_session = StartSession()
+        self.start_session.populate_menus()
         self.start_session.show()
         self.start_session.present()
 
@@ -264,11 +259,11 @@ class StartSession(Gtk.Window):
         self.set_title("Start Xpra Session")
         self.set_position(Gtk.WindowPosition.CENTER)
         self.set_size_request(640, 300)
-        icon = get_pixbuf("xpra")
+        icon = get_icon_pixbuf("xpra.png")
         if icon:
             self.set_icon(icon)
-        self.connect("delete-event", self.close)
-        add_close_accel(self, self.close)
+        self.connect("delete-event", self.hide_window)
+        add_close_accel(self, self.hide_window)
 
         vbox = Gtk.VBox(False, 0)
         vbox.set_spacing(0)
@@ -347,13 +342,12 @@ class StartSession(Gtk.Window):
             btn = Gtk.Button(label)
             btn.set_tooltip_text(tooltip)
             btn.connect("clicked", callback)
-            if icon_name:
-                icon = get_pixbuf(icon_name)
-                if icon:
-                    btn.set_image(scaled_image(icon, 24))
+            icon = get_icon_pixbuf(icon_name)
+            if icon:
+                btn.set_image(scaled_image(icon, 24))
             hbox.pack_start(btn)
             return btn
-        self.cancel_btn = btn("Cancel", "", self.close, "quit.png")
+        self.cancel_btn = btn("Cancel", "", self.hide_window, "quit.png")
         self.run_btn = btn("Start", "Start this command in an xpra session", self.run_command, "forward.png")
         self.run_btn.set_sensitive(False)
 
@@ -361,11 +355,6 @@ class StartSession(Gtk.Window):
         self.add(vbox)
 
 
-    def show(self):
-        Gtk.Window.show(self)
-        self.populate_menus()
-
-
     def populate_menus(self):
         if xdg:
             self.entry_al.hide()
@@ -465,8 +454,8 @@ class StartSession(Gtk.Window):
         self.populate_menus()
 
 
-    def close(self, *args):
-        log("close%s", args)
+    def hide_window(self, *args):
+        log("hide_window%s", args)
         self.hide()
         return True
 
@@ -502,8 +491,6 @@ def main(): # pragma: no cover
     from xpra.platform import program_context
     from xpra.log import enable_color
     from xpra.platform.gui import init, ready
-    from xpra.gtk_common.quit import gtk_main_quit_on_fatal_exceptions_enable
-    gtk_main_quit_on_fatal_exceptions_enable()
     with program_context("xpra-gui", "Xpra GUI"):
         enable_color()
         init()
