@@ -13,10 +13,13 @@ from socket import error as socket_error
 from threading import Lock, Event
 from queue import Queue
 
-from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, hexstr
-from xpra.util import repr_ellipsized, ellipsizer, csv, envint, envbool, typedict, nonl
+from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, hexstr, monotonic_time
+from xpra.util import repr_ellipsized, ellipsizer, csv, envint, envbool, typedict
 from xpra.make_thread import make_thread, start_thread
-from xpra.net.common import ConnectionClosedException,may_log_packet    #@UndefinedVariable (pydev false positive)
+from xpra.net.common import (
+    ConnectionClosedException, may_log_packet,
+    MAX_PACKET_SIZE, FLUSH_HEADER,
+    )
 from xpra.net.bytestreams import ABORT
 from xpra.net import compression
 from xpra.net.compression import (
@@ -24,12 +27,12 @@ from xpra.net.compression import (
     InvalidCompressionException, Compressed, LevelCompressed, Compressible, LargeStructure,
     )
 from xpra.net import packet_encoding
-from xpra.net.socket_util import guess_header_protocol
+from xpra.net.socket_util import guess_packet_type
 from xpra.net.packet_encoding import (
     decode, sanity_checks as packet_encoding_sanity_checks,
     InvalidPacketEncodingException,
     )
-from xpra.net.header import unpack_header, pack_header, FLAGS_CIPHER, FLAGS_NOHEADER, HEADER_SIZE
+from xpra.net.header import unpack_header, pack_header, FLAGS_CIPHER, FLAGS_NOHEADER, FLAGS_FLUSH, HEADER_SIZE
 from xpra.net.crypto import get_encryptor, get_decryptor, pad, INITIAL_PADDING
 from xpra.log import Logger
 
@@ -132,6 +135,7 @@ class Protocol:
         """
         assert scheduler is not None
         assert conn is not None
+        self.start_time = monotonic_time()
         self.timeout_add = scheduler.timeout_add
         self.idle_add = scheduler.idle_add
         self.source_remove = scheduler.source_remove
@@ -160,17 +164,18 @@ class Protocol:
         self.output_packetcount = 0
         self.output_raw_packetcount = 0
         #initial value which may get increased by client/server after handshake:
-        self.max_packet_size = 4*1024*1024
+        self.max_packet_size = MAX_PACKET_SIZE
         self.abs_max_packet_size = 256*1024*1024
-        self.large_packets = [b"hello", b"window-metadata", b"sound-data", b"notify_show", b"setting-change"]
+        self.large_packets = [b"hello", b"window-metadata", b"sound-data", b"notify_show", b"setting-change", b"shell-reply"]
         self.send_aliases = {}
+        self.send_flush_flag = False
         self.receive_aliases = {}
         self._log_stats = None          #None here means auto-detect
         self._closed = False
         self.encoder = "none"
-        self._encoder = self.noencode
+        self._encoder = packet_encoding.get_encoder("none")
         self.compressor = "none"
-        self._compress = compression.nocompress
+        self._compress = compression.get_compressor("none")
         self.compression_level = 0
         self.cipher_in = None
         self.cipher_in_name = None
@@ -215,12 +220,12 @@ class Protocol:
     def wait_for_io_threads_exit(self, timeout=None):
         io_threads = [x for x in (self._read_thread, self._write_thread) if x is not None]
         for t in io_threads:
-            if t.isAlive():
+            if t.is_alive():
                 t.join(timeout)
         exited = True
         cinfo = self._conn or "cleared connection"
         for t in io_threads:
-            if t.isAlive():
+            if t.is_alive():
                 log.warn("Warning: %s thread of %s is still alive (timeout=%s)", t.name, cinfo, timeout)
                 exited = False
         return exited
@@ -263,6 +268,8 @@ class Protocol:
     def parse_remote_caps(self, caps : typedict):
         for k,v in caps.dictget("aliases", {}).items():
             self.send_aliases[bytestostr(k)] = v
+        if FLUSH_HEADER:
+            self.send_flush_flag = caps.boolget("flush", False)
 
     def get_info(self, alias_info=True) -> dict:
         info = {
@@ -270,25 +277,23 @@ class Protocol:
             "compression_level"     : self.compression_level,
             "max_packet_size"       : self.max_packet_size,
             "aliases"               : USE_ALIASES,
+            "flush"                 : self.send_flush_flag,
             }
-        c = self._compress
+        c = self.compressor
         if c:
-            info["compressor"] = compression.get_compressor_name(self._compress)
-        e = self._encoder
+            info["compressor"] = c
+        e = self.encoder
         if e:
-            if self._encoder==self.noencode:        #pylint: disable=comparison-with-callable
-                info["encoder"] = "noencode"
-            else:
-                info["encoder"] = packet_encoding.get_encoder_name(self._encoder)
+            info["encoder"] = e
         if alias_info:
             info["send_alias"] = self.send_aliases
             info["receive_alias"] = self.receive_aliases
         c = self._conn
         if c:
             try:
-                info.update(self._conn.get_info())
+                info.update(c.get_info())
             except Exception:
-                log.error("error collecting connection information on %s", self._conn, exc_info=True)
+                log.error("error collecting connection information on %s", c, exc_info=True)
         #add stats to connection info:
         info.setdefault("input", {}).update({
                        "buffer-size"            : self.read_buffer_size,
@@ -356,12 +361,12 @@ class Protocol:
         shm.set()
         #start the format thread:
         if not self._write_format_thread and not self._closed:
-            self._write_format_thread = make_thread(self._write_format_thread_loop, "format", daemon=True)
+            self._write_format_thread = make_thread(self.write_format_thread_loop, "format", daemon=True)
             self._write_format_thread.start()
         #from now on, take shortcut:
         self.source_has_more = self._source_has_more.set
 
-    def _write_format_thread_loop(self):
+    def write_format_thread_loop(self):
         log("write_format_thread_loop starting")
         try:
             while not self._closed:
@@ -422,9 +427,12 @@ class Protocol:
                 log("sending %s bytes without header", payload_size)
                 items.append(data)
             else:
+                #if the other end can use this flag, expose it:
+                if self.send_flush_flag and not more and index==0:
+                    proto_flags |= FLAGS_FLUSH
                 #the xpra packet header:
                 #(WebSocketProtocol may also add a websocket header too)
-                header = self.make_chunk_header(packet_type, proto_flags, level, index, payload_size, actual_size)
+                header = self.make_chunk_header(packet_type, proto_flags, level, index, payload_size)
                 if actual_size<PACKET_JOIN_SIZE:
                     if not isinstance(data, bytes):
                         data = memoryview_to_bytes(data)
@@ -444,7 +452,7 @@ class Protocol:
                 items.insert(0, frame_header)
         self.raw_write(packet_type, items, start_send_cb, end_send_cb, fail_cb, synchronous, more)
 
-    def make_xpra_header(self, _packet_type, proto_flags, level, index, payload_size, actual_size) -> bytes:
+    def make_xpra_header(self, _packet_type, proto_flags, level, index, payload_size) -> bytes:
         return pack_header(proto_flags, level, index, payload_size)
 
     def noframe_header(self, _packet_type, _items):
@@ -457,6 +465,7 @@ class Protocol:
     def raw_write(self, packet_type, items, start_cb=None, end_cb=None, fail_cb=None, synchronous=True, more=False):
         """ Warning: this bypasses the compression and packet encoder! """
         if self._write_thread is None:
+            log("raw_write for %s, starting write thread", packet_type)
             self.start_write_thread()
         self._write_queue.put((items, start_cb, end_cb, fail_cb, synchronous, more))
 
@@ -494,12 +503,15 @@ class Protocol:
             self.enable_compressor("none")
             return
         opts = compression.get_enabled_compressors(order=compression.PERFORMANCE_ORDER)
+        compressors = caps.strtupleget("compressors")
         log("enable_compressor_from_caps(..) options=%s", opts)
         for c in opts:      #ie: [zlib, lz4, lzo]
-            if caps.boolget(c):
+            if c=="none":
+                continue
+            if c in compressors or caps.boolget(c):
                 self.enable_compressor(c)
                 return
-        log.warn("compression disabled: no matching compressor found")
+        log.warn("Warning: compression disabled, no matching compressor found")
         self.enable_compressor("none")
 
     def enable_compressor(self, compressor):
@@ -508,16 +520,6 @@ class Protocol:
         log("enable_compressor(%s): %s", compressor, self._compress)
 
 
-    def noencode(self, data):
-        #just send data as a string for clients that don't understand xpra packet format:
-        import codecs
-        def b(x):
-            if isinstance(x, bytes):
-                return x
-            return codecs.latin_1_encode(x)[0]
-        return b(": ".join(str(x) for x in data)+"\n"), FLAGS_NOHEADER
-
-
     def encode(self, packet_in):
         """
         Given a packet (tuple or list of items), converts it for the wire.
@@ -547,13 +549,11 @@ class Protocol:
             try:
                 l = len(item)
             except TypeError as e:
-                raise TypeError("invalid type %s in %s packet at index %s: %s" % (ti, packet[0], i, e))
+                raise TypeError("invalid type %s in %s packet at index %s: %s" % (ti, packet[0], i, e)) from None
             if ti==LargeStructure:
-                item = item.data
-                packet[i] = item
-                ti = type(item)
+                packet[i] = item.data
                 continue
-            elif ti==Compressible:
+            if ti==Compressible:
                 #this is a marker used to tell us we should compress it now
                 #(used by the client for clipboard data)
                 item = item.compress()
@@ -576,7 +576,7 @@ class Protocol:
                     packet[i] = item.data
                     min_comp_size += l
                     size_check += l
-            elif ti in (str, bytes) and level>0 and l>LARGE_PACKET_SIZE:
+            elif ti==bytes and level>0 and l>LARGE_PACKET_SIZE:
                 log.warn("Warning: found a large uncompressed item")
                 log.warn(" in packet '%s' at position %i: %s bytes", packet[0], i, len(item))
                 #add new binary packet with large item:
@@ -767,11 +767,11 @@ class Protocol:
     def _invalid_header(self, proto, data, msg=""):
         log("invalid_header(%s, %s bytes: '%s', %s)",
                proto, len(data or ""), msg, ellipsizer(data))
-        guess = guess_header_protocol(data)
-        if guess[0]:
-            err = "invalid packet format, %s" % guess[1]
+        guess = guess_packet_type(data)
+        if guess:
+            err = "invalid packet format: %s" % guess
         else:
-            err = "%s: '%s'" % (msg, hexstr(data[:HEADER_SIZE]))
+            err = "%s: 0x%s" % (msg, hexstr(data[:HEADER_SIZE]))
             if len(data)>1:
                 err += " read buffer=%s (%i bytes)" % (repr_ellipsized(data), len(data))
         self.gibberish(err, data)
@@ -837,7 +837,7 @@ class Protocol:
                     #try to handle the first buffer:
                     buf = read_buffers[0]
                     if not header and buf[0]!=ord("P"):
-                        self.invalid_header(self, buf, "invalid packet header byte %s" % nonl(bytestostr(buf)))
+                        self.invalid_header(self, buf, "invalid packet header byte")
                         return
                     #how much to we need to slice off to complete the header:
                     read = min(len(buf), HEADER_SIZE-len(header))
@@ -930,7 +930,7 @@ class Protocol:
                     if padding_size > 0:
                         def debug_str(s):
                             try:
-                                return hexstr(bytearray(s))
+                                return hexstr(s)
                             except Exception:
                                 return csv(tuple(s))
                         # pad byte value is number of padding bytes added
@@ -940,10 +940,13 @@ class Protocol:
                         else:
                             actual_padding = data[-padding_size:]
                             cryptolog.warn("Warning: %s decryption failed: invalid padding", self.cipher_in_name)
-                            cryptolog(" data does not end with %s padding bytes %s",
-                                      self.cipher_in_padding, debug_str(padtext))
-                            cryptolog(" but with %s (%s)", debug_str(actual_padding), type(data))
-                            cryptolog(" decrypted data: %s", debug_str(data[:128]))
+                            cryptolog(" cipher block size=%i, data size=%i", self.cipher_in_block_size, data_size)
+                            cryptolog(" data does not end with %i %s padding bytes %s (%s)",
+                                      padding_size, self.cipher_in_padding, debug_str(padtext), type(padtext))
+                            cryptolog(" but with %i bytes: %s (%s)",
+                                      len(actual_padding), debug_str(actual_padding), type(data))
+                            cryptolog(" decrypted data (%i bytes): %r..", len(data), data[:128])
+                            cryptolog(" decrypted data (hex): %s..", debug_str(data[:128]))
                             self._internal_error("%s encryption padding error - wrong key?" % self.cipher_in_name)
                             return
                         data = data[:-padding_size]
@@ -984,7 +987,7 @@ class Protocol:
                     continue
                 #final packet (packet_index==0), decode it:
                 try:
-                    packet = decode(data, protocol_flags)
+                    packet = list(decode(data, protocol_flags))
                 except InvalidPacketEncodingException as e:
                     self.invalid("invalid packet encoding: %s" % e, data)
                     return
