@@ -5,8 +5,10 @@
 
 import sys
 import os
+import shlex
 import socket
 import signal
+from time import sleep
 from subprocess import PIPE, Popen
 
 from xpra.scripts.main import InitException, InitExit, shellquote, host_target_string
@@ -14,21 +16,22 @@ from xpra.platform.paths import get_ssh_known_hosts_files
 from xpra.platform import get_username
 from xpra.scripts.config import parse_bool
 from xpra.net.bytestreams import SocketConnection, SOCKET_TIMEOUT, ConnectionClosedException
+from xpra.make_thread import start_thread
 from xpra.exit_codes import (
     EXIT_SSH_KEY_FAILURE, EXIT_SSH_FAILURE,
     EXIT_CONNECTION_FAILED,
     )
 from xpra.os_util import (
-    bytestostr, osexpand, load_binary_file,
+    bytestostr, osexpand, load_binary_file, monotonic_time,
     nomodule_context, umask_context, is_main_thread,
     WIN32, OSX, POSIX,
     )
-from xpra.util import envint, envbool, nonl, engs, csv
+from xpra.util import envint, envbool, envfloat, engs, csv
 from xpra.log import Logger, is_debug_enabled
 
 log = Logger("network", "ssh")
 
-INITENV_COMMAND = os.environ.get("XPRA_INITENV_COMMAND", "xpra initenv")
+INITENV_COMMAND = os.environ.get("XPRA_INITENV_COMMAND", "")    #"xpra initenv"
 WINDOW_SIZE = envint("XPRA_SSH_WINDOW_SIZE", 2**27-1)
 TIMEOUT = envint("XPRA_SSH_TIMEOUT", 60)
 SKIP_UI = envbool("XPRA_SKIP_UI", False)
@@ -44,6 +47,10 @@ KEY_AUTH = envbool("XPRA_SSH_KEY_AUTH", True)
 PASSWORD_AUTH = envbool("XPRA_SSH_PASSWORD_AUTH", True)
 PASSWORD_RETRY = envint("XPRA_SSH_PASSWORD_RETRY", 2)
 assert PASSWORD_RETRY>=0
+MAGIC_QUOTES = envbool("XPRA_SSH_MAGIC_QUOTES", True)
+TEST_COMMAND_TIMEOUT = envint("XPRA_SSH_TEST_COMMAND_TIMEOUT", 10)
+EXEC_STDOUT_TIMEOUT = envfloat("XPRA_SSH_EXEC_STDOUT_TIMEOUT", 2)
+EXEC_STDERR_TIMEOUT = envfloat("XPRA_SSH_EXEC_STDERR_TIMEOUT", 0)
 
 
 def keymd5(k) -> str:
@@ -121,7 +128,7 @@ def confirm_key(info=()) -> bool:
         r = code==200
         log.info("host key %sconfirmed", ["not ", ""][r])
         return r
-    log("confirm_key(%s) will use stdin prompt", nonl(info))
+    log("confirm_key(%r) will use stdin prompt", info)
     prompt = "Are you sure you want to continue connecting (yes/NO)? "
     sys.stderr.write(os.linesep.join(info)+os.linesep+prompt)
     try:
@@ -147,29 +154,28 @@ def input_pass(prompt) -> str:
 
 class SSHSocketConnection(SocketConnection):
 
-    def __init__(self, ssh_channel, sock, sockname, peername, target, info=None):
-        super().__init__(ssh_channel, sockname, peername, target, "ssh", info)
+    def __init__(self, ssh_channel, sock, sockname, peername, target, info=None, socket_options=None):
         self._raw_socket = sock
+        super().__init__(ssh_channel, sockname, peername, target, "ssh", info, socket_options)
+
+    def get_raw_socket(self):
+        return self._raw_socket
 
     def start_stderr_reader(self):
-        from xpra.make_thread import start_thread
         start_thread(self._stderr_reader, "ssh-stderr-reader", daemon=True)
 
     def _stderr_reader(self):
         #stderr = self._socket.makefile_stderr(mode="rb", bufsize=1)
         chan = self._socket
         stderr = chan.makefile_stderr("rb", 1)
-        errs = []
         while self.active:
             v = stderr.readline()
             if not v:
-                log("SSH EOF on stderr of %s", chan.get_name())
-                return
-            errs.append(bytestostr(v.rstrip(b"\n\r")))
-        if errs:
-            log.warn("remote SSH stderr:")
-            for e in errs:
-                log.warn(" %s", e)
+                log.info("SSH EOF on stderr of %s", chan.get_name())
+                break
+            s = bytestostr(v.rstrip(b"\n\r"))
+            if s:
+                log.info(" SSH: %r", s)
 
     def peek(self, n):
         if not self._raw_socket:
@@ -265,7 +271,11 @@ def ssh_paramiko_connect_to(display_desc):
         if os.path.exists(user_config_file):
             with open(user_config_file) as f:
                 ssh_config.parse(f)
-            log("parsed user config '%s': %i hosts found", user_config_file, len(ssh_config.get_hostnames()))
+            log("parsed user config '%s'", user_config_file)
+            try:
+                log("%i hosts found", len(ssh_config.get_hostnames()))
+            except KeyError:
+                pass
             host_config = ssh_config.lookup(host)
             if host_config:
                 log("got host config for '%s': %s", host, host_config)
@@ -274,8 +284,8 @@ def ssh_paramiko_connect_to(display_desc):
                 cport = host_config.get("port", port)
                 try:
                     port = int(cport)
-                except ValueError:
-                    raise InitExit(EXIT_SSH_FAILURE, "invalid ssh port specified: '%s'" % cport)
+                except (TypeError, ValueError):
+                    raise InitExit(EXIT_SSH_FAILURE, "invalid ssh port specified: '%s'" % cport) from None
                 proxycommand = host_config.get("proxycommand")
                 if proxycommand:
                     log("found proxycommand='%s' for host '%s'", proxycommand, chost)
@@ -303,7 +313,7 @@ def ssh_paramiko_connect_to(display_desc):
                     chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
                     peername = (chost, cport)
                     conn = SSHProxyCommandConnection(chan, peername, peername, socket_info)
-                    conn.target = host_target_string("ssh", cusername, chost, cport, display)
+                    conn.target = host_target_string("ssh", cusername, chost, port, display)
                     conn.timeout = SOCKET_TIMEOUT
                     conn.start_stderr_reader()
                     conn.process = (sock.process, "ssh", cmd)
@@ -719,7 +729,7 @@ keymd5(host_key),
 
 def paramiko_run_test_command(transport, cmd):
     from paramiko import SSHException
-    log("run_test_command('%s')", cmd)
+    log("paramiko_run_test_command(transport, %r)", cmd)
     try:
         chan = transport.open_session(window_size=None, max_packet_size=0, timeout=60)
         chan.set_name("find %s" % cmd)
@@ -727,10 +737,29 @@ def paramiko_run_test_command(transport, cmd):
         log("open_session", exc_info=True)
         raise InitExit(EXIT_SSH_FAILURE, "failed to open SSH session: %s" % e) from None
     chan.exec_command(cmd)
-    out = chan.makefile().read()
-    err = chan.makefile_stderr().read()
+    log("exec_command returned")
+    start = monotonic_time()
+    while not chan.exit_status_ready():
+        if monotonic_time()-start>TEST_COMMAND_TIMEOUT:
+            chan.close()
+            raise InitException("SSH test command '%s' timed out" % cmd)
+        log("exit status is not ready yet, sleeping")
+        sleep(0.01)
     code = chan.recv_exit_status()
-    log("exec_command('%s')=%s, out=%s, err=%s", cmd, code, out, err)
+    log("exec_command('%s')=%s", cmd, code)
+    def chan_read(read_fn):
+        try:
+            return read_fn()
+        except socket.error:
+            log("chan_read(%s)", read_fn, exc_info=True)
+            return b""
+    #don't wait too long for the data:
+    chan.settimeout(EXEC_STDOUT_TIMEOUT)
+    out = chan_read(chan.makefile().readline)
+    log("exec_command out=%r", out)
+    chan.settimeout(EXEC_STDERR_TIMEOUT)
+    err = chan_read(chan.makefile_stderr().readline)
+    log("exec_command err=%r", err)
     chan.close()
     return out, err, code
 
@@ -748,7 +777,7 @@ def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=Non
             if r[2]!=0:
                 continue
             #let's find where xpra is installed:
-            r = rtc("FOR /F \"usebackq tokens=3*\" %A IN (`REG QUERY \"HKEY_LOCAL_MACHINE\Software\Xpra\" /v InstallPath`) DO (echo %A %B)")
+            r = rtc("FOR /F \"usebackq tokens=3*\" %A IN (`REG QUERY \"HKEY_LOCAL_MACHINE\\Software\\Xpra\" /v InstallPath`) DO (echo %A %B)")  #pylint: disable=line-too-long
             if r[2]==0:
                 #found in registry:
                 lines = r[0].splitlines()
@@ -764,6 +793,12 @@ def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=Non
             r = rtc("which %s" % xpra_cmd)
             if r[2]!=0:
                 continue
+            if r[0]:
+                #use the actual path returned by 'which':
+                try:
+                    xpra_cmd = r[0].decode().rstrip("\n\r")
+                except:
+                    pass
         if xpra_cmd in tried:
             continue
         tried.add(xpra_cmd)
@@ -788,15 +823,15 @@ def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=Non
             log("channel exec_command(%s)" % cmd)
             chan.exec_command(cmd)
             return chan
-    raise Exception("all SSH remote proxy commands have failed")
+    raise Exception("all SSH remote proxy commands have failed - is xpra installed on the remote host?")
 
 
 def ssh_connect_failed(_message):
     #by the time ssh fails, we may have entered the gtk main loop
     #(and more than once thanks to the clipboard code..)
-    if "gtk" in sys.modules or "gi.repository.Gtk" in sys.modules:
-        from xpra.gtk_common.quit import gtk_main_quit_really
-        gtk_main_quit_really()
+    if "gi.repository.Gtk" in sys.modules:
+        from gi.repository import Gtk
+        Gtk.main_quit()
 
 
 def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
@@ -804,7 +839,7 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         ssh_fail_cb = ssh_connect_failed
     sshpass_command = None
     try:
-        cmd = display_desc["full_ssh"]
+        cmd = list(display_desc["full_ssh"])
         kwargs = {}
         env = display_desc.get("env")
         kwargs["stderr"] = sys.stderr
@@ -813,10 +848,12 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
             startupinfo = STARTUPINFO()
             startupinfo.dwFlags |= STARTF_USESHOWWINDOW
             startupinfo.wShowWindow = 0     #aka win32.con.SW_HIDE
-            kwargs["startupinfo"] = startupinfo
             flags = CREATE_NEW_PROCESS_GROUP | CREATE_NEW_CONSOLE
-            kwargs["creationflags"] = flags
-            kwargs["stderr"] = PIPE
+            kwargs.update({
+                "startupinfo"   : startupinfo,
+                "creationflags" : flags,
+                "stderr"        : PIPE,
+                })
         elif not display_desc.get("exit_ssh", False) and not OSX:
             kwargs["start_new_session"] = True
         remote_xpra = display_desc["remote_xpra"]
@@ -842,8 +879,15 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         remote_cmd += "else echo \"no run-xpra command found\"; exit 1; fi"
         if INITENV_COMMAND:
             remote_cmd = INITENV_COMMAND + ";" + remote_cmd
-        #this gives us better compatibility with weird login shells:
-        cmd.append("sh -c '%s'" % remote_cmd)
+        #how many times we need to escape the remote command string
+        #depends on how many times the ssh command is parsed
+        nssh = sum(int(x=="ssh") for x in cmd)
+        if nssh>=2 and MAGIC_QUOTES:
+            for _ in range(nssh):
+                remote_cmd = shlex.quote(remote_cmd)
+        else:
+            remote_cmd = "'%s'" % remote_cmd
+        cmd.append("sh -c %s" % remote_cmd)
         if debug_cb:
             debug_cb("starting %s tunnel" % str(cmd[0]))
         #non-string arguments can make Popen choke,
@@ -955,4 +999,24 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
     conn.endpoint = host_target_string("ssh", username, host, port, display)
     conn.timeout = 0            #taken care of by abort_test
     conn.process = (child, "ssh", cmd)
+    if kwargs.get("stderr")==PIPE:
+        def stderr_reader():
+            errs = []
+            while child.poll() is None:
+                try:
+                    v = child.stderr.readline()
+                except OSError:
+                    log("stderr_reader()", exc_info=True)
+                    break
+                if not v:
+                    log("SSH EOF on stderr of %s", cmd)
+                    break
+                s = bytestostr(v.rstrip(b"\n\r"))
+                if s:
+                    errs.append(s)
+            if errs:
+                log.warn("remote SSH stderr:")
+                for e in errs:
+                    log.warn(" %s", e)
+        start_thread(stderr_reader, "ssh-stderr-reader", daemon=True)
     return conn
