@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -37,12 +37,84 @@ def is_main_thread():
     return threading.current_thread()==main_thread
 
 
+def get_frame_info(ignore_threads=()):
+    info = {
+        "count"        : threading.active_count() - len(ignore_threads),
+        }
+    try:
+        import traceback
+        def nn(x):
+            if x is None:
+                return ""
+            return str(x)
+        thread_ident = {}
+        for t in threading.enumerate():
+            if t not in ignore_threads:
+                thread_ident[t.ident] = t.getName()
+            else:
+                thread_ident[t.ident] = None
+        thread_ident.update({
+                threading.current_thread().ident    : "info",
+                main_thread.ident                   : "main",
+                })
+        frames = sys._current_frames()  #pylint: disable=protected-access
+        stack = None
+        for i,frame_pair in enumerate(frames.items()):
+            stack = traceback.extract_stack(frame_pair[1])
+            tident = thread_ident.get(frame_pair[0], "unknown")
+            if tident is None:
+                continue
+            #sanitize stack to prevent None values (which cause encoding errors with the bencoder)
+            sanestack = []
+            for e in stack:
+                sanestack.append(tuple([nn(x) for x in e]))
+            info[i] = {
+                ""          : tident,
+                "stack"     : sanestack,
+                }
+        del frames, stack
+    except Exception as e:
+        get_util_logger().error("failed to get frame info: %s", e)
+    return info
+
+def get_info_env():
+    filtered_env = os.environ.copy()
+    if filtered_env.get('XPRA_PASSWORD'):
+        filtered_env['XPRA_PASSWORD'] = "*****"
+    if filtered_env.get('XPRA_ENCRYPTION_KEY'):
+        filtered_env['XPRA_ENCRYPTION_KEY'] = "*****"
+    return filtered_env
+
+def get_sysconfig_info():
+    import sysconfig
+    sysinfo = {}
+    log = get_util_logger()
+    for attr in (
+        "platform",
+        "python-version",
+        "config-vars",
+        "paths",
+        ):
+        fn = "get_%s" % attr.replace("-", "_")
+        getter = getattr(sysconfig, fn, None)
+        if getter:
+            try:
+                sysinfo[attr] = getter()  #pylint: disable=not-callable
+            except ModuleNotFoundError:
+                log("sysconfig.%s", fn, exc_info=True)
+                if attr=="config-vars" and WIN32:
+                    continue
+                log.warn("Warning: failed to collect %s sysconfig information", attr)
+            except Exception:
+                log.error("Error calling sysconfig.%s", fn, exc_info=True)
+    return sysinfo
+
 def strtobytes(x) -> bytes:
     if isinstance(x, bytes):
         return x
     return str(x).encode("latin1")
 def bytestostr(x) -> str:
-    if isinstance(x, bytes):
+    if isinstance(x, (bytes, bytearray)):
         return x.decode("latin1")
     return str(x)
 def hexstr(v) -> str:
@@ -225,8 +297,9 @@ def is_X11() -> bool:
         get_util_logger().debug("failed to load x11 bindings", exc_info=True)
         return True
 
+saved_env = os.environ.copy()
 def is_Wayland() -> bool:
-    return _is_Wayland(os.environ)
+    return _is_Wayland(saved_env)
 
 def _is_Wayland(env : dict) -> bool:
     backend = env.get("GDK_BACKEND", "")
@@ -259,9 +332,10 @@ def get_distribution_version_id() -> bool:
         return ""
     try:
         v = load_os_release_file()
-        for l in v.splitlines():
-            if l.startswith(b"VERSION_ID="):
-                return l.split(b"=", 1)[1]
+        for line in v.splitlines():
+            l = line.decode()
+            if l.startswith("VERSION_ID="):
+                return l.split("=", 1)[1].strip('"')
     except Exception:
         pass
     return ""
@@ -318,7 +392,7 @@ def get_linux_distribution():
         except Exception:
             try:
                 import platform
-                _linux_distribution = platform.linux_distribution()  #pylint: disable=deprecated-method
+                _linux_distribution = platform.linux_distribution()  #@UndefinedVariable, pylint: disable=deprecated-method, no-member
             except Exception:
                 _linux_distribution = ("unknown", "unknown", "unknown")
         else:
@@ -400,6 +474,27 @@ def load_binary_file(filename) -> bytes:
         get_util_logger().warn(" %s", e)
         return None
 
+def parse_encoded_bin_data(data):
+    if not data:
+        return None
+    header = bytestostr(data).lower()[:10]
+    if header.startswith("0x"):
+        return binascii.unhexlify(data[2:])
+    import base64
+    if header.startswith("b64:"):
+        return base64.b64decode(data[4:])
+    if header.startswith("base64:"):
+        return base64.b64decode(data[7:])
+    try:
+        return binascii.unhexlify(data)
+    except (TypeError, binascii.Error):
+        try:
+            return base64.b64decode(data)
+        except Exception:
+            pass
+    return None
+
+
 #here so we can override it when needed
 def force_quit(status=1):
     os._exit(status)  #pylint: disable=protected-access
@@ -478,12 +573,22 @@ def shellsub(s, subs=None):
     """ shell style string substitution using the dictionary given """
     if subs:
         for var,value in subs.items():
-            s = s.replace("$%s" % var, str(value))
-            s = s.replace("${%s}" % var, str(value))
+            try:
+                if isinstance(s, bytes):
+                    s = s.replace(("$%s" % var).encode(), str(value).encode())
+                    s = s.replace(("${%s}" % var).encode(), str(value).encode())
+                else:
+                    s = s.replace("$%s" % var, str(value))
+                    s = s.replace("${%s}" % var, str(value))
+            except (TypeError, ValueError):
+                raise Exception("failed to substitute '%s' with value '%s' (%s) in '%s'" % (
+                    var, value, type(value), s)) from None
     return s
 
 
 def osexpand(s, actual_username="", uid=0, gid=0, subs=None):
+    if not s:
+        return s
     def expanduser(s):
         if actual_username and s.startswith("~/"):
             #replace "~/" with "~$actual_username/"
@@ -501,7 +606,9 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs=None):
             })
         if not OSX:
             from xpra.platform.xposix.paths import get_runtime_dir
-            d["XDG_RUNTIME_DIR"] = os.environ.get("XDG_RUNTIME_DIR", get_runtime_dir())
+            rd = get_runtime_dir()
+            if rd and "XDG_RUNTIME_DIR" not in os.environ:
+                d["XDG_RUNTIME_DIR"] = rd
     if actual_username:
         d["USERNAME"] = actual_username
         d["USER"] = actual_username
@@ -624,10 +731,7 @@ class nomodule_context:
     def __exit__(self, *_args):
         if sys.modules.get(self.module_name) is None:
             if self.saved_module is None:
-                try:
-                    del sys.modules[self.module_name]
-                except KeyError:
-                    pass
+                sys.modules.pop(self.module_name, None)
             else:
                 sys.modules[self.module_name] = self.saved_module
     def __repr__(self):
