@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2017-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -13,8 +13,6 @@ from ctypes import (
 from ctypes.wintypes import (
     HWND, DWORD, WPARAM, LPARAM, HDC, HMONITOR, HMODULE,
     SHORT, ATOM, RECT, POINT, MAX_PATH, WCHAR, BYTE,
-    )
-from ctypes.wintypes import (
     HANDLE, LPSTR, LPCWSTR, UINT, INT, BOOL, WORD, HGDIOBJ,
     LONG, LPVOID, HBITMAP, LPCSTR, LPWSTR, HWINSTA,
     HINSTANCE, HMENU, ULONG, HHOOK, LPMSG,
@@ -103,6 +101,29 @@ class CIEXYZTRIPLE(Structure):
         ('ciexyzGreen', CIEXYZ),
     ]
 
+class BITMAPINFOHEADER(Structure):
+    _fields_ = [
+        ("biSize",          DWORD),
+        ("biWidth",         LONG),
+        ("biHeight",        LONG),
+        ("biPlanes",        WORD),
+        ("biBitCount",      WORD),
+        ("biCompression",   DWORD),
+        ("biSizeImage",     DWORD),
+        ("biXPelsPerMeter", LONG),
+        ("biYPelsPerMeter", LONG),
+        ("biClrUsed",       DWORD),
+        ("biClrImportant",  DWORD),
+        ]
+PBITMAPINFOHEADER = POINTER(BITMAPINFOHEADER)
+
+class BITMAPINFO(Structure):
+    _fields_ = [
+        ("bmiHeader",       BITMAPINFOHEADER),
+        ("bmiColors",       DWORD),
+        ]
+PBITMAPINFO = POINTER(BITMAPINFO)
+
 class BITMAPV5HEADER(Structure):
     _fields_ = [
         ('bV5Size',             DWORD),
@@ -130,6 +151,7 @@ class BITMAPV5HEADER(Structure):
         ('bV5ProfileSize',      DWORD),
         ('bV5Reserved',         DWORD),
     ]
+PBITMAPV5HEADER = POINTER(BITMAPV5HEADER)
 
 CCHDEVICENAME = 32
 class MONITORINFOEX(Structure):
@@ -211,6 +233,9 @@ GlobalFree.argtypes = [HGLOBAL]
 GlobalLock = kernel32.GlobalLock
 GlobalLock.restype = LPVOID
 GlobalLock.argtypes = [HGLOBAL]
+GlobalSize = kernel32.GlobalSize
+GlobalSize.argtypes = [HGLOBAL]
+GlobalSize.restype = c_size_t
 GlobalUnlock = kernel32.GlobalUnlock
 GlobalUnlock.restype = BOOL
 GlobalUnlock.argtypes = [HGLOBAL]
@@ -358,6 +383,8 @@ LoadIconA.restype = HICON
 #LoadIconA.argtypes = [HINSTANCE, LPCSTR]
 RegisterWindowMessageA = user32.RegisterWindowMessageA
 UpdateWindow = user32.UpdateWindow
+UpdateWindow.restype = BOOL
+UpdateWindow.argtypes = [HWND]
 DestroyIcon = user32.DestroyIcon
 DestroyIcon.restype = BOOL
 DestroyIcon.argtypes = [HICON]
@@ -469,9 +496,6 @@ SetClipboardData.argtypes = [UINT, HANDLE]
 RegisterClipboardFormatA = user32.RegisterClipboardFormatA
 RegisterClipboardFormatA.restype = UINT
 RegisterClipboardFormatA.argtypes = [LPCSTR]
-RegisterClipboardFormatA = user32.RegisterClipboardFormatA
-RegisterClipboardFormatA.restype = UINT
-RegisterClipboardFormatA.argtypes = [LPCSTR]
 GetClipboardData = user32.GetClipboardData
 GetClipboardData.restype = HANDLE
 GetClipboardData.argtypes = [UINT]
@@ -559,6 +583,8 @@ advapi32 = WinDLL("advapi32")
 CreateProcessAsUserA = advapi32.CreateProcessAsUserA
 CreateProcessAsUserA.restype = BOOL
 CreateProcessAsUserA.argtypes = [HANDLE, LPCSTR, LPSTR, LPSECURITY_ATTRIBUTES, LPSECURITY_ATTRIBUTES, BOOL, DWORD, LPVOID, LPCSTR, LPCSTR, PSTARTUPINFOA, PPROCESS_INFORMATION]
+GetUserNameA = advapi32.GetUserNameA
+GetUserNameA.restype = BOOL
 
 gdi32 = WinDLL("gdi32", use_last_error=True)
 CreateCompatibleDC = gdi32.CreateCompatibleDC
@@ -591,6 +617,9 @@ DeleteDC = gdi32.DeleteDC
 CreateDIBSection = gdi32.CreateDIBSection
 CreateDIBSection.restype = HBITMAP
 CreateDIBSection.argtypes = [HANDLE, POINTER(BITMAPV5HEADER), UINT, POINTER(c_void_p), HANDLE, DWORD]
+CreateDIBitmap = gdi32.CreateDIBitmap
+CreateDIBitmap.restype = HBITMAP
+CreateDIBitmap.argtypes = [HDC, PBITMAPINFOHEADER, DWORD, c_void_p, PBITMAPINFO, UINT]
 DeleteObject = gdi32.DeleteObject
 DeleteObject.argtypes = [HGDIOBJ]
 DeleteObject.restype = BOOL
