@@ -9,11 +9,17 @@ import os
 from xpra.os_util import get_util_logger, osexpand
 from xpra.platform.dotxpra_common import LIVE, DEAD, UNKNOWN, INACCESSIBLE
 
+DISPLAY_PREFIX = ""
+
 PIPE_PREFIX = "Xpra\\"
 PIPE_ROOT = "\\\\"
 PIPE_PATH = "%s.\\pipe\\" % PIPE_ROOT
 
 
+def norm_makepath(dirpath, name):
+    return PIPE_PATH+PIPE_PREFIX+name.lstrip(":")
+
+
 class DotXpra:
     def __init__(self, sockdir=None, sockdirs=(), actual_username="", *_args, **_kwargs):
         self.username = actual_username
@@ -26,12 +32,16 @@ class DotXpra:
         pass
 
 
+    def displays(self, check_uid=0, matching_state=None):
+        return tuple(self.get_all_namedpipes().keys())
+
+
     def norm_socket_paths(self, local_display_name):
         return [self.socket_path(local_display_name)]
 
 
     def socket_path(self, local_display_name):
-        return PIPE_PATH+PIPE_PREFIX+local_display_name.replace(":", "")
+        return norm_makepath(None, local_display_name)
 
     LIVE = LIVE
     DEAD = DEAD
