@@ -40,7 +40,6 @@ from xpra.platform.win32.common import (
 from xpra.util import AdHocStruct, csv, envint, envbool, typedict
 from xpra.os_util import get_util_logger, strtobytes
 from xpra.log import Logger
-from xpra.platform.win32.constants import ENDSESSION_LOGOFF
 
 log = Logger("win32")
 grablog = Logger("win32", "grab")
@@ -112,6 +111,11 @@ def do_init():
     init_dpi()
     if APP_ID:
         init_appid()
+    try:
+        from xpra.platform.win32.dwm_color import match_window_color
+        match_window_color()
+    except Exception:
+        log.error("Error: failed to setup dwm window color matching", exc_info=True)
 
 def init_dpi():
     #tell win32 we handle dpi
@@ -504,14 +508,8 @@ def apply_maxsize_hints(window, hints):
     #remove them so GTK doesn't try to set attributes,
     #which would remove the maximize button:
     for x in ("min_width", "min_height", "max_width", "max_height"):
-        try:
-            del hints[x]
-        except KeyError:
-            pass
-        try:
-            del hints[strtobytes(x)]
-        except KeyError:
-            pass
+        hints.pop(x, None)
+        hints.pop(strtobytes(x), None)
     window_state_updated(window)
 
 def apply_geometry_hints(self, hints):
@@ -1034,6 +1032,7 @@ class ClientExtras:
                           win32con.WM_SYSKEYUP      : "SYSKEYUP",
                           }
         def low_level_keyboard_handler(nCode, wParam, lParam):
+            log("WH_KEYBOARD_LL: %s", (nCode, wParam, lParam))
             try:
                 scan_code = lParam.contents.scan_code
                 vk_code = lParam.contents.vk_code
@@ -1097,7 +1096,7 @@ class ClientExtras:
             except Exception as e:
                 keylog.error("Error: low level keyboard hook failed")
                 keylog.error(" %s", e)
-            return CallNextHookEx(keyboard_hook_id, nCode, wParam, lParam)
+            return CallNextHookEx(0, nCode, wParam, lParam)
         # Our low level handler signature.
         CMPFUNC = CFUNCTYPE(c_int, WPARAM, LPARAM, POINTER(KBDLLHOOKSTRUCT))
         # Convert the Python handler into C pointer.
