@@ -10,7 +10,7 @@ import struct
 
 from xpra.os_util import bytestostr, hexstr
 from xpra.util import iround, envbool, envint, csv, ellipsizer
-from xpra.os_util import is_unity, is_gnome, is_kde, is_Ubuntu, is_Fedora, is_X11, is_Wayland
+from xpra.os_util import is_unity, is_gnome, is_kde, is_Ubuntu, is_Fedora, is_X11, is_Wayland, saved_env
 from xpra.log import Logger
 
 log = Logger("posix")
@@ -36,6 +36,17 @@ def X11WindowBindings():
                 log.error(" %s", e)
     return _X11Window
 
+def X11RandRBindings():
+    if is_X11():
+        try:
+            from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
+            return RandRBindings()
+        except Exception as e:
+            log("RandRBindings()", exc_info=True)
+            log.error("Error: no X11 RandR bindings")
+            log.error(" %s", e)
+    return None
+
 X11XI2 = False
 def X11XI2Bindings():
     global X11XI2
@@ -69,9 +80,12 @@ def get_native_system_tray_classes():
     return []
 
 def get_wm_name():
-    wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
+    wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "") or os.environ.get("XDG_SESSION_DESKTOP") or os.environ.get("DESKTOP_SESSION")
     if os.environ.get("XDG_SESSION_TYPE")=="wayland" or os.environ.get("GDK_BACKEND")=="wayland":
-        wm_name = "wayland"
+        if wm_name:
+            wm_name += " on wayland"
+        else:
+            wm_name = "wayland"
     elif is_X11():
         try:
             wm_check = _get_X11_root_property("_NET_SUPPORTING_WM_CHECK", "WINDOW")
@@ -90,7 +104,7 @@ def get_wm_name():
 
 def get_clipboard_native_class():
     if is_Wayland():
-        return None
+        return "xpra.gtk_common.gtk_clipboard.GTK_Clipboard"
     return "xpra.x11.gtk_x11.clipboard.X11Clipboard"
 
 def get_native_tray_classes():
@@ -102,6 +116,12 @@ def get_native_tray_classes():
             c.append(AppindicatorTray)
         except (ImportError, ValueError):
             traylog("cannot load appindicator tray", exc_info=True)
+            traylog.warn("Warning: appindicator library not found")
+            traylog.warn(" you may want to install libappindicator")
+            traylog.warn(" to enable the system tray.")
+            if saved_env.get("XDG_CURRENT_DESKTOP")=="GNOME":
+                traylog.warn(" With gnome-shell, you may also need some extensions:")
+                traylog.warn(" 'top icons plus' and / or 'appindicator'")
     return c
 
 
@@ -155,6 +175,8 @@ def _get_X11_root_property(name, req_type):
 def _get_xsettings():
     from xpra.gtk_common.error import xlog
     X11Window = X11WindowBindings()
+    if not X11Window:
+        return None
     with xlog:
         selection = "_XSETTINGS_S0"
         owner = X11Window.XGetSelectionOwner(selection)
@@ -201,10 +223,9 @@ def _get_xsettings_dpi():
 def _get_randr_dpi():
     if RANDR_DPI and not is_Wayland():
         from xpra.gtk_common.error import xlog
-        from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
         with xlog:
-            randr_bindings = RandRBindings()
-            if randr_bindings.has_randr():
+            randr_bindings = X11RandRBindings()
+            if randr_bindings and randr_bindings.has_randr():
                 wmm, hmm = randr_bindings.get_screen_size_mm()
                 if wmm>0 and hmm>0:
                     w, h =  randr_bindings.get_screen_size()
@@ -457,7 +478,9 @@ def system_bell(window, device, percent, _pitch, _duration, bell_class, bell_id,
             x11_bell()
         return  True
     except XError as e:
-        log.error("error using device_bell: %s, switching native X11 bell support off", e)
+        log("x11_bell()", exc_info=True)
+        log.error("Error using device_bell: %s", e)
+        log.error(" switching native X11 bell support off")
         device_bell = False
         return False
 
@@ -476,7 +499,9 @@ def _send_client_message(window, message_type, *values):
         SubstructureNotifyMask = constants["SubstructureNotifyMask"]
         SubstructureRedirectMask = constants["SubstructureRedirectMask"]
         event_mask = SubstructureNotifyMask | SubstructureRedirectMask
-        X11Window.sendClientMessage(root_xid, xid, False, event_mask, message_type, *values)
+        from xpra.gtk_common.error import xsync
+        with xsync:
+            X11Window.sendClientMessage(root_xid, xid, False, event_mask, message_type, *values)
     except Exception as e:
         log.warn("failed to send client message '%s' with values=%s: %s", message_type, values, e)
 
@@ -905,9 +930,9 @@ class ClientExtras:
 
     def do_setup_xi(self):
         if self.client.server_input_devices not in ("xi", "uinput"):
-            xinputlog.info("server does not support xi input devices")
+            xinputlog("server does not support xi input devices")
             if self.client.server_input_devices:
-                log.info(" server uses: %s", self.client.server_input_devices)
+                log(" server uses: %s", self.client.server_input_devices)
             return False
         try:
             from xpra.gtk_common.error import xsync, XError
@@ -921,7 +946,8 @@ class ClientExtras:
                     XI2.select_xi2_events()
             except XError:
                 self.xi_setup_failures += 1
-                xinputlog("select_xi2_events() failed, attempt %i", self.xi_setup_failures,exc_info=True)
+                xinputlog("select_xi2_events() failed, attempt %i",
+                          self.xi_setup_failures, exc_info=True)
                 return self.xi_setup_failures<10    #try again
             with xsync:
                 XI2.gdk_inject()
