@@ -12,6 +12,7 @@ from xpra.os_util import (
     WIN32, OSX, POSIX,
     osexpand, getuid, getgid, get_username_for_uid,
     is_Debian, is_Ubuntu, is_arm,
+    which,
     )
 
 def warn(msg):
@@ -59,6 +60,9 @@ def has_sound_support():
 
 
 def get_xorg_bin():
+    xorg = os.environ.get("XPRA_XORG_BIN")
+    if xorg:
+        return xorg
     # Detect Xorg Binary
     if is_arm() and is_Debian() and os.path.exists("/usr/bin/Xorg"):
         #Raspbian breaks if we use a different binary..
@@ -106,17 +110,18 @@ def get_Xdummy_command(xorg_cmd="Xorg", log_dir="${XPRA_LOG_DIR}", xorg_conf="/e
           ]
     return cmd
 
-def get_Xvfb_command():
+def get_Xvfb_command(width=8192, height=4096, dpi=96):
     cmd = ["Xvfb",
            "+extension", "GLX",
            "+extension", "Composite",
-           "-screen", "0", "5760x2560x24+32",
+           "-screen", "0", "%ix%ix24+32" % (width, height),
            #better than leaving to vfb after a resize?
-           "-dpi", "96",
            "-nolisten", "tcp",
            "-noreset",
            "-auth", "$XAUTHORITY"
            ]
+    if dpi>0:
+        cmd += ["-dpi", str(dpi)]
     return cmd
 
 def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
@@ -179,7 +184,32 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
     return Xorg_suid_check()
 
 
-def OpenGL_safety_check():
+def xvfb_cmd_str(xvfb):
+    xvfb_str = ""
+    while xvfb:
+        s = ""
+        while xvfb:
+            item = xvfb[0]
+            l = len(item)
+            if (item.startswith("-") or item.startswith("+")) and len(xvfb)>1:
+                l += len(xvfb[1])
+            if s and len(s)+l>55:
+                break
+            v = xvfb.pop(0)
+            if not s:
+                s += v
+            else:
+                s += " "+v
+        if xvfb_str:
+            xvfb_str += " \\\n    "
+        xvfb_str += s
+    return xvfb_str
+
+
+def OpenGL_safety_check() -> str:
+    return is_VirtualBox()
+
+def is_VirtualBox() -> str:
     #try to detect VirtualBox:
     #based on the code found here:
     #http://spth.virii.lu/eof2/articles/WarGame/vboxdetect.html
@@ -195,7 +225,7 @@ def OpenGL_safety_check():
             try:
                 f = None
                 f = open("\\\\.\\VBoxMiniRdrDN", "r")
-                return True, "VirtualBox is present (VBoxMiniRdrDN)"
+                return "VirtualBox is present (VBoxMiniRdrDN)"
             finally:
                 if f:
                     f.close()
@@ -209,15 +239,15 @@ def OpenGL_safety_check():
 def get_build_info():
     info = []
     try:
-        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS             #@UnresolvedImport
+        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS, BRANCH, COMMIT #@UnresolvedImport
+        info.append("revision %s" % REVISION)
+        if COMMIT:
+            info.append("commit %s from %s branch" % (COMMIT, BRANCH))
         try:
             mods = int(LOCAL_MODIFICATIONS)
+            info.append("with %s local changes" % (mods))
         except ValueError:
-            mods = 0
-        if mods==0:
-            info.append("revision %s" % REVISION)
-        else:
-            info.append("revision %s with %s local changes" % (REVISION, LOCAL_MODIFICATIONS))
+            pass
     except Exception as e:
         warn("Error: could not find the source information: %s" % e)
     try:
@@ -523,6 +553,7 @@ OPTION_TYPES = {
                     "open-url"          : str,
                     "file-transfer"     : str,
                     "printing"          : str,
+                    "headerbar"         : str,
                     "challenge-handlers": list,
                     #ssl options:
                     "ssl"               : str,
@@ -547,7 +578,7 @@ OPTION_TYPES = {
                     "min-speed"         : int,
                     "compression_level" : int,
                     "dpi"               : int,
-                    "file-size-limit"   : int,
+                    "file-size-limit"   : str,
                     "idle-timeout"      : int,
                     "server-idle-timeout" : int,
                     "sync-xvfb"         : int,
@@ -564,7 +595,7 @@ OPTION_TYPES = {
                     "attach"            : bool,
                     "use-display"       : str,
                     "fake-xinerama"     : str,
-                    "resize_display"    : bool,
+                    "resize_display"    : str,
                     "tray"              : bool,
                     "pulseaudio"        : bool,
                     "dbus-proxy"        : bool,
@@ -596,9 +627,11 @@ OPTION_TYPES = {
                     "modal-windows"     : bool,
                     "bandwidth-detection" : bool,
                     "ssh-upgrade"       : bool,
+                    "splash"            : bool,
                     #arrays of strings:
                     "pulseaudio-configure-commands" : list,
                     "socket-dirs"       : list,
+                    "client-socket-dirs" : list,
                     "remote-xpra"       : list,
                     "encodings"         : list,
                     "proxy-video-encoders" : list,
@@ -654,7 +687,6 @@ NON_COMMAND_LINE_OPTIONS = [
     "pdf-printer",
     "postscript-printer",
     "add-printer-options",
-    "file-size-limit",
     ]
 
 START_COMMAND_OPTIONS = [
@@ -665,10 +697,10 @@ START_COMMAND_OPTIONS = [
     ]
 BIND_OPTIONS = ["bind", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-vsock", "bind-rfb"]
 
-#keep track of the options added since v1,
+#keep track of the options added since v3,
 #so we can generate command lines that work with older supported versions:
 OPTIONS_ADDED_SINCE_V3 = [
-    "source", "source-start",
+    "source", "source-start", "headerbar",
     ]
 OPTIONS_COMPAT_NAMES = {
     "--compression_level=" : "-z"
@@ -680,10 +712,11 @@ CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "remote-clipboard", "local-clipboard",
                   "tcp-encryption", "tcp-encryption-keyfile", "encryption",  "encryption-keyfile",
                   "systemd-run", "systemd-run-args",
-                  "socket-dir", "socket-dirs",
+                  "socket-dir", "socket-dirs", "client-socket-dirs",
                   "border", "window-close", "min-size", "max-size", "desktop-scaling",
                   "file-transfer", "file-size-limit", "download-path",
                   "open-command", "open-files", "printing", "open-url",
+                  "headerbar",
                   "challenge-handlers",
                   "dbus-proxy",
                   "remote-logging",
@@ -735,6 +768,7 @@ PROXY_START_OVERRIDABLE_OPTIONS = [
     "av-sync", "global-menus",
     "forward-xdg-open", "modal-windows", "bandwidth-detection",
     "ssh-upgrade",
+    "splash",
     "printing", "file-transfer", "open-command", "open-files", "open-url", "start-new-commands",
     "mmap", "mmap-group", "mdns",
     "auth", "vsock-auth", "tcp-auth", "udp-auth", "ws-auth", "wss-auth", "ssl-auth", "ssh-auth", "rfb-auth",
@@ -761,10 +795,7 @@ def get_default_key_shortcuts():
                (True,   "#+F2:show_start_new_command"),
                (True,   "#+F3:show_bug_report"),
                (True,   "#+F4:quit"),
-               (True,   "#+F5:increase_quality"),
-               (True,   "#+F6:decrease_quality"),
-               (True,   "#+F7:increase_speed"),
-               (True,   "#+F8:decrease_speed"),
+               (True,   "#+F5:show_window_info"),
                (True,   "#+F10:magic_key"),
                (True,   "#+F11:show_session_info"),
                (True,   "#+F12:toggle_debug"),
@@ -779,6 +810,8 @@ def get_default_key_shortcuts():
                (True,   "#+bar:scalereset"),
                (True,   "#+question:scalingoff"),
                (OSX,    "#+degree:scalereset"),
+               (OSX,    "meta+grave:void"),
+               (OSX,    "meta+shift+asciitilde:void"),
                )
                  if e]
 
@@ -823,7 +856,7 @@ def get_defaults():
         OPEN_COMMAND, DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS,
         DEFAULT_ENV, CAN_DAEMONIZE, SYSTEM_PROXY_SOCKET,
         )
-    from xpra.platform.paths import get_download_dir, get_remote_run_xpra_scripts, get_socket_dirs
+    from xpra.platform.paths import get_download_dir, get_remote_run_xpra_scripts, get_socket_dirs, get_client_socket_dirs
     try:
         from xpra.platform.info import get_username
         username = get_username()
@@ -851,22 +884,18 @@ def get_defaults():
         if conf_dir and os.path.exists(conf_dir):
             break
     xvfb = detect_xvfb_command(conf_dir, bin_dir, warn=None)
-    if WIN32:
-        bind_dirs = ["Main"]
-    else:
-        bind_dirs = ["auto"]
+    xvfb_str = xvfb_cmd_str(xvfb)
 
     ssl_protocol = "TLSv1_2"
 
-    if POSIX and not OSX:
-        from xpra.x11.fakeXinerama import find_libfakeXinerama
-        fake_xinerama = find_libfakeXinerama() or "no"
+    if POSIX and not OSX and not (is_Debian() or is_Ubuntu()):
+        fake_xinerama = "auto"
     else:
         fake_xinerama = "no"
 
     GLOBAL_DEFAULTS = {
                     "encoding"          : "auto",
-                    "title"             : "@title@ on @hostname@",
+                    "title"             : "@title@ on @hostinfo@",
                     "username"          : username,
                     "password"          : "",
                     "wm-name"           : DEFAULT_NET_WM_NAME,
@@ -896,7 +925,7 @@ def get_defaults():
                     "systemd-run"       : get_default_systemd_run(),
                     "systemd-run-args"  : "",
                     "system-proxy-socket" : SYSTEM_PROXY_SOCKET,
-                    "xvfb"              : " ".join(xvfb),
+                    "xvfb"              : xvfb_str,
                     "chdir"             : "",
                     "socket-dir"        : "",
                     "log-dir"           : "auto",
@@ -923,7 +952,7 @@ def get_defaults():
                     "socket-permissions": "600",
                     "exec-wrapper"      : "",
                     "dbus-launch"       : "dbus-launch --sh-syntax --close-stderr",
-                    "webcam"            : ["auto", "no"][OSX],
+                    "webcam"            : ["auto", "no"][OSX or WIN32],
                     "mousewheel"        : "on",
                     "input-devices"     : "auto",
                     "shortcut-modifiers": "auto",
@@ -931,6 +960,7 @@ def get_defaults():
                     "open-url"          : "auto",
                     "file-transfer"     : "auto",
                     "printing"          : "yes",
+                    "headerbar"         : ["auto", "no"][OSX or WIN32],
                     "challenge-handlers": ["all"],
                     #ssl options:
                     "ssl"               : "auto",
@@ -943,8 +973,8 @@ def get_defaults():
                     "ssl-client-verify-mode"   : "optional",
                     "ssl-server-verify-mode"   : "required",
                     "ssl-verify-flags"  : "X509_STRICT",
-                    "ssl-check-hostname": False,
-                    "ssl-server-hostname": "localhost",
+                    "ssl-check-hostname": True,
+                    "ssl-server-hostname": "",
                     "ssl-options"       : "ALL,NO_COMPRESSION",
                     "quality"           : 0,
                     "min-quality"       : 30,
@@ -952,7 +982,7 @@ def get_defaults():
                     "min-speed"         : 30,
                     "compression_level" : 1,
                     "dpi"               : 0,
-                    "file-size-limit"   : 100,
+                    "file-size-limit"   : "100M",
                     "idle-timeout"      : 0,
                     "server-idle-timeout" : 0,
                     "sync-xvfb"         : 0,
@@ -965,9 +995,9 @@ def get_defaults():
                     "daemon"            : CAN_DAEMONIZE,
                     "start-via-proxy"   : False,
                     "attach"            : None,
-                    "use-display"       : "no",
+                    "use-display"       : "auto",
                     "fake-xinerama"     : fake_xinerama,
-                    "resize-display"    : not OSX and not WIN32,
+                    "resize-display"    : ["no", "yes"][not OSX and not WIN32],
                     "tray"              : True,
                     "pulseaudio"        : DEFAULT_PULSEAUDIO,
                     "dbus-proxy"        : not OSX and not WIN32,
@@ -1006,8 +1036,10 @@ def get_defaults():
                     "modal-windows"     : False,
                     "bandwidth-detection" : True,
                     "ssh-upgrade"       : True,
+                    "splash"            : None,
                     "pulseaudio-configure-commands"  : [" ".join(x) for x in DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS],
                     "socket-dirs"       : get_socket_dirs(),
+                    "client-socket-dirs" : get_client_socket_dirs(),
                     "remote-xpra"       : get_remote_run_xpra_scripts(),
                     "encodings"         : ["all"],
                     "proxy-video-encoders" : [],
@@ -1019,7 +1051,7 @@ def get_defaults():
                     "compressors"       : ["all"],
                     "packet-encoders"   : ["all"],
                     "key-shortcut"      : get_default_key_shortcuts(),
-                    "bind"              : bind_dirs,
+                    "bind"              : ["auto"],
                     "bind-vsock"        : [],
                     "bind-tcp"          : [],
                     "bind-udp"          : [],
@@ -1117,6 +1149,8 @@ def print_number(i, auto_value=0):
     return str(i)
 
 def parse_with_unit(numtype, v, subunit="bps", min_value=250000):
+    if isinstance(v, int):
+        return v
     #special case for bandwidth-limit, which can be specified using units:
     try:
         v = str(v).lower().strip()
@@ -1146,7 +1180,10 @@ def parse_with_unit(numtype, v, subunit="bps", min_value=250000):
         raise InitException("invalid value for %s '%s': %s" % (numtype, v, e)) from None
 
 
-def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_validation={}):
+def validate_config(d=None, discard=NO_FILE_OPTIONS, extras_types=None, extras_validation=None):
+    return do_validate_config(d or {}, discard, extras_types or {}, extras_validation or {})
+
+def do_validate_config(d:dict, discard, extras_types:dict, extras_validation:dict):
     """
         Validates all the options given in a dict with fields as keys and
         strings or arrays of strings as values.
@@ -1204,16 +1241,20 @@ def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_valid
     return nd
 
 
-def make_defaults_struct(extras_defaults={}, extras_types={}, extras_validation={}, username="", uid=getuid(), gid=getgid()):
+def make_defaults_struct(extras_defaults=None, extras_types=None, extras_validation=None, username="", uid=getuid(), gid=getgid()):
+    return do_make_defaults_struct(extras_defaults or {}, extras_types or {}, extras_validation or {}, username, uid, gid)
+
+def do_make_defaults_struct(extras_defaults:dict, extras_types:dict, extras_validation:dict, username:str, uid:int, gid:int):
     #populate config with default values:
     if not username and uid:
         username = get_username_for_uid(uid)
     defaults = read_xpra_defaults(username, uid, gid)
     return dict_to_validated_config(defaults, extras_defaults, extras_types, extras_validation)
 
-def dict_to_validated_config(d={}, extras_defaults={}, extras_types={}, extras_validation={}):
+def dict_to_validated_config(d:dict, extras_defaults=None, extras_types=None, extras_validation=None):
     options = get_defaults().copy()
-    options.update(extras_defaults)
+    if extras_defaults:
+        options.update(extras_defaults)
     #parse config:
     validated = validate_config(d, extras_types=extras_types, extras_validation=extras_validation)
     options.update(validated)
@@ -1295,11 +1336,14 @@ def fixup_video_all_or_none(options):
     options.proxy_video_encoders = getlist(pvestr, "proxy video encoders", "HARDWARE_ENCODER_OPTIONS")
 
 def fixup_socketdirs(options):
-    if isinstance(options.socket_dirs, str):
-        options.socket_dirs = options.socket_dirs.split(os.path.pathsep)
-    else:
-        assert isinstance(options.socket_dirs, (list, tuple))
-        options.socket_dirs = [v for x in options.socket_dirs for v in x.split(os.path.pathsep)]
+    for option_name in ("socket_dirs", "client_socket_dirs"):
+        value = getattr(options, option_name)
+        if isinstance(value, str):
+            value = value.split(os.path.pathsep)
+        else:
+            assert isinstance(options.socket_dirs, (list, tuple))
+            value = [v for x in value for v in x.split(os.path.pathsep)]
+        setattr(options, option_name, value)
 
 def fixup_pings(options):
     #pings used to be a boolean, True mapped to "5"
@@ -1318,7 +1362,7 @@ def fixup_pings(options):
 
 def fixup_encodings(options):
     try:
-        from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
+        from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER
     except ImportError:
         return
     RENAME = {"jpg" : "jpeg"}
@@ -1327,7 +1371,7 @@ def fixup_encodings(options):
     estr = _csvstr(options.encodings)
     if estr=="all":
         #replace with an actual list
-        options.encodings = list(PREFERED_ENCODING_ORDER)
+        options.encodings = list(PREFERRED_ENCODING_ORDER)
         return
     encodings = [RENAME.get(x, x) for x in _nodupes(estr)]
     if "rgb" in encodings:
@@ -1397,6 +1441,11 @@ def fixup_clipboard(options):
         options.clipboard_direction = "disabled"
 
 def abs_paths(options):
+    ew = options.exec_wrapper
+    if ew and not os.path.isabs(ew):
+        ew = which(ew)
+        if ew:
+            options.exec_wrapper = ew
     #convert to absolute paths before we daemonize
     for k in ("clipboard-filter-file",
               "tcp-encryption-keyfile", "encryption-keyfile",
