@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,6 +11,7 @@ from xpra.platform.features import COMMAND_SIGNALS
 from xpra.child_reaper import getChildReaper, reaper_cleanup
 from xpra.os_util import monotonic_time, bytestostr, OSX, WIN32, POSIX
 from xpra.util import envint, csv
+from xpra.make_thread import start_thread
 from xpra.scripts.parsing import parse_env, get_subcommands
 from xpra.server.server_util import source_env
 from xpra.server import EXITING_CODE
@@ -52,6 +53,7 @@ class ChildCommandServer(StubServerMixin):
         self.start_on_last_client_exit = []
         self.start_child_on_last_client_exit = []
         self.exit_with_children = False
+        self.children_count = 0
         self.start_after_connect_done = False
         self.start_new_commands = False
         self.source_env = {}
@@ -104,7 +106,9 @@ class ChildCommandServer(StubServerMixin):
                 log.error("Error setting up menu watcher:")
                 log.error(" %s", e)
             from xpra.platform.xposix.xdg_helper import load_xdg_menu_data
-            load_xdg_menu_data()
+            #start loading in a thread,
+            #so server startup can complete:
+            start_thread(load_xdg_menu_data, "load-xdg-menu-data", True)
 
 
     def setup_menu_watcher(self):
@@ -132,7 +136,7 @@ class ChildCommandServer(StubServerMixin):
         watched = []
         for data_dir in data_dirs:
             menu_dir = os.path.join(data_dir, "applications")
-            if not os.path.exists(menu_dir):
+            if not os.path.exists(menu_dir) or menu_dir in watched:
                 continue
             wdd = self.watch_manager.add_watch(menu_dir, mask)
             watched.append(menu_dir)
@@ -183,24 +187,28 @@ class ChildCommandServer(StubServerMixin):
             return None
         if not self.start_new_commands:
             return None
-        if POSIX or OSX:
+        if OSX:
+            return None
+        if POSIX:
             from xpra.platform.xposix.xdg_helper import load_xdg_menu_data
             return load_xdg_menu_data(force_reload)
-        elif WIN32:
+        if WIN32:
             from xpra.platform.win32.menu_helper import load_menu
             return load_menu()
+        log.error("Error: unsupported platform!")
+        return None
 
     def get_caps(self, source) -> dict:
         caps = {}
         if not source:
             return caps
         #don't assume we have a real ClientConnection object:
-        if getattr(source, "wants_features", False):
+        if getattr(source, "wants_features", False) and getattr(source, "ui_client", False):
             caps["xdg-menu"] = {}
             if not source.xdg_menu_update:
                 #we have to send it now:
                 xdg_menu = self._get_xdg_menu_data()
-                log.warn("%i entries sent in hello", len(xdg_menu))
+                log("%i entries sent in hello", len(xdg_menu or ()))
                 if xdg_menu:
                     l = len(str(xdg_menu))
                     #arbitrary: don't use more than half
@@ -216,9 +224,16 @@ class ChildCommandServer(StubServerMixin):
 
     def send_initial_data(self, ss, caps, send_ui, share_count):
         if ss.xdg_menu_update:
-            xdg_menu = self._get_xdg_menu_data() or {}
-            log("%i entries sent in initial data", len(xdg_menu))
-            ss.send_setting_change("xdg-menu", xdg_menu)
+            #this method may block if the menus are still being loaded,
+            #so do it in a throw-away thread:
+            start_thread(self.send_xdg_menu_data, "send-xdg-menu-data", True, (ss,))
+
+    def send_xdg_menu_data(self, ss):
+        if ss.is_closed():
+            return
+        xdg_menu = self._get_xdg_menu_data() or {}
+        log("%i entries sent in initial data", len(xdg_menu))
+        ss.send_setting_change("xdg-menu", xdg_menu)
 
     def schedule_xdg_menu_reload(self):
         xmrt = self.xdg_menu_reload_timer
@@ -233,6 +248,7 @@ class ChildCommandServer(StubServerMixin):
         for source in tuple(self._server_sources.values()):
             if source.xdg_menu_update:
                 source.send_setting_change("xdg-menu", xdg_menu or {})
+        return False
 
     def get_info(self, _proto) -> dict:
         info = {
@@ -307,23 +323,28 @@ class ChildCommandServer(StubServerMixin):
             self.idle_add(self.guess_session_name, procs)
 
     def start_command(self, name, child_cmd, ignore=False, callback=None, use_wrapper=True, shell=False, **kwargs):
-        log("start_command%s exec_wrapper=%s",
-            (name, child_cmd, ignore, callback, use_wrapper, shell, kwargs), self.exec_wrapper)
         from subprocess import Popen
         env = self.get_child_env()
+        log("start_command%s exec_wrapper=%s, exec_cwd=%s",
+            (name, child_cmd, ignore, callback, use_wrapper, shell, kwargs), self.exec_wrapper, self.exec_cwd)
+        real_cmd = []
         try:
             real_cmd = self.get_full_child_command(child_cmd, use_wrapper)
             log("full child command(%s, %s)=%s", child_cmd, use_wrapper, real_cmd)
             proc = Popen(real_cmd, env=env, shell=shell, cwd=self.exec_cwd, **kwargs)
             procinfo = self.add_process(proc, name, real_cmd, ignore=ignore, callback=callback)
             log("pid(%s)=%s", real_cmd, proc.pid)
+            log.info("started command '%s' with pid %s", " ".join(bytestostr(x) for x in real_cmd), proc.pid)
             if not ignore:
-                log.info("started command '%s' with pid %s", " ".join(real_cmd), proc.pid)
+                self.children_count += 1
             self.children_started.append(procinfo)
             return proc
         except (OSError, ValueError) as e:
             log("start_command%s", (name, child_cmd, ignore, callback, use_wrapper, shell, kwargs), exc_info=True)
             log.error("Error spawning child '%s':" % (child_cmd, ))
+            if len(real_cmd)>1:
+                log.error(" using command:")
+                log.error(" %s", " ".join("'%s'" % x for x in real_cmd))
             log.error(" %s" % (e,))
             return None
 
@@ -336,7 +357,7 @@ class ChildCommandServer(StubServerMixin):
 
     def reaper_exit_check(self):
         log("reaper_exit_check() exit_with_children=%s", self.exit_with_children)
-        if self.exit_with_children:
+        if self.exit_with_children and self.children_count:
             log.info("all children have exited and --exit-with-children was specified, exiting")
             self.idle_add(self.clean_quit)
 
