@@ -8,10 +8,10 @@ import socket
 from time import sleep, time
 from queue import Queue
 
-from xpra.net import compression
 from xpra.net.net_util import get_network_caps
 from xpra.net.compression import Compressed, compressed_wrapper
 from xpra.net.protocol import Protocol
+from xpra.net.common import MAX_PACKET_SIZE
 from xpra.codecs.loader import load_codec, get_codec
 from xpra.codecs.image_wrapper import ImageWrapper
 from xpra.codecs.video_helper import getVideoHelper, PREFERRED_ENCODER_ORDER
@@ -37,7 +37,7 @@ PROXY_QUEUE_SIZE = envint("XPRA_PROXY_QUEUE_SIZE", 10)
 #for testing only: passthrough as RGB:
 PASSTHROUGH_RGB = envbool("XPRA_PROXY_PASSTHROUGH_RGB", False)
 VIDEO_TIMEOUT = 5                  #destroy video encoder after N seconds of idle state
-LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
+LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", False)
 PASSTHROUGH_AUTH = envbool("XPRA_PASSTHROUGH_AUTH", True)
 
 PING_INTERVAL = max(1, envint("XPRA_PROXY_PING_INTERVAL", 5))*1000
@@ -91,10 +91,13 @@ class ProxyInstance:
     def is_alive(self):
         return not self.exit
 
-    def run(self):
+    def log_start(self):
+        assert self.client_protocol and self.server_protocol
         log.info("started %s", self)
         log.info(" for client %s", self.client_protocol._conn)
         log.info(" and server %s", self.server_protocol._conn)
+
+    def run(self):
         self.video_init()
 
         #setup protocol wrappers:
@@ -171,7 +174,6 @@ class ProxyInstance:
             for x in reasons:
                 log.info(" %s", x)
             self.exit = True
-            log.info("stopping %s", self)
         self.cancel_client_ping_timer()
         self.cancel_server_ping_timer()
         self.stop_encode_thread()
@@ -301,7 +303,7 @@ class ProxyInstance:
         #server wants a packet
         p = self.client_packets.get()
         s = self.client_packets.qsize()
-        log("sending to client: %s (queue size=%i)", p[0], s)
+        log("sending to client: %s (queue size=%i)", bytestostr(p[0]), s)
         return p, None, None, None, True, s>0
 
     def process_client_packet(self, proto, packet):
@@ -360,7 +362,7 @@ class ProxyInstance:
         #server wants a packet
         p = self.server_packets.get()
         s = self.server_packets.qsize()
-        log("sending to server: %s (queue size=%i)", p[0], s)
+        log("sending to server: %s (queue size=%i)", bytestostr(p[0]), s)
         return p, None, None, None, True, s>0
 
 
@@ -371,9 +373,9 @@ class ProxyInstance:
                 packet[index] = strtobytes(data)
                 return
             #this is ugly and not generic!
-            zlib = compression.use_zlib and self.caps.boolget("zlib", True)
-            lz4 = compression.use_lz4 and self.caps.boolget("lz4", False)
-            lzo = compression.use_lzo and self.caps.boolget("lzo", False)
+            zlib = self.caps.boolget("zlib", True)
+            lz4 = self.caps.boolget("lz4", False)
+            lzo = self.caps.boolget("lzo", False)
             if zlib or lz4 or lzo:
                 packet[index] = compressed_wrapper(name, data, zlib=zlib, lz4=lz4, lzo=lzo, can_inline=False)
             else:
@@ -461,12 +463,7 @@ class ProxyInstance:
                 auth_caps = new_cipher_caps(self.client_protocol, self.cipher, self.encryption_key, padding_options)
                 caps.update(auth_caps)
             #may need to bump packet size:
-            proto.max_packet_size = maxw*maxh*4*4
-            file_transfer = self.caps.boolget("file-transfer") and c.boolget("file-transfer")
-            file_size_limit = max(self.caps.intget("file-size-limit"), c.intget("file-size-limit"))
-            file_max_packet_size = int(file_transfer) * (1024 + file_size_limit*1024*1024)
-            self.client_protocol.max_packet_size = max(self.client_protocol.max_packet_size, file_max_packet_size)
-            self.server_protocol.max_packet_size = max(self.server_protocol.max_packet_size, file_max_packet_size)
+            proto.max_packet_size = max(MAX_PACKET_SIZE, maxw*maxh*4*4)
             packet = ("hello", caps)
         elif packet_type=="ping_echo" and self.server_ping_timer and len(packet)>=7 and packet[6]==strtobytes(self.uuid):
             #this is one of our ping packets:
