@@ -16,9 +16,10 @@ from xpra.util import (
     )
 from xpra.os_util import (
     get_username_for_uid, get_groups, get_home_for_uid, bytestostr,
-    getuid, getgid, WIN32, POSIX,
-    monotonic_time,
+    getuid, getgid, WIN32, POSIX, OSX,
+    monotonic_time, umask_context, get_group_id,
     )
+from xpra.net.socket_util import SOCKET_DIR_MODE, SOCKET_DIR_GROUP
 from xpra.server.server_core import ServerCore
 from xpra.server.control_command import ArgsControlCommand, ControlError
 from xpra.child_reaper import getChildReaper
@@ -49,7 +50,7 @@ if WIN32:
     #DEFAULT_ENV_WHITELIST = "ALLUSERSPROFILE,APPDATA,COMMONPROGRAMFILES,COMMONPROGRAMFILES(X86),COMMONPROGRAMW6432,COMPUTERNAME,COMSPEC,FP_NO_HOST_CHECK,LOCALAPPDATA,NUMBER_OF_PROCESSORS,OS,PATH,PATHEXT,PROCESSOR_ARCHITECTURE,PROCESSOR_ARCHITECTURE,PROCESSOR_IDENTIFIER,PROCESSOR_LEVEL,PROCESSOR_REVISION,PROGRAMDATA,PROGRAMFILES,PROGRAMFILES(X86),PROGRAMW6432,PSMODULEPATH,PUBLIC,SYSTEMDRIVE,SYSTEMROOT,TEMP,TMP,USERDOMAIN,WORKGROUP,USERNAME,USERPROFILE,WINDIR,XPRA_REDIRECT_OUTPUT,XPRA_LOG_FILENAME,XPRA_ALL_DEBUG"
     DEFAULT_ENV_WHITELIST = "*"
 else:
-    DEFAULT_ENV_WHITELIST = "LANG,HOSTNAME,PWD,TERM,SHELL,SHLVL,PATH"
+    DEFAULT_ENV_WHITELIST = "LANG,HOSTNAME,PWD,TERM,SHELL,SHLVL,PATH,USER,HOME"
 ENV_WHITELIST = os.environ.get("XPRA_PROXY_ENV_WHITELIST", DEFAULT_ENV_WHITELIST).split(",")
 
 
@@ -104,6 +105,46 @@ class ProxyServer(ServerCore):
         from xpra.version_util import get_platform_info
         get_platform_info()
         self.child_reaper = getChildReaper()
+        self.create_system_dir(opts.system_proxy_socket)
+
+    def create_system_dir(self, sps):
+        if not POSIX or OSX or not sps:
+            return
+        xpra_group_id = get_group_id(SOCKET_DIR_GROUP)
+        if sps.startswith("/run/xpra") or sps.startswith("/var/run/xpra"):
+            #create the directory and verify its permissions
+            #which should have been set correctly by tmpfiles.d,
+            #but may have been set wrong if created by systemd's socket activation instead
+            d = sps.split("/xpra")[0]+"/xpra"
+            try:
+                if os.path.exists(d):
+                    stat = os.stat(d)
+                    mode = stat.st_mode
+                    if (mode & SOCKET_DIR_MODE)!=SOCKET_DIR_MODE:
+                        log.warn("Warning: invalid permissions on '%s' : %s", d, oct(mode))
+                        mode = mode | SOCKET_DIR_MODE
+                        log.warn(" changing to %s", oct(mode))
+                        os.chmod(d, mode)
+                    if xpra_group_id>=0 and stat.st_gid!=xpra_group_id:
+                        import grp
+                        group = grp.getgrgid(stat.st_gid)[0]
+                        log.warn("Warning: invalid group on '%s': %s", d, group)
+                        log.warn(" changing to '%s'", SOCKET_DIR_GROUP)
+                        os.lchown(d, stat.st_uid, xpra_group_id)
+                else:
+                    log.info("creating '%s' with permissions %s and group '%s'",
+                             d, oct(SOCKET_DIR_MODE), SOCKET_DIR_GROUP)
+                    with umask_context(0):
+                        os.mkdir(d, SOCKET_DIR_MODE)
+                    stat = os.stat(d)
+                    if xpra_group_id>=0 and stat.st_gid!=xpra_group_id:
+                        os.lchown(d, stat.st_uid, xpra_group_id)
+                mode = os.stat(d).st_mode
+                log("%s permissions: %s", d, oct(mode))
+            except OSError as e:
+                log("create_system_dir()", exc_info=True)
+                log.error("Error: failed to create or change the permissions on '%s':", d)
+                log.error(" %s", e)
 
     def init_control_commands(self):
         ServerCore.init_control_commands(self)
@@ -129,7 +170,7 @@ class ProxyServer(ServerCore):
 
     def _process_proxy_shutdown_server(self, proto, _packet):
         assert proto in self._requests
-        self.quit(False)
+        self.clean_quit(False)
 
 
     def print_screen_info(self):
@@ -193,7 +234,7 @@ class ProxyServer(ServerCore):
 
     def cleanup(self):
         self.stop_all_proxies()
-        ServerCore.cleanup(self)
+        super().cleanup()
         start = monotonic_time()
         live = True
         log("cleanup() proxy instances: %s", self.instances)
@@ -211,11 +252,13 @@ class ProxyServer(ServerCore):
 
     def do_quit(self):
         self.main_loop.quit()
-        log.info("Proxy Server process ended")
         #from now on, we can't rely on the main loop:
         from xpra.os_util import register_SIGUSR_signals
         register_SIGUSR_signals()
 
+    def log_closing_message(self):
+        log.info("Proxy Server process ended")
+
 
     def verify_connection_accepted(self, protocol):
         #if we start a proxy, the protocol will be closed
@@ -364,12 +407,15 @@ class ProxyServer(ServerCore):
             authlog("proxy_session: proxy-virtual-display=%s (ignored), user specified display=%s, found displays=%s",
                     proxy_virtual_display, display, displays)
             if display==proxy_virtual_display:
-                disconnect(SESSION_NOT_FOUND, "invalid display")
+                disconnect(SESSION_NOT_FOUND, "invalid display: proxy display")
                 return
             if display:
                 if display not in displays:
-                    disconnect(SESSION_NOT_FOUND, "display '%s' not found" % display)
-                    return
+                    if (":%s" % display) in displays:
+                        display = ":%s" % display
+                    else:
+                        disconnect(SESSION_NOT_FOUND, "display '%s' not found" % display)
+                        return
             else:
                 if len(displays)!=1:
                     disconnect(SESSION_NOT_FOUND,
