@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 from time import sleep
 from threading import Event
 from collections import deque
@@ -87,9 +88,6 @@ class ClientConnection(StubSourceMixin):
         self.server_bandwidth_limit = bandwidth_limit
         self.bandwidth_detection = bandwidth_detection
 
-        #these statistics are shared by all WindowSource instances:
-        self.statistics = GlobalPerformanceStatistics()
-
     def run(self):
         # ready for processing:
         self.queue_encode = self.start_queue_encode
@@ -101,9 +99,6 @@ class ClientConnection(StubSourceMixin):
     def init_state(self):
         self.hello_sent = False
         self.info_namespace = False
-        self.send_notifications = False
-        self.send_notifications_actions = False
-        self.notification_callbacks = {}
         self.share = False
         self.lock = False
         self.control_commands = ()
@@ -112,6 +107,9 @@ class ClientConnection(StubSourceMixin):
         self.soft_bandwidth_limit = self.bandwidth_limit
         self.bandwidth_warnings = True
         self.bandwidth_warning_time = 0
+        self.client_connection_data = {}
+        self.adapter_type = ""
+        self.jitter = 0
         #what we send back in hello packet:
         self.ui_client = True
         self.wants_aliases = True
@@ -121,6 +119,8 @@ class ClientConnection(StubSourceMixin):
         self.wants_display = True
         self.wants_events = False
         self.wants_default_cursor = False
+        #these statistics are shared by all WindowSource instances:
+        self.statistics = GlobalPerformanceStatistics()
 
 
     def is_closed(self) -> bool:
@@ -133,6 +133,15 @@ class ClientConnection(StubSourceMixin):
         self.statistics.reset(0)
 
 
+    def may_notify(self, *args, **kwargs):
+        #fugly workaround,
+        #MRO is depth first and would hit the default implementation
+        #instead of the mixin unless we force it:
+        notification_mixin = sys.modules.get("xpra.server.source.notification_mixin")
+        if notification_mixin and isinstance(self, notification_mixin.NotificationMixin):
+            notification_mixin.NotificationMixin.may_notify(self, *args, **kwargs)
+
+
     def compressed_wrapper(self, datatype, data, min_saving=128):
         if self.zlib or self.lz4 or self.lzo:
             cw = compressed_wrapper(datatype, data, zlib=self.zlib, lz4=self.lz4, lzo=self.lzo, can_inline=False)
@@ -191,9 +200,6 @@ class ClientConnection(StubSourceMixin):
     def parse_client_caps(self, c : typedict):
         #general features:
         self.info_namespace = c.boolget("info-namespace")
-        self.send_notifications = c.boolget("notifications")
-        self.send_notifications_actions = c.boolget("notifications.actions")
-        log("notifications=%s, actions=%s", self.send_notifications, self.send_notifications_actions)
         self.share = c.boolget("share")
         self.lock = c.boolget("lock")
         self.control_commands = c.strtupleget("control_commands")
@@ -206,18 +212,15 @@ class ClientConnection(StubSourceMixin):
         if self.bandwidth_detection:
             self.bandwidth_detection = c.boolget("bandwidth-detection", True)
         self.client_connection_data = c.dictget("connection-data", {})
-        self.jitter = typedict(self.client_connection_data).intget("jitter", 0)
+        ccd = typedict(self.client_connection_data)
+        self.adapter_type = ccd.strget("adapter-type", "")
+        self.jitter = ccd.intget("jitter", 0)
         bandwidthlog("server bandwidth-limit=%s, client bandwidth-limit=%s, value=%s, detection=%s",
                      server_bandwidth_limit, bandwidth_limit, self.bandwidth_limit, self.bandwidth_detection)
 
         if getattr(self, "mmap_size", 0)>0:
             log("mmap enabled, ignoring bandwidth-limit")
             self.bandwidth_limit = 0
-        #adjust max packet size if file transfers are enabled:
-        #TODO: belongs in mixin:
-        file_transfer = getattr(self, "file_transfer", None)
-        if file_transfer:
-            self.protocol.max_packet_size = max(self.protocol.max_packet_size, self.file_size_limit*1024*1024)
 
     def get_socket_bandwidth_limit(self) -> int:
         p = self.protocol
@@ -355,6 +358,7 @@ class ClientConnection(StubSourceMixin):
                 "counter"           : self.counter,
                 "hello-sent"        : self.hello_sent,
                 "jitter"            : self.jitter,
+                "adapter-type"      : self.adapter_type,
                 "bandwidth-limit"   : {
                     "detection"     : self.bandwidth_detection,
                     "actual"        : self.soft_bandwidth_limit or 0,
@@ -372,7 +376,6 @@ class ClientConnection(StubSourceMixin):
         info = {
             "lock"  : bool(self.lock),
             "share" : bool(self.share),
-            "notifications" : self.send_notifications,
             }
         return info
 
@@ -382,6 +385,7 @@ class ClientConnection(StubSourceMixin):
 
 
     def send_setting_change(self, setting, value):
+        #we always subclass InfoMixin which defines "client_setting_change":
         if self.client_setting_change:
             self.send_more("setting-change", setting, value)
 
@@ -391,56 +395,6 @@ class ClientConnection(StubSourceMixin):
             self.send_more("server-event", *args)
 
 
-    ######################################################################
-    # notifications:
-    # Utility functions for mixins (makes notifications optional)
-    def may_notify(self, nid, summary, body, actions=(), hints=None, expire_timeout=10*1000,
-                   icon_name=None, user_callback=None):
-        try:
-            from xpra.platform.paths import get_icon_filename
-            from xpra.notifications.common import parse_image_path
-        except ImportError as e:
-            notifylog("not sending notification: %s", e)
-        else:
-            icon_filename = get_icon_filename(icon_name)
-            icon = parse_image_path(icon_filename) or ""
-            self.notify("", nid, "Xpra", 0, "",
-                        summary, body, actions, hints or {},
-                        expire_timeout, icon, user_callback)
-
-    def notify(self, dbus_id, nid, app_name, replaces_nid, app_icon,
-               summary, body, actions, hints, expire_timeout, icon, user_callback=None):
-        args = (dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon)
-        notifylog("notify%s types=%s", args, tuple(type(x) for x in args))
-        if not self.send_notifications:
-            notifylog("client %s does not support notifications", self)
-            return False
-        if self.suspended:
-            notifylog("client %s is suspended, notification not sent", self)
-            return False
-        if user_callback:
-            self.notification_callbacks[nid] = user_callback
-        #this is one of the few places where we actually do care about character encoding:
-        try:
-            summary = summary.encode("utf8")
-        except UnicodeEncodeError:
-            summary = str(summary)
-        try:
-            body = body.encode("utf8")
-        except UnicodeEncodeError:
-            body = str(body)
-        if self.hello_sent:
-            #Warning: actions and hints are send last because they were added later (in version 2.3)
-            self.send_async("notify_show", dbus_id, nid, app_name, replaces_nid, app_icon,
-                            summary, body, expire_timeout, icon, actions, hints)
-        return True
-
-    def notify_close(self, nid : int):
-        if not self.send_notifications or self.suspended  or not self.hello_sent:
-            return
-        self.send_more("notify_close", nid)
-
-
     def set_deflate(self, level : int):
         self.send("set_deflate", level)
 
