@@ -16,6 +16,7 @@ from xpra.util import typedict, envint, envbool, DEFAULT_METADATA_SUPPORTED, XPR
 from xpra.log import Logger
 
 log = Logger("server")
+focuslog = Logger("focus")
 cursorlog = Logger("cursor")
 metalog = Logger("metadata")
 bandwidthlog = Logger("bandwidth")
@@ -51,6 +52,10 @@ class WindowsMixin(StubSourceMixin):
         self.get_window_id = None
         self.window_filters = []
         self.readonly = False
+        #duplicated from encodings:
+        self.global_batch_config = None
+        #duplicated from clientconnection:
+        self.statistics = None
 
     def init_from(self, _protocol, server):
         self.get_transient_for  = server.get_transient_for
@@ -73,6 +78,7 @@ class WindowsMixin(StubSourceMixin):
         self.pointer_grabs = False
         self.window_min_size = 0, 0
         self.window_max_size = 0, 0
+        self.window_restack = False
         self.system_tray = False
         self.metadata_supported = ()
 
@@ -127,8 +133,7 @@ class WindowsMixin(StubSourceMixin):
 
 
     def parse_client_caps(self, c):
-        #self.ui_client = c.boolget("ui_client", True)
-        self.send_windows = self.ui_client and c.boolget("windows", True)
+        self.send_windows = c.boolget("ui_client", True) and c.boolget("windows", True)
         self.pointer_grabs = c.boolget("pointer.grabs")
         self.send_cursors = self.send_windows and c.boolget("cursors")
         self.cursor_encodings = c.strtupleget("encodings.cursor")
@@ -138,10 +143,9 @@ class WindowsMixin(StubSourceMixin):
         self.window_frame_sizes = typedict(c.dictget("window.frame_sizes", {}))
         self.window_min_size = c.inttupleget("window.min-size", (0, 0))
         self.window_max_size = c.inttupleget("window.max-size", (0, 0))
-        log("cursors=%s (encodings=%s), bell=%s, notifications=%s",
-            self.send_cursors, self.cursor_encodings, self.send_bell, self.send_notifications)
-        log("client uuid %s", self.uuid)
-
+        self.window_restack = c.boolget("window.restack", False)
+        log("cursors=%s (encodings=%s), bell=%s",
+            self.send_cursors, self.cursor_encodings, self.send_bell)
         #window filters:
         try:
             for object_name, property_name, operator, value in c.tupleget("window-filters"):
@@ -188,10 +192,13 @@ class WindowsMixin(StubSourceMixin):
                                "packet_queue"           : {"size" : {"current" : len(self.packet_queue)}},
                                "packet_queue_pixels"    : pqpi,
                                },
-                "batch"     : self.global_batch_config.get_info(),
-                }
-        info.update(self.statistics.get_info())
-
+            }
+        gbc = self.global_batch_config
+        if gbc:
+            info["batch"] = self.global_batch_config.get_info()
+        s = self.statistics
+        if s:
+            info.update(s.get_info())
         if self.window_sources:
             total_pixels = 0
             total_time = 0.0
@@ -234,8 +241,9 @@ class WindowsMixin(StubSourceMixin):
         if not self.send_cursors or self.suspended or not self.hello_sent:
             return
         #if not pending already, schedule it:
-        if not self.cursor_timer:
-            delay = max(10, int(self.global_batch_config.delay/4))
+        gbc = self.global_batch_config
+        if not self.cursor_timer and gbc:
+            delay = max(10, int(gbc.delay/4))
             self.cursor_timer = self.timeout_add(delay, self.do_send_cursor, delay)
 
     def cancel_cursor_timer(self):
@@ -466,6 +474,20 @@ class WindowsMixin(StubSourceMixin):
         if ws:
             ws.unmap()
 
+    def restack_window(self, wid, window, detail, sibling):
+        focuslog("restack_window%s", (wid, window, detail, sibling))
+        if not self.can_send_window(window):
+            return
+        if not self.window_restack:
+            #older clients can only handle "raise-window"
+            if detail!=0:
+                return
+            self.send_async("raise-window", wid)
+            return
+        sibling_wid = 0
+        if sibling:
+            sibling_wid = self.get_window_id(sibling)
+        self.send_async("restack-window", wid, detail, sibling_wid)
 
     def raise_window(self, wid, window):
         if not self.can_send_window(window):
@@ -476,14 +498,10 @@ class WindowsMixin(StubSourceMixin):
         """ The given window is gone, ensure we free all the related resources """
         if not self.can_send_window(window):
             return
-        ws = self.window_sources.get(wid)
+        ws = self.window_sources.pop(wid, None)
         if ws:
-            del self.window_sources[wid]
             ws.cleanup()
-        try:
-            del self.calculate_window_pixels[wid]
-        except KeyError:
-            pass
+        self.calculate_window_pixels.pop(wid, None)
 
 
     def refresh(self, wid, window, opts):
@@ -532,7 +550,7 @@ class WindowsMixin(StubSourceMixin):
                               self.idle_add, self.timeout_add, self.source_remove,
                               ww, wh,
                               self.record_congestion_event, self.encode_queue_size,
-                              self.call_in_encode_thread, self.queue_packet, self.compressed_wrapper,
+                              self.call_in_encode_thread, self.queue_packet,
                               self.statistics,
                               wid, window, batch_config, self.auto_refresh_delay,
                               av_sync, av_sync_delay,
@@ -563,7 +581,9 @@ class WindowsMixin(StubSourceMixin):
             damage_options = options.copy()
         else:
             damage_options = {}
-        self.statistics.damage_last_events.append((wid, monotonic_time(), w*h))
+        s = self.statistics
+        if s:
+            s.damage_last_events.append((wid, monotonic_time(), w*h))
         ws = self.make_window_source(wid, window)
         ws.damage(x, y, w, h, damage_options)
 
