@@ -17,7 +17,7 @@ from xpra.gtk_common.gtk_util import get_default_root_window, GDKWindow
 from xpra.x11.common import Unmanageable
 from xpra.x11.gtk_x11.selection import ManagerSelection
 from xpra.x11.models.window import WindowModel, configure_bits
-from xpra.x11.gtk_x11.world_window import WorldWindow
+from xpra.x11.gtk_x11.world_window import WorldWindow, destroy_world_window
 from xpra.x11.gtk_x11.gdk_bindings import (
     add_event_receiver,                              #@UnresolvedImport
     add_fallback_receiver, remove_fallback_receiver, #@UnresolvedImport
@@ -137,8 +137,7 @@ DEFAULT_NET_SUPPORTED = [
         "_MOTIF_WM_INFO",
 
         "_NET_REQUEST_FRAME_EXTENTS",
-        # Not at all yet:
-        #"_NET_RESTACK_WINDOW",
+        "_NET_RESTACK_WINDOW",
         ]
 FRAME_EXTENTS = envbool("XPRA_FRAME_EXTENTS", True)
 if FRAME_EXTENTS:
@@ -369,7 +368,6 @@ class Wm(GObject.GObject):
         #   _NET_CURRENT_DESKTOP
         #   _NET_WM_PING responses
         # and maybe:
-        #   _NET_RESTACK_WINDOW
         #   _NET_WM_STATE
         log("do_xpra_client_message_event(%s)", event)
         if event.message_type=="_NET_SHOWING_DESKTOP":
@@ -406,6 +404,8 @@ class Wm(GObject.GObject):
         with xswallow:
             prop_del(self._ewmh_window, "_NET_SUPPORTING_WM_CHECK")
             prop_del(self._ewmh_window, "_NET_WM_NAME")
+        destroy_world_window()
+
 
     def do_child_map_request_event(self, event):
         log("Found a potential client")
