@@ -7,7 +7,7 @@
 from gi.repository import GObject
 
 from xpra.util import WORKSPACE_UNSET, WORKSPACE_ALL
-from xpra.x11.models.core import CoreX11WindowModel, xswallow
+from xpra.x11.models.core import CoreX11WindowModel, xswallow, Above, RESTACKING_STR
 from xpra.x11.bindings.window_bindings import X11WindowBindings, constants      #@UnresolvedImport
 from xpra.server.window.content_guesser import guess_content_type, get_content_type_properties
 from xpra.x11.gtk_x11.gdk_bindings import get_pywindow, get_pyatom              #@UnresolvedImport
@@ -17,7 +17,6 @@ from xpra.log import Logger
 log = Logger("x11", "window")
 workspacelog = Logger("x11", "window", "workspace")
 metalog = Logger("x11", "window", "metadata")
-geomlog = Logger("x11", "window", "geometry")
 
 
 dbus_helper = None
@@ -31,20 +30,22 @@ _NET_WM_STATE_REMOVE = 0
 _NET_WM_STATE_ADD    = 1
 _NET_WM_STATE_TOGGLE = 2
 STATE_STRING = {
-            _NET_WM_STATE_REMOVE    : "REMOVE",
-            _NET_WM_STATE_ADD       : "ADD",
-            _NET_WM_STATE_TOGGLE    : "TOGGLE",
-                }
+    _NET_WM_STATE_REMOVE    : "REMOVE",
+    _NET_WM_STATE_ADD       : "ADD",
+    _NET_WM_STATE_TOGGLE    : "TOGGLE",
+    }
 IconicState = constants["IconicState"]
 NormalState = constants["NormalState"]
 ICONIC_STATE_STRING = {
-                       IconicState  : "Iconic",
-                       NormalState  : "Normal",
-                       }
+    IconicState  : "Iconic",
+    NormalState  : "Normal",
+    }
 
 #add user friendly workspace logging:
-WORKSPACE_STR = {WORKSPACE_UNSET    : "UNSET",
-                 WORKSPACE_ALL      : "ALL"}
+WORKSPACE_STR = {
+    WORKSPACE_UNSET    : "UNSET",
+    WORKSPACE_ALL      : "ALL",
+    }
 def workspacestr(w):
     return WORKSPACE_STR.get(w, w)
 
@@ -386,23 +387,23 @@ class BaseWindowModel(CoreX11WindowModel):
 
 
     def _content_type_related_property_change(self, *_args):
-        #watch for changes to properties that are used to derive the content-type:
-        content_type = self.prop_get("_XPRA_CONTENT_TYPE", "latin1", True)
-        if content_type:
-            #the _XPRA_CONTENT_TYPE property takes precedence
-            return
-        content_type = guess_content_type(self)
-        metalog("guess_content_type(%s)=%s", self, content_type)
-        self._updateprop("content-type", content_type)
+        self._update_content_type()
 
     def _handle_xpra_content_type_change(self):
-        content_type = self.prop_get("_XPRA_CONTENT_TYPE", "latin1", True) or ""
-        metalog("content_type=%s", content_type)
+        self._update_content_type()
+
+    def _update_content_type(self):
+        #watch for changes to properties that are used to derive the content-type:
+        content_type = self.prop_get("_XPRA_CONTENT_TYPE", "latin1", True)
+        #the _XPRA_CONTENT_TYPE property takes precedence
         if not content_type:
             content_type = guess_content_type(self)
-            metalog("guess_content_type(%s)=%s", self, content_type)
+            if not content_type and self.is_tray():
+                content_type = "picture"
+        metalog("_update_content_type() %s", content_type)
         self._updateprop("content-type", content_type)
 
+
     def _handle_xpra_quality_change(self):
         quality = self.prop_get("_XPRA_QUALITY", "u32", True) or -1
         metalog("quality=%s", quality)
@@ -515,7 +516,7 @@ class BaseWindowModel(CoreX11WindowModel):
         #intercept state properties to route via update_state()
         if pspec.name in self._state_properties:
             #virtual property for WM_STATE:
-            self.update_state(pspec.name, value)
+            self.update_wm_state(pspec.name, value)
             return
         super().do_set_property(pspec, value)
 
@@ -556,7 +557,6 @@ class BaseWindowModel(CoreX11WindowModel):
         #   _NET_CURRENT_DESKTOP
         #   _NET_WM_PING responses
         # and maybe:
-        #   _NET_RESTACK_WINDOW
         #   _NET_WM_STATE (more fully)
 
         if event.message_type=="_NET_WM_STATE":
@@ -625,10 +625,13 @@ class BaseWindowModel(CoreX11WindowModel):
             log("_NET_WM_MOVERESIZE: %s", event)
             self.emit("initiate-moveresize", event)
             return True
-        if event.message_type=="_NET_ACTIVE_WINDOW" and event.data[0] in (0, 1):
+        if event.message_type=="_NET_ACTIVE_WINDOW":
+            #to filter based on the source indication:
+            #ACTIVE_WINDOW_SOURCE = tuple(int(x) for x in os.environ.get("XPRA_ACTIVE_WINDOW_SOURCE", "0,1").split(","))
+            #if event.data[0] in ACTIVE_WINDOW_SOURCE:
             log("_NET_ACTIVE_WINDOW: %s", event)
             self.set_active()
-            self.emit("raised", event)
+            self.emit("restack", Above, None)
             return True
         if event.message_type=="_NET_WM_DESKTOP":
             workspace = int(event.data[0])
@@ -663,8 +666,9 @@ class BaseWindowModel(CoreX11WindowModel):
         if event.message_type=="_NET_RESTACK_WINDOW":
             source = {1 : "application", 2 : "pager"}.get(event.data[0], "default (%s)" % event.data[0])
             sibling_window = event.data[1]
-            log.info("%s not handled yet, sent to window %#x for sibling %#x from %s",
-                     event.message_type, event.window, sibling_window, source)
+            log("%s sent to window %#x for sibling %#x from %s with detail=%s",
+                event.message_type, event.window, sibling_window, source, RESTACKING_STR.get(event.detail, event.detail))
+            self.emit("restack", event.detail, sibling_window)
             return True
         #TODO: maybe we should process _NET_MOVERESIZE_WINDOW here?
         # it may make sense to apply it to the client_window
