@@ -67,7 +67,7 @@ def configure_bits(value_mask):
     return "|".join(v for k,v in CW_MASK_TO_NAME.items() if k&value_mask)
 
 
-FORCE_XSETINPUTFOCUS = envbool("XPRA_FORCE_XSETINPUTFOCUS", False)
+FORCE_XSETINPUTFOCUS = envbool("XPRA_FORCE_XSETINPUTFOCUS", True)
 VALIDATE_CONFIGURE_REQUEST = envbool("XPRA_VALIDATE_CONFIGURE_REQUEST", False)
 CLAMP_OVERLAP = envint("XPRA_WINDOW_CLAMP_OVERLAP", 20)
 assert CLAMP_OVERLAP>=0
@@ -590,6 +590,9 @@ class WindowModel(BaseWindowModel):
         if event.value_mask & CWWidth or event.value_mask & CWHeight:
             self._updateprop("requested-size", (rw, rh))
 
+        if event.value_mask & CWStackMode:
+            self.emit("restack", event.detail, event.above)
+
         if VALIDATE_CONFIGURE_REQUEST:
             w, h = self.calc_constrained_size(w, h, hints)
         #update the geometry now, as another request may come in
@@ -726,10 +729,7 @@ class WindowModel(BaseWindowModel):
         sanitize_size_hints(hints)
         #we don't use the "size" attribute for anything yet,
         #and changes to this property could send us into a loop
-        try:
-            del hints["size"]
-        except KeyError:
-            pass
+        hints.pop("size", None)
         # Don't send out notify and ConfigureNotify events when this property
         # gets no-op updated -- some apps like FSF Emacs 21 like to update
         # their properties every time they see a ConfigureNotify, and this
