@@ -8,7 +8,7 @@
 
 from gi.repository import Gdk
 
-from xpra.util import csv, nonl, envbool, repr_ellipsized
+from xpra.util import csv, envbool
 from xpra.os_util import bytestostr
 from xpra.gtk_common.keymap import get_gtk_keymap
 from xpra.gtk_common.gtk_util import get_default_root_window
@@ -31,6 +31,7 @@ log = Logger("keyboard")
 X11Keyboard = X11KeyboardBindings()
 
 MAP_MISSING_MODIFIERS = envbool("XPRA_MAP_MISSING_MODIFIERS", True)
+SHIFT_LOCK = envbool("XPRA_SHIFT_LOCK", False)
 
 ALL_X11_MODIFIERS = {
                     "shift"     : 0,
@@ -47,7 +48,6 @@ class KeyboardConfig(KeyboardConfigBase):
     def __init__(self):
         KeyboardConfigBase.__init__(self)
         self.xkbmap_raw = False
-        self.xkbmap_print = None
         self.xkbmap_query = None
         self.xkbmap_query_struct = None
         self.xkbmap_mod_meanings = {}
@@ -117,7 +117,7 @@ class KeyboardConfig(KeyboardConfigBase):
             for mod, mod_name in self.xkbmap_mod_meanings.items():
                 modinfo[mod] = mod_name
         info["x11_keycode"] = self.xkbmap_x11_keycodes
-        for x in ("print", "layout", "variant", "mod_managed", "mod_pointermissing", "raw", "layout_groups"):
+        for x in ("layout", "variant", "mod_managed", "mod_pointermissing", "raw", "layout_groups"):
             v = getattr(self, "xkbmap_%s" % x)
             if v:
                 info[x] = v
@@ -145,7 +145,7 @@ class KeyboardConfig(KeyboardConfigBase):
                 setattr(self, prop, nv)
                 modded[prop] = nv
         #plain strings:
-        for x in ("print", "query"):
+        for x in ("query", ):
             parse_option(x, props.strget)
         #lists:
         parse_option("keycodes", props.tupleget)
@@ -173,7 +173,7 @@ class KeyboardConfig(KeyboardConfigBase):
         def hashadd(v):
             m.update(("/%s" % str(v)).encode("utf8"))
         m.update(KeyboardConfigBase.get_hash(self))
-        for x in (self.xkbmap_print, self.xkbmap_query, self.xkbmap_raw, \
+        for x in (self.xkbmap_query, self.xkbmap_raw, \
                   self.xkbmap_mod_meanings, self.xkbmap_mod_pointermissing, \
                   self.xkbmap_keycodes, self.xkbmap_x11_keycodes):
             hashadd(x)
@@ -298,6 +298,9 @@ class KeyboardConfig(KeyboardConfigBase):
 
     def set_layout(self, layout, variant, options):
         log("set_layout(%s, %s, %s)", layout, variant, options)
+        if layout=="en":
+            log.warn("Warning: invalid keyboard layout name '%s', using 'us' instead", layout)
+            layout = "us"
         if layout!=self.xkbmap_layout or variant!=self.xkbmap_variant or options!=self.xkbmap_options:
             self.xkbmap_layout = layout
             self.xkbmap_variant = variant
@@ -309,9 +312,9 @@ class KeyboardConfig(KeyboardConfigBase):
     def set_keymap(self, translate_only=False):
         if not self.enabled:
             return
-        log("set_keymap(%s) layout=%s, variant=%s, options=%s, print=%s, query=%s",
+        log("set_keymap(%s) layout=%r, variant=%r, options=%r, query=%r",
             translate_only, self.xkbmap_layout, self.xkbmap_variant, self.xkbmap_options,
-            nonl(self.xkbmap_print), nonl(self.xkbmap_query))
+            self.xkbmap_query)
         if translate_only:
             self.keycode_translation = set_keycode_translation(self.xkbmap_x11_keycodes, self.xkbmap_keycodes)
             self.add_gtk_keynames()
@@ -324,8 +327,8 @@ class KeyboardConfig(KeyboardConfigBase):
         with xlog:
             clean_keyboard_state()
             do_set_keymap(self.xkbmap_layout, self.xkbmap_variant, self.xkbmap_options,
-                          self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct)
-        log("set_keymap: xkbmap_print=%s, xkbmap_query=%s", nonl(self.xkbmap_print), nonl(self.xkbmap_query))
+                          self.xkbmap_query, self.xkbmap_query_struct)
+        log("set_keymap: xkbmap_query=%r", self.xkbmap_query)
         with xlog:
             #first clear all existing modifiers:
             clean_keyboard_state()
@@ -431,7 +434,7 @@ class KeyboardConfig(KeyboardConfigBase):
         self.keycode_mappings = get_keycode_mappings()
 
 
-    def do_get_keycode(self, client_keycode, keyname, pressed, modifiers, group):
+    def do_get_keycode(self, client_keycode, keyname, pressed, modifiers, keyval, keystr, group):
         if not self.enabled:
             log("ignoring keycode since keyboard is turned off")
             return -1, group
@@ -446,7 +449,7 @@ class KeyboardConfig(KeyboardConfigBase):
                 l = log
             l(msg, *args)
         def klog(msg, *args):
-            kmlog("do_get_keycode%s"+msg, (client_keycode, keyname, pressed, modifiers, group), *args)
+            kmlog("do_get_keycode%s"+msg, (client_keycode, keyname, pressed, modifiers, keyval, keystr, group), *args)
         keycode = None
         rgroup = group
         if self.xkbmap_query:
@@ -465,7 +468,8 @@ class KeyboardConfig(KeyboardConfigBase):
             """
             #non-native: try harder to find matching keysym
             #first, try to honour shift state:
-            shift = ("shift" in modifiers) ^ ("lock" in modifiers)
+            lock = ("lock" in modifiers) and (SHIFT_LOCK or (bool(keystr) and keystr.isalpha()))
+            shift = ("shift" in modifiers) ^ lock
             mode = 0
             numlock = 0
             numlock_modifier = None
@@ -496,12 +500,12 @@ class KeyboardConfig(KeyboardConfigBase):
                 if keycode:
                     keysyms = self.keycode_mappings.get(keycode)
                     klog("=%i (level=%i, shift=%s, mode=%i, keysyms=%s)", keycode, level, shift, mode, keysyms)
-                    if self.xkbmap_raw:
-                        break
                     level0 = levels[0]
-                    if len(keysyms)>level0 and keysyms[level0]=="":
+                    uq_keysyms = set(keysyms)
+                    if len(uq_keysyms)<=1 or (len(keysyms)>level0 and keysyms[level0]==""):
                         #if the keysym we would match for this keycode is 'NoSymbol',
                         #then we can probably ignore it ('NoSymbol' shows up as "")
+                        #same if there's only one actual keysym for this keycode
                         kmlog("not toggling any modifiers state for keysyms=%s", keysyms)
                         break
                     def toggle_modifier(mod):
@@ -539,6 +543,15 @@ class KeyboardConfig(KeyboardConfigBase):
             if keycode is None:
                 keycode = self.keycode_translation.get(keyname, -1)
                 klog("=%i, %i (keyname translation)", keycode, rgroup)
+            if keycode is None:
+                #last resort, find using the keyval:
+                display = Gdk.Display.get_default()
+                keymap = Gdk.Keymap.get_for_display(display)
+                b, entries = keymap.get_entries_for_keyval(keyval)
+                if b and entries:
+                    for entry in entries:
+                        if entry.group==group:
+                            return entry.keycode, entry.group
         return keycode, rgroup
 
 
@@ -600,11 +613,14 @@ class KeyboardConfig(KeyboardConfigBase):
             failed = []
             for modifier in modifiers:
                 modifier = bytestostr(modifier)
-                keynames = self.keynames_for_mod.get(modifier)
-                if not keynames:
+                keynames = self.keynames_for_mod.get(modifier, None)
+                if keynames is None:
                     log.error("Error: unknown modifier '%s'", modifier)
                     log.error(" known modifiers: %s", csv(self.keynames_for_mod.keys()))
                     continue
+                if not keynames:
+                    log.warn("Warning: found no keynames for '%s'", modifier)
+                    continue
                 #find the keycodes that match the keynames for this modifier
                 keycodes = []
                 #log.info("keynames(%s)=%s", modifier, keynames)
