@@ -4,14 +4,14 @@
 # later version. See the file COPYING for details.
 
 import os
-import re
 
 #ensure that we use gtk as display source:
 from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
-from xpra.util import std, csv
+from xpra.util import std, csv, envbool
 from xpra.os_util import bytestostr
 from xpra.gtk_common.error import xsync, xlog
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
+from xpra.keyboard.layouts import parse_xkbmap_query
 from xpra.log import Logger
 
 init_gdk_display_source()
@@ -19,6 +19,7 @@ X11Keyboard = X11KeyboardBindings()
 
 log = Logger("x11", "keyboard")
 
+XKB = envbool("XPRA_XKB", True)
 DEBUG_KEYSYMS = [x for x in os.environ.get("XPRA_DEBUG_KEYSYMS", "").split(",") if len(x)>0]
 
 #keys we choose not to map if the free space in the keymap is too limited
@@ -58,7 +59,7 @@ def clean_keyboard_state():
 # keyboard layouts
 
 def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
-                  xkbmap_print, xkbmap_query, xkbmap_query_struct={}):
+                  xkbmap_query, xkbmap_query_struct):
     """ xkbmap_layout is the generic layout name (used on non posix platforms)
         xkbmap_variant is the layout variant (may not be set)
         xkbmap_print is the output of "setxkbmap -print" on the client
@@ -70,18 +71,16 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
     #First we try to use data from setxkbmap -query,
     #preferably as structured data:
     if xkbmap_query and not xkbmap_query_struct:
-        from xpra.keyboard.layouts import parse_xkbmap_query
         xkbmap_query_struct = parse_xkbmap_query(xkbmap_query)
     if xkbmap_query_struct:
         log("do_set_keymap using xkbmap_query struct=%s", xkbmap_query_struct)
-        """ The xkbmap_query_struct data will look something like this:
-            {
-            b"rules"       : b"evdev",
-            b"model"       : b"pc105",
-            b"layout"      : b"gb",
-            b"options"     : b"grp:shift_caps_toggle",
-            }
-        """
+        #The xkbmap_query_struct data will look something like this:
+        #    {
+        #    b"rules"       : b"evdev",
+        #    b"model"       : b"pc105",
+        #    b"layout"      : b"gb",
+        #    b"options"     : b"grp:shift_caps_toggle",
+        #    }
         #parse the data into a dict:
         rules = xkbmap_query_struct.get(b"rules")
         model = xkbmap_query_struct.get(b"model")
@@ -95,21 +94,6 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
             safe_setxkbmap(rules, model, layout, variant, options)
         else:
             safe_setxkbmap(rules, model, "", "", "")
-    if xkbmap_print:
-        #TODO: this is no longer used with any client, remove it
-        log("do_set_keymap using xkbmap_print")
-        #try to guess the layout by parsing "setxkbmap -print"
-        try:
-            sym_re = re.compile(r"\s*xkb_symbols\s*{\s*include\s*\"([\w\+]*)")
-            for line in xkbmap_print.splitlines():
-                m = sym_re.match(line)
-                if m:
-                    layout = std(m.group(1))
-                    log.info("guessing keyboard layout='%s'" % layout)
-                    safe_setxkbmap("evdev", "pc105", layout, "", xkbmap_options)
-                    return
-        except Exception as e:
-            log.info("error setting keymap: %s" % e)
     #fallback for non X11 clients:
     layout = xkbmap_layout or "us"
     log.info("setting keyboard layout to '%s'", std(layout))
@@ -152,6 +136,11 @@ def apply_xmodmap(instructions):
     return unset
 
 
+def get_keycode_mappings():
+    if XKB and X11Keyboard.hasXkb():
+        return X11Keyboard.get_xkb_keycode_mappings()
+    return X11Keyboard.get_keycode_mappings()
+
 def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
     """
         Translate the given keycodes into the existing keymap
@@ -167,7 +156,7 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
     #keycodes = {
     #     9: set([('', 1), ('Escape', 4), ('', 3), ('Escape', 0), ('Escape', 2)]),
     #     10: set([('onesuperior', 4), ('onesuperior', 8), ('exclam', 1), ('1', 6), ('exclam', 3), ('1', 2), ('exclamdown', 9), ('exclamdown', 5), ('1', 0), ('exclam', 7)]),
-    x11_keycodes = X11Keyboard.get_keycode_mappings()
+    x11_keycodes = get_keycode_mappings()
     log(" x11_keycodes=%s", x11_keycodes)
     #x11_keycodes = {
     #    8: ['Mode_switch', '', 'Mode_switch', '', 'Mode_switch'],
@@ -344,7 +333,10 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     for try_harder in (False, True):
         filtered_preserve_keycode_entries = filter_mappings(indexed_mappings(preserve_keycode_entries), try_harder)
         log("filtered_preserve_keycode_entries=%s", filtered_preserve_keycode_entries)
-        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes, filtered_preserve_keycode_entries, keysym_to_modifier, try_harder)
+        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes,
+                                                                   filtered_preserve_keycode_entries,
+                                                                   keysym_to_modifier,
+                                                                   try_harder)
         if not missing_keycodes:
             break
     instructions = keymap_to_xmodmap(new_keycodes)
@@ -417,7 +409,7 @@ def x11_keycodes_to_list(x11_mappings):
     return entries
 
 
-def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keysym_to_modifier={}, try_harder=False):
+def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries, keysym_to_modifier, try_harder=False):
     """
         The keycodes given may not match the range that the server supports,
         or some of those keycodes may not be usable (only one modifier can
